/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */
 
#include "init.h"
#include "sensitivity.h"
#include "main_window_callbacks.h"
#include "menu_callbacks.h"
#include "toolbar_callbacks.h"
#include "welpage_callbacks.h"
#include "../otree/otree_callbacks.h"
#include "../otree/popup_menu_callbacks.h"
#include "../GUI_wizards/db_profile_wizard.h"
#include "../GUI_wizards/db_wizard.h"
#include "../GUI_wizards/schema_wizard.h"
#include "../GUI_wizards/table_wizard.h"
#include "../GUI_wizards/trigger_wizard.h"
#include "../GUI_wizards/rule_wizard.h"
#include "../tools/db_profile_manager.h"
#include "../tools/sql_editor.h"

/*
 * Runs graphical user interface
 */
void
run_GUI (int argc, char **argv)
{
	gtk_init(&argc, &argv);
	
	// init globals
	notebook.page = NULL;
	notebook.n_pages = 0;
	
	// parse glade file
	xml = NULL;
	xml = glade_xml_new(GLADE_DIRECTORY "/pguiman.glade", NULL, NULL);
	if(xml == NULL) error("Error: libglade", "Could not open glade file.");
	
	// get main window widget
	glade_xml_get_widget(xml,"main_window");
	
	// set some default colors
	GdkColor color;
  color.red = 204*257;
  color.green = 0;
  color.blue = 0;
  
  color.red = 152*257;
  color.green = 15*257;
  color.blue = 15*257;
 	gtk_widget_modify_bg (glade_xml_get_widget(xml,"eventbox2"), GTK_STATE_NORMAL, &color);
	
	color.red = 255*257;
  color.green = 255*257;
  color.blue = 255*257;
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"welcome_page"), GTK_STATE_NORMAL, &color);
	gtk_notebook_set_menu_label_text(GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook")),glade_xml_get_widget(xml,"welcome_page"),"Welcome page");
	gtk_widget_modify_bg (glade_xml_get_widget(xml, "password_dialog_bg"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml, "filter_data_dialog_bg"), GTK_STATE_NORMAL, &color);
	
	color.red = 132*257;
  color.green = 5*257;
  color.blue = 5*257;
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "password_dialog_hr"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "filter_data_dialog_hr"), GTK_STATE_NORMAL, &color);
  
	color.red = 128*257;
  color.green = 128*257;
  color.blue = 128*257;  
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"info_database_bg"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"ssd_SQL_bg"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"ssd_messages_bg"), GTK_STATE_NORMAL, &color);

	// init o_tree
	o_tree.server = NULL;
	o_tree.n_servers = 0;
	    
	// connect signal handlers
	connect_main_window_signals(xml);
	connect_object_tree_signals(xml);
	connect_menu_signals(xml);
	connect_toolbar_signals(xml);
	connect_welpage_signals(xml);
	connect_cdbp_wiz_signals(xml);
	connect_cdb_wiz_signals(xml);
	connect_csw_signals(xml);
	connect_ctw_signals(xml);
	connect_ctrw_signals(xml);
	connect_crw_signals(xml);
	connect_popup_menu_signals(xml);
	connect_dbpm_signals(xml);
	connect_sqle_signals(xml);
			
	// init wizard
	init_c_db_profile_wizard();
	init_c_db_wizard();
	
	// init tools
	init_db_profile_manager();
	init_sql_editor();
	
	// no database connected
	some_db_connected = FALSE;
	
	// create object treeview		
	create_object_tree_view(glade_xml_get_widget(xml, "object_tree"));
	
	// show tree lines in treeview
	gtk_tree_view_set_enable_tree_lines(GTK_TREE_VIEW(glade_xml_get_widget(xml,"object_tree")),TRUE);
	
	// set sensitivity
	basic_sensitivity();
	
	// run
	gtk_main();
	
	// free xml
	if(xml != NULL) g_object_unref (G_OBJECT (xml));
	
} // run_GUI ()

/*
 * Creates object tree view
 */
void
create_object_tree_view (GtkWidget * view)
{
    GtkTreeModel      *model;
    GtkTreeViewColumn *col;
    GtkCellRenderer   *renderer;
 
    model = GTK_TREE_MODEL(gtk_tree_store_new(NUM_COLS, GDK_TYPE_PIXBUF, G_TYPE_STRING));
    
    gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
		
    col = gtk_tree_view_column_new();
    
    renderer = gtk_cell_renderer_pixbuf_new();
    gtk_tree_view_column_pack_start(col, renderer, FALSE);
    gtk_tree_view_column_set_attributes(col, renderer,
                                        "pixbuf", COL_ICON,
                                        NULL);
                                        
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer,
                                        "markup", COL_TEXT,
                                        NULL);
                                            
    gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
    
    gtk_widget_show_all(view);
} // create_object_tree_view ()
