/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */
 
#include "main_window.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Managing notebook pages list                                                                                      */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

gboolean
append_page_to_list(T_Notebook * nb, T_Page * page, T_Page ** existing_page){
	//check if not in list
	for(int i=0; i < nb->n_pages; i++){
		//if(strcmp(nb->page[i]->path, page->path)==0){ // found -> free page struct and return false
		if(nb->page[i]->object == page->object){ // found -> free page struct and return false
			gtk_widget_destroy(page->page);
			free(page->path);
			free(page);
			*existing_page = nb->page[i];
			
			return FALSE;
		}
	}
	
	// if not found resize pages array
	if(nb->page == NULL){
		if((nb->page = (T_Page **) malloc(sizeof(T_Page*))) == NULL){
	 		 fprintf(stderr, "Memory allocation error.");
			 exit(0);
		}
	}
	else{
		if((nb->page = (T_Page **) realloc((nb->page), sizeof(T_Page*)*(nb->n_pages+1))) == NULL){
	   	fprintf(stderr, "Memory allocation error.");
	 		exit(0);
		}
	}
	
	nb->page[nb->n_pages] = page;
	nb->n_pages++;
	
	return TRUE;
}

T_Page *
find_widget_in_page_list(T_Notebook * nb, GtkWidget * widget){
	
	for(int i=0; i < nb->n_pages; i++){
		if(nb->page[i]->page == widget){ // found
			return nb->page[i];
		}
	}
			
	return NULL;
}

gboolean
remove_page_from_list(T_Notebook * nb, T_Page * page){
	for(int i=0; i < nb->n_pages; i++){
		if(nb->page[i]->page == page->page && strcmp(nb->page[i]->path, page->path) == 0){ // found
			// remove page
			gtk_widget_destroy(page->page);
			free(page->path);
			free(page);
			
			for(int k=i+1; k< nb->n_pages; k++){ // move array
				nb->page[k-1] = nb->page[k];
			}
			nb->n_pages--;
			if(nb->n_pages == 0){
				free(nb->page);
				nb->page = NULL;
			}
			
			return TRUE;
		}
	}
	
	return FALSE; // page not in list
}

gboolean
close_pages_of_profile(T_Notebook * nb, T_Profile * profile){
	gboolean found = FALSE;
	gint index;
	do{
		found = FALSE;
		index = 0;
		for(int i=0; i < nb->n_pages; i++){
			GtkTreePath * p = gtk_tree_path_new_from_string(nb->page[i]->path);
			gint depth = gtk_tree_path_get_depth(p);
			gint * intpath = gtk_tree_path_get_indices(p);
			if(depth >=3 ){
				if(o_tree.server[intpath[0]]->profile[intpath[2]] == profile){
					found = TRUE;
					index = i;
					break;				
				}
			}
		}
		if(found){
			if(!(*nb->page[index]->close_function)(NULL, nb->page[index]->page)) return FALSE;
		}
	}while(found);
	
	return TRUE;
}

// main_window.c
