/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "menu_callbacks.h"
#include "GUI_functions.h"
#include "../database/db_connection.h"
#include "../database/create_db_objects.h"
#include "../database/drop_db_objects.h"
#include "../GUI_wizards/db_profile_wizard.h"
#include "../GUI_wizards/db_wizard.h"
#include "../otree/show_otree.h"
#include "../tools/db_profile_manager.h"
#include "../tools/sql_editor.h"

/*
 * Connects menu callbacks
 */
void
connect_menu_signals (GladeXML * source)
{
	GtkWidget * widget;

	/* Menu Database */	
	// Create database profile
	widget = glade_xml_get_widget(source, "mb_create_db_profile");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_create_db_profile_activate),    NULL);

	// Edit database profile
	widget = glade_xml_get_widget(source, "mb_edit_db_profile");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_edit_db_profile_activate),    NULL);
	// Remove database profile
	widget = glade_xml_get_widget(source, "mb_remove_db_profile");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_remove_db_profile_activate),    NULL);


	// Connect to database
	widget = glade_xml_get_widget(source, "mb_connect_to_db");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_connect_to_db_activate),    NULL);
	
	// Disconnect from database
	widget = glade_xml_get_widget(source, "mb_disconnect_from_db");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_disconnect_from_db_activate),    NULL);


	// Create new database
	widget = glade_xml_get_widget(source, "mb_create_new_db");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_create_new_db_activate),    NULL);

	// Drop database
	widget = glade_xml_get_widget(source, "mb_drop_db");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_drop_db_activate),    NULL);


	// Exit
	widget = glade_xml_get_widget(source, "mb_exit");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_exit_activate),    NULL);

	/* Menu Tools */
	// Database profile manager
	widget = glade_xml_get_widget(source, "mb_db_profile_man");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_db_profile_man_activate),    NULL);
	
	// SQL Editor
	widget = glade_xml_get_widget(source, "mb_sql_editor");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_sql_editor_activate),    NULL);

	/* Menu View */
	// Refresh
	widget = glade_xml_get_widget(source, "mb_refresh");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_refresh_activate),    NULL);
	
	// Show tool panel
	widget = glade_xml_get_widget(source, "mb_show_tool_panel");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_show_tool_panel_activate),    NULL);
	// Show SQL code
	widget = glade_xml_get_widget(source, "mb_show_wel_page");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_show_wel_page_activate),    NULL);
	// Show status bar
	widget = glade_xml_get_widget(source, "mb_show_status_bar");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_show_status_bar_activate),    NULL);

	/* Menu Help */
	// Help
	widget = glade_xml_get_widget(source, "mb_help");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_help_activate),    NULL);
	// About
	widget = glade_xml_get_widget(source, "mb_about");
	g_signal_connect ((gpointer) widget, "activate",    G_CALLBACK (on_mb_about_activate),    NULL);
}

/////////////////////////////////////// Menu Database //////////////////////////////////////////////////////////////////////////
void
on_mb_create_db_profile_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	run_c_db_profile_wizard();
}

void
on_mb_edit_db_profile_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	run_db_profile_manager();
}

void
on_mb_remove_db_profile_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_remove_profile(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
	gtk_tree_view_expand_to_path (view, path); gtk_tree_selection_select_path(selection, path);
}

// Sep

void
on_mb_connect_to_db_activate (GtkMenuItem *menuitem,    gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_connect_db(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
	gtk_tree_view_expand_to_path (view, path);
}

void
on_mb_disconnect_from_db_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_disconnect_db(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
}

// Sep

void
on_mb_create_new_db_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	run_c_db_wizard();
}

void
on_mb_drop_db_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 3){
		// get object name and database profile
		T_Profile * profile = o_tree.server[intpath[0]]->profile[intpath[2]];
		
		// run dialog to confirm delete
		gchar * label = append_string(NULL, "Do you really want to drop database '");
		label = append_string(label, profile->dbname);	
		label = append_string(label, "'?");
		gchar * l = normal_text(label);
		free(label);
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"yes_no_dialog_label")), l);
		free(l);
		gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "yes_no_dialog")));
		if(res == GTK_RESPONSE_YES){
			gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));
		}
		else{
			gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));	
			return;
		}
		
		// create sql code
		gchar * sql = create_drop_database_SQL_query(profile);
		
		// execute sql
		if(sql != NULL){
			gui_disconnect_db(o_tree.server[intpath[0]] , profile);
			if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE))
				gui_remove_profile(o_tree.server[intpath[0]] , profile);
			free(sql);
		}
		
	}
}

// Sep

void
on_mb_exit_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	// run dialog to confirm delete
	gchar * l = normal_text("Do you really want to exit from Postgre SQL Manager?");
	gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"yes_no_dialog_label")), l);
	free(l);
	gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "yes_no_dialog")));
	if(res == GTK_RESPONSE_YES){
		gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));
		if(!gui_disconnect_all_dbs()) return;
		gtk_main_quit();
	}
	else{
		//gtk_widget_show(object);
		gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));	
	}
}

/////////////////////////////////////// Menu Tools  /////////////////////////////////////////////////////////////////////////////////
void
on_mb_db_profile_man_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	run_db_profile_manager();	
} 
// Sep
void
on_mb_sql_editor_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	run_sql_editor();
}

/////////////////////////////////////// Menu View //////////////////////////////////////////////////////////////////////////////////
void
on_mb_refresh_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_refresh(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
}

// Sep

void
on_mb_show_tool_panel_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget(xml, "mb_show_tool_panel")))){
		gtk_widget_show(glade_xml_get_widget(xml, "hbox1"));
	}
	else{
		gtk_widget_hide(glade_xml_get_widget(xml, "hbox1"));
	}
}

void
on_mb_show_status_bar_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget(xml, "mb_show_status_bar")))){
		gtk_widget_show(glade_xml_get_widget(xml, "statusbar"));
	}
	else{
		gtk_widget_hide(glade_xml_get_widget(xml, "statusbar"));
	}	
}

void
on_mb_show_wel_page_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget(xml, "mb_show_wel_page")))){
		gtk_widget_show(glade_xml_get_widget(xml, "welcome_page"));
	}
	else{
		gtk_widget_hide(glade_xml_get_widget(xml, "welcome_page"));
	}	
}

/////////////////////////////////////// Menu Help ///////////////////////////////////////////////////////////////////////////////////
void
on_mb_help_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "help_dialog")));	
	gtk_widget_hide(glade_xml_get_widget(xml, "help_dialog"));
}

// Sep

void
on_mb_about_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "about_dialog")));	
	gtk_widget_hide(glade_xml_get_widget(xml, "about_dialog"));
}
