/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */
 
#include "textview_functions.h"
#include "GUI_functions.h"

#define N_KEYWORDS 333

gchar *keyword[]={
 "abort",
	"absolute",
	"access",
	"action",
	"add", 
	"admin",
	"after",
	"aggregate",
	"all",
	"also",
	"alter",
	"analyse", // British spelling 
	"analyze",
	"and",
	"any",
	"array",
	"as",
	"asc",
	"assertion",
	"assignment",
	"asymmetric",
	"at",
	"authorization",
	"backward",
	"before",
	"begin",
	"between",
	"bigint",
	"binary",
	"bit",
	"boolean",
	"both",
	"by",
	"cache",
	"called", 
	"cascade",
	"case",
	"cast", 
	"chain",
	"char",
	"character",
	"characteristics",
	"check",
"checkpoint",
"class",
"close",
"cluster",
"coalesce",
"collate",
"column",
"comment",
"commit",
"committed",
"connection",
"constraint",
"constraints",
"conversion",
"convert",
"copy",
"create",
"createdb",
"createrole",
"createuser",
"cross",
"csv",
"current_date",
"current_role",
"current_time",
"current_timestamp",
"current_user",
"cursor",
"cycle",
"database",
"day",
"deallocate",
"dec",
"decimal",
"declare",
"default",
"defaults",
"deferrable",
"deferred",
"definer",
"delete",
"delimiter",
"delimiters",
"desc",
"disable",
"distinct",
"do",
"domain",
"double",
"drop",
"each",
"else",
"enable",
"encoding",
"encrypted",
"end",
"escape",
"except",
"excluding",
"exclusive",
"execute",
"exists",
"explain",
"external",
"extract",
"false",
"fetch",
"first",
"float",
"for",
"force",
"foreign",
"forward",
"freeze",
"from",
"full",
"function",
"global",
"grant",
"granted",
"greatest",
"group",
"handler",
"having",
"header",
"hold",
"hour",
"ilike",
"immediate",
"immutable",
"implicit",
"in",
"including",
"increment",
"index",
"inherit",
"inherits",
"initially",
"inner",
"inout",
"input",
"insensitive",
"insert",
"instead",
"int",
"integer",
"intersect",
"interval",
"into",
"invoker",
"is",
"isnull",
"isolation",
"join",
"key",
"lancompiler",
"language",
"large",
"last",
"leading",
"least",
"left",
"level",
"like",
"limit",
"listen",
"load",
"local",
"localtime",
"localtimestamp",
"location",
"lock",
"login",
"match",
"maxvalue",
"minute",
"minvalue",
"mode",
"month",
"move",
"names",
"national",
"natural",
"nchar",
"new",
"next",
"no",
"nocreatedb",
"nocreaterole",
"nocreateuser",
"noinherit",
"nologin",
"none",
"nosuperuser",
"not",
"nothing",
"notify",
"notnull",
"nowait",
"null",
"nullif",
"numeric",
"object",
"of",
"off",
"offset",
"oids",
"old",
"on",
"only",
"operator",
"option",
"or",
"order",
"out",
"outer",
"overlaps",
"overlay",
"owner",
"partial",
"password",
"placing",
"position",
"precision",
"prepare",
"prepared",
"preserve",
"primary",
"prior",
"privileges",
"procedural",
"procedure",
"quote",
"read",
"real",
"recheck",
"references",
"reindex",
"relative",
"release",
"rename",
"repeatable",
"replace",
"reset",
"restart",
"restrict",
"returns",
"revoke",
"right",
"role",
"rollback",
"row",
"rows",
"rule",
"savepoint",
"schema",
"scroll",
"second",
"security",
"select",
"sequence",
"serializable",
"session",
"session_user",
"set",
"setof",
"share",
"show",
"similar",
"simple",
"smallint",
"some",
"stable",
"start",
"statement",
"statistics",
"stdin",
"stdout",
"storage",
"strict",
"substring",
"superuser",
"symmetric",
"sysid",
"system",
"table",
"tablespace",
"temp",
"template",
"temporary",
"then",
"time",
"timestamp",
"to",
"toast",
"trailing",
"transaction",
"treat",
"trigger",
"trim",
"true",
"truncate",
"trusted",
"type",
"uncommitted",
"unencrypted",
"union",
"unique",
"unknown",
"unlisten",
"until",
"update",
"user",
"using",
"vacuum",
"valid",
"validator",
"values",
"varchar",
"varying",
"verbose",
"view",
"volatile",
"when",
"where",
"with",
"without",
"work",
"write",
"year",
"zone"
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Init sql text views                                                                                                          */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void
init_text_view(GtkWidget * textview, gboolean editable, const gchar * text){
	GtkTextBuffer * buffer;
	
	gtk_text_view_set_editable(GTK_TEXT_VIEW (textview), editable);
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW (textview), editable);
	
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
  
 // clear all tags
 GtkTextIter start, end;
		
	gtk_text_buffer_get_start_iter(buffer, &start);
	gtk_text_buffer_get_end_iter(buffer, &end);
	gtk_text_buffer_remove_all_tags(buffer,&start, &end);
	
  GtkTextTagTable * table =  gtk_text_buffer_get_tag_table(buffer); 
  GtkTextTag  * tag = gtk_text_tag_table_lookup(table, "keyword");
  if(tag != NULL) gtk_text_tag_table_remove (table, tag);
 tag = gtk_text_tag_table_lookup(table, "comment");
  if(tag != NULL) gtk_text_tag_table_remove (table, tag);
  tag = gtk_text_tag_table_lookup(table, "normal");
  if(tag != NULL) gtk_text_tag_table_remove (table, tag);
  
 gtk_text_buffer_create_tag (buffer, "keyword", 
                              "font", "courier bold 11", 
                              "foreground", "#980F0F",
                              NULL);

  gtk_text_buffer_create_tag (buffer, "comment", 
                              "font", "courier italic 11", 
                              "foreground", "#909090",
                              NULL);
  
  gtk_text_buffer_create_tag (buffer, "normal", 
                              "font", "courier 11", 
                              "foreground", "#000000",
                              NULL);
                              
	g_signal_connect (G_OBJECT (buffer), "changed", 
                    G_CALLBACK (buffer_insert_text), textview);

	GtkTextIter iter;
  gtk_text_buffer_get_start_iter(buffer, &iter);
  gtk_text_buffer_insert (buffer, &iter, text, -1);                   
}

void
buffer_insert_text (GtkTextBuffer *textbuffer, GtkTextIter *arg1,
                    gchar *arg2, gint arg3, gpointer user_data)
{
	GtkTextIter match_start;
	GtkTextIter match_end;
	GtkTextIter iter, start, end;
		
	gtk_text_buffer_get_start_iter(textbuffer, &start);
	gtk_text_buffer_get_end_iter(textbuffer, &end);
	gtk_text_buffer_remove_all_tags(textbuffer,&start, &end);
	gtk_text_buffer_apply_tag_by_name (textbuffer, "normal", &start, &end);
	
	
	
	match_end = start;
	while( !gtk_text_iter_is_end(&match_end) ){
		gtk_text_iter_forward_word_end(&match_end);
	  match_start = match_end;
	  gtk_text_iter_backward_word_start(&match_start);
	  
	  gchar  * word = gtk_text_iter_get_text(&match_start, &match_end);
	  for(int i=0; i <strlen(word); i++){
	  	word[i] = tolower(word[i]);
	  }
	  for(int i=0; i < N_KEYWORDS; i++){
	  	if(strcmp(word, keyword[i]) == 0){
	  		// check chars after and before string
	  		GtkTextIter pre, post;
				pre = match_start;
				post = match_end;
				gtk_text_iter_backward_char(&pre);		
		
				gunichar c1 = gtk_text_iter_get_char(&pre);   //1#$_
				gunichar c2 = gtk_text_iter_get_char(&post); //  '@_"
	  		
	  		if( c1 != '@' && c1 != '_' &&
	  				(c2 < '0' || c2 > '9') && c2 != '#' && c2 != '$' && c2 != '_' ){
			  		gtk_text_buffer_remove_all_tags(textbuffer,&match_start, &match_end);
						gtk_text_buffer_apply_tag_by_name (textbuffer, "keyword", &match_start, &match_end);
				}
				break;
	  	}
	  }
	}

	// find apostrophe
	gboolean found;
	iter = start;
	while(gtk_text_iter_forward_search( &iter, "\'", 0, &match_start, &match_end, NULL )){
		match_end = match_start;
		found = gtk_text_iter_forward_find_char (&match_end, (GtkTextCharPredicate) is_ap1, NULL, NULL);
		if(!found) gtk_text_buffer_get_end_iter(textbuffer, &match_end);
		gtk_text_buffer_remove_all_tags(textbuffer,&match_start, &match_end);
		gtk_text_buffer_apply_tag_by_name (textbuffer, "normal", &match_start, &match_end);
		
		if(found) gtk_text_iter_forward_char(&match_end);
		iter = match_end;
	}
	
	// find double apostrophe
	iter = start;
	while(gtk_text_iter_forward_search( &iter, "\"", 0, &match_start, &match_end, NULL )){
		match_end = match_start;
		found = gtk_text_iter_forward_find_char (&match_end, (GtkTextCharPredicate) is_ap2, NULL, NULL);
		if(!found) gtk_text_buffer_get_end_iter(textbuffer, &match_end);		 
		gtk_text_buffer_remove_all_tags(textbuffer,&match_start, &match_end);
		gtk_text_buffer_apply_tag_by_name (textbuffer, "normal", &match_start, &match_end);
		
		if(found) gtk_text_iter_forward_char(&match_end);
		iter = match_end;
	}
		
	// find comments
	iter = start;
	while(gtk_text_iter_forward_search( &iter, "--", 0, &match_start, &match_end, NULL )){
		match_end = match_start;
		gtk_text_iter_forward_to_line_end (&match_end);
		gtk_text_buffer_remove_all_tags(textbuffer,&match_start, &match_end);
		gtk_text_buffer_apply_tag_by_name (textbuffer, "comment", &match_start, &match_end);
		iter = match_end;
	}
	
	iter = start;
	while(gtk_text_iter_forward_search( &iter, "/*", 0, &match_start, &match_end, NULL )){
		match_end = match_start;
		GtkTextIter s;
		if(!gtk_text_iter_forward_search( &match_start, "*/", 0, &s, &match_end, NULL )) match_end = end;
		
		gtk_text_buffer_remove_all_tags(textbuffer,&match_start, &match_end);
		gtk_text_buffer_apply_tag_by_name (textbuffer, "comment", &match_start, &match_end);
		iter = match_end;
	}
	
}

gboolean 
is_ap1 (gunichar ch, gpointer data)
{
  if (ch == '\'')
    return TRUE;
  else
    return FALSE;
}

gboolean 
is_ap2 (gunichar ch, gpointer data)
{
  if (ch == '"')
    return TRUE;
  else
    return FALSE;
}

void
init_message_view(GtkWidget * textview, gboolean editable, const gchar * text){
	GtkTextBuffer * buffer;
	
	gtk_text_view_set_editable(GTK_TEXT_VIEW (textview), editable);
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW (textview), editable);
	
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
  
  // clear all tags
 GtkTextIter start, end;
		
	gtk_text_buffer_get_start_iter(buffer, &start);
	gtk_text_buffer_get_end_iter(buffer, &end);
	gtk_text_buffer_remove_all_tags(buffer,&start, &end);
	
  GtkTextTagTable * table =  gtk_text_buffer_get_tag_table(buffer); 
  GtkTextTag  * tag = gtk_text_tag_table_lookup(table, "error");
  if(tag != NULL) gtk_text_tag_table_remove (table, tag);
 tag = gtk_text_tag_table_lookup(table, "information");
  if(tag != NULL) gtk_text_tag_table_remove (table, tag);
  tag = gtk_text_tag_table_lookup(table, "warning");
  if(tag != NULL) gtk_text_tag_table_remove (table, tag);
   tag = gtk_text_tag_table_lookup(table, "normal");
  if(tag != NULL) gtk_text_tag_table_remove (table, tag);
  
 	gtk_text_buffer_create_tag (buffer, "error", 
                              "font", "courier bold 10", 
                              "foreground", "#980F0F",
                              NULL);
	
	gtk_text_buffer_create_tag (buffer, "information", 
                              "font", "courier bold 10", 
                              "foreground", "#0096FF",
                              NULL);

  gtk_text_buffer_create_tag (buffer, "warning", 
                              "font", "courier bold 10", 
                              "foreground", "#FF9000",
                              NULL);
                                                          
  gtk_text_buffer_create_tag (buffer, "normal", 
                              "font", "courier 10", 
                              "foreground", "#000000",
                              NULL);
                              
	g_signal_connect (G_OBJECT (buffer), "changed", 
                    G_CALLBACK (mess_buffer_insert_text), textview);

	GtkTextIter iter;
  gtk_text_buffer_get_start_iter(buffer, &iter);
  gtk_text_buffer_insert (buffer, &iter, text, -1);                   
}

void
mess_buffer_insert_text (GtkTextBuffer *textbuffer, GtkTextIter *arg1,
                    gchar *arg2, gint arg3, gpointer user_data)
{
	GtkTextIter match_start;
	GtkTextIter match_end;
	GtkTextIter start, end;
		
	gtk_text_buffer_get_start_iter(textbuffer, &start);
	gtk_text_buffer_get_end_iter(textbuffer, &end);
	gtk_text_buffer_remove_all_tags(textbuffer,&start, &end);
	gtk_text_buffer_apply_tag_by_name (textbuffer, "normal", &start, &end);
	
	
	
	match_end = start;
	while( !gtk_text_iter_is_end(&match_end) ){
		gtk_text_iter_forward_word_end(&match_end);
	  match_start = match_end;
	  gtk_text_iter_backward_word_start(&match_start);
	  
	  gchar  * word = gtk_text_iter_get_text(&match_start, &match_end);
	  if(strcmp(word, "ERROR") == 0){
	  	gtk_text_iter_forward_char(&match_end);
	  	gtk_text_buffer_remove_all_tags(textbuffer,&match_start, &match_end);
			gtk_text_buffer_apply_tag_by_name (textbuffer, "error", &match_start, &match_end);
		}
		else if(strcmp(word, "Information") == 0){
	  	gtk_text_iter_forward_char(&match_end);
	  	gtk_text_buffer_remove_all_tags(textbuffer,&match_start, &match_end);
			gtk_text_buffer_apply_tag_by_name (textbuffer, "information", &match_start, &match_end);
		}
		else if(strcmp(word, "Warning") == 0){
	  	gtk_text_iter_forward_char(&match_end);
	  	gtk_text_buffer_remove_all_tags(textbuffer,&match_start, &match_end);
			gtk_text_buffer_apply_tag_by_name (textbuffer, "warning", &match_start, &match_end);
		}
	}
}

// textview_functions.c
