/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "db_wizard.h"
#include "../functions.h"
#include "db_profile_wizard.h"
#include "../database/db_connection.h"
#include "../database/create_db_objects.h"
#include "../database/get_db_objects.h"
#include "../tools/db_profile_manager.h"
#include "../otree/show_otree.h"
#include "../GUI/GUI_functions.h"
#include "../GUI/properties_widget.h"

// Structure for new database
T_Server * cdbw_server;

#define N_ENCODINGS 36

gchar *encoding[] ={
"",
"SQL_ASCII",
"EUC_JP",
"EUC_CN",
"EUC_KR",
"EUC_TW",
"JOHAB",
"UTF8",
"MULE_INTERNAL",
"LATIN1",
"LATIN2",
"LATIN3",
"LATIN4",
"LATIN5",
"LATIN6",
"LATIN7",
"LATIN8",
"LATIN9",
"LATIN10",
"WIN1256",
"WIN1258",
"WIN866",
"WIN874",
"KOI8",
"WIN1251",
"WIN1252",
"ISO_8859_5",
"ISO_8859_6",
"ISO_8859_7",
"ISO_8859_8",
"WIN1250",
"SJIS",
"BIG5",
"GBK",
"UHC",
"GB18030"
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*						Create database profile wizard                                                                          */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void init_c_db_wizard(void){
	// Color settings
	GdkColor color;
	color.red = 255*257;
  color.green = 255*257;
  color.blue = 255*257;  
  gtk_widget_modify_bg (glade_xml_get_widget(xml,"cdbw1_header_bg"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml,"cdbw2_header_bg"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml,"cdbw3_header_bg"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"cdbw3_properties_bg2"), GTK_STATE_NORMAL, &color);
	
	color.red = 204*257;
  color.green = 0*257;
  color.blue = 0*257;  
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"cdbw1_hr_bg"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"cdbw2_hr_bg"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"cdbw3_hr_bg"), GTK_STATE_NORMAL, &color);

	color.red = 128*257;
  color.green = 128*257;
  color.blue = 128*257;  
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"cdbw3_properties_label_bg"), GTK_STATE_NORMAL, &color);
	
	cdbw_server = NULL;
}

void
run_c_db_wizard(void){

	GtkWidget * widget = NULL;
	gint page = 1;
	gboolean not_end = TRUE;
	
	// Clear database name
	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cdbw1_bt_next"), FALSE);
	gtk_entry_set_text(GTK_ENTRY( glade_xml_get_widget(xml,"cdbw1_database_name")), "");
	gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(xml,"cdbw2_SSL")), 2);
	
	// Clear connection textboxes
	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cdbw2_bt_next"), FALSE);
	gtk_entry_set_text(GTK_ENTRY( glade_xml_get_widget(xml,"cdbw2_hostname")), "");
	gtk_entry_set_text(GTK_ENTRY( glade_xml_get_widget(xml,"cdbw2_username")), "");
	gtk_entry_set_text(GTK_ENTRY( glade_xml_get_widget(xml,"cdbw2_password")), "");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(xml,"cdbw2_port")), (gdouble)5432);
	gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(xml,"cdbw2_SSL")), 2);	

 	not_end = TRUE;
	while(not_end){
		switch(page){
			case 1:	widget = glade_xml_get_widget(xml,"c_database_wiz1"); break;
			case 2: widget = glade_xml_get_widget(xml,"c_database_wiz2"); break;
			case 3: widget = glade_xml_get_widget(xml,"c_database_wiz3"); break;
			default:;
		};
		gint res = gtk_dialog_run(GTK_DIALOG(widget));
		
		if(res == GTK_RESPONSE_YES){
			if(page < 3){
				if(page == 1){ // get new db name
					page++;
					gtk_widget_hide(widget);
				}
				else if(page == 2){ // set up connection
									//try to connect
									if(cdbw_connect_db(
																			(char *)gtk_entry_get_text( GTK_ENTRY( glade_xml_get_widget(xml,"cdbw2_hostname") ) ),
																			itoa_n(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(glade_xml_get_widget(xml,"cdbw2_port"))),0),
																			(char *)gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"cdbw2_username"))),
																			(char *)gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"cdbw2_password"))),
																			itoa_n(gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(xml,"cdbw2_SSL"))),0)																			
										)){
												page++;
  				 							gtk_widget_hide(widget);
/////////////////////////////// show_properties ///////////////////////////////////////////////////////////////////////////////
	// clear properties
	GtkWidget * wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_database_wiz3")), "properties_column1");
	if(wid != NULL) gtk_widget_destroy(wid);
	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_database_wiz3")), "properties_column2");
	if(wid != NULL) gtk_widget_destroy(wid);
	
	// add properties
	GtkWidget * dialog = glade_xml_get_widget(xml, "c_database_wiz3");
		
	// add properties
	GtkWidget * properties_columns = glade_xml_get_widget(xml, "cdbw3_properties");
  GtkWidget * properties_column1 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column1, "properties_column1");
  gtk_widget_show (properties_column1);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column1, FALSE, TRUE, 0);
 
  GtkWidget * properties_column2 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column2, "properties_column2");
  gtk_widget_show (properties_column2);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column2, TRUE, TRUE, 0);
	
	gint height = 0;
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Common",  PIXMAP_DIRECTORY "/Common.png");
	height += 21;
	T_Roles * roles = get_roles(cdbw_server->profile[0], TRUE);
	if(roles != NULL){
	gint role_id=-1;
	add_combo_property("property_owner", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Owner", roles->name,  roles->n_roles, role_id);
	for(int i = 0; i < roles->n_roles; i++){
		free(roles->name[i]);
	}
	free(roles->name);
	free(roles);
	}
	height += 21;
	add_comment_entry_property("property_comment", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Comment", "");
	height += 41;
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Database",  PIXMAP_DIRECTORY "/Database.png");
	height += 21;
	add_combo_property("property_encoding", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Encoding", encoding,  N_ENCODINGS, -1);
	height += 21;

	gint rows = 0;
	gchar ** list = get_database_list(cdbw_server->profile[0]->conn, FALSE, TRUE, &rows);
	add_combo_property("property_template", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Template", list,  rows, -1);
	height += 21;
	for(int i = 0; i < rows; i++)	free(list[i]);
	free(list);
	
	T_Tablespaces * tspces = get_Tablespaces(cdbw_server->profile[0]);
	if(tspces != NULL){
		gint tspc_id=-1;
		tspces->name = (gchar ** )realloc(tspces->name, sizeof(gchar *)*(tspces->n_tablespaces+1));
		if(tspces->name == NULL){
			fprintf(stderr, "Memory allocation error.");
	 		exit(0);
		}
		tspces->name[tspces->n_tablespaces] = copy_string("");
		tspces->n_tablespaces++;
		add_combo_property("property_tablespace", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Tablespace", tspces->name,  tspces->n_tablespaces, tspc_id);
		for(int i = 0; i < tspces->n_tablespaces; i++){
			free(tspces->name[i]);
		}
		free(tspces->name);
		free(tspces);
	}
	height += 21;
	gtk_widget_set_size_request (glade_xml_get_widget(xml, "cdbw3_scrolledwindow"), -1, height);
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////  				 							
  				 						}
				}
				else{	// get properties
					 page++;
  				 gtk_widget_hide(widget);
				}
			}
			else{ // end the wizard and optionally run database profile manager
					not_end = FALSE;
					// get new database name
					cdbw_server->profile[0]->dbname = copy_string(gtk_entry_get_text( GTK_ENTRY( glade_xml_get_widget(xml,"cdbw1_database_name") ) ));
					
					// get properties
					cdbw_get_properties(cdbw_server);
										
					/* create database */
					// create sql from T_Profile struct
					gchar * sql = create_new_database_SQL_query(cdbw_server->profile[0]);
					// execute sql
					if(!execute_sql(sql, NULL, NULL, cdbw_server->profile[0], TRUE, TRUE)){
						not_end = TRUE;
						free(sql);
					}
					else{
					  gtk_widget_hide(widget);
						free(sql);
					
						// run dbprofile manager
						if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml,"cdbw1_create_profile")))){
							gchar * alias = append_string(NULL, cdbw_server->profile[0]->dbname);
							alias = append_string(alias, "@");
							alias = append_string(alias, cdbw_server->host);
							create_db_profile(	
																copy_string(cdbw_server->host),	// server properties
																cdbw_server->port, 
																copy_string(cdbw_server->uname),
																copy_string(cdbw_server->passwd),
																cdbw_server->SSL,
																copy_string(cdbw_server->profile[0]->dbname),	// profile properties
																alias
															);
							show_object_tree();
							run_db_profile_manager();
						}
					}
			}
		}
		else if(res == GTK_RESPONSE_NO){
			if(page > 1){
				if(page == 3){
					if(cdbw_server != NULL){ // clear server struct
						if(cdbw_server->profile != NULL){ // clear database struct
		  				if(cdbw_server->profile[0] != NULL){ // clear database struct	
		  					if(cdbw_server->profile[0]->dbname != NULL) free(cdbw_server->profile[0]->dbname);
		  					if(cdbw_server->profile[0]->database.owner != NULL) free(cdbw_server->profile[0]->database.owner);
								if(cdbw_server->profile[0]->database.comment != NULL) free(cdbw_server->profile[0]->database.comment);
								if(cdbw_server->profile[0]->database.encoding != NULL) free(cdbw_server->profile[0]->database.encoding);
								if(cdbw_server->profile[0]->database.template != NULL) free(cdbw_server->profile[0]->database.template);
								if(cdbw_server->profile[0]->database.tablespace != NULL) free(cdbw_server->profile[0]->database.tablespace);
								disconnect_db(cdbw_server->profile[0]->conn);
								free(cdbw_server->profile[0]);
		  				}
		  				free(cdbw_server->profile);
						}
						if(cdbw_server->host != NULL) free(cdbw_server->host);
 						if(cdbw_server->uname != NULL) free(cdbw_server->uname);
 						if(cdbw_server->passwd != NULL) free(cdbw_server->passwd);
 						if(cdbw_server->name != NULL) free(cdbw_server->name);		
  					free(cdbw_server);
  					cdbw_server = NULL;
					}
				}
				page--;
				gtk_widget_hide(widget);
			}
		}
		else if(res == GTK_RESPONSE_CANCEL || res == GTK_RESPONSE_DELETE_EVENT){
				not_end = FALSE;
				
				gtk_widget_hide(widget);
		}
	}
	
	if(cdbw_server != NULL){ // clear server struct
		if(cdbw_server->profile != NULL){ // clear database struct
			if(cdbw_server->profile[0] != NULL){ // clear database struct	
				if(cdbw_server->profile[0]->dbname != NULL) free(cdbw_server->profile[0]->dbname);
				if(cdbw_server->profile[0]->database.owner != NULL) free(cdbw_server->profile[0]->database.owner);
				if(cdbw_server->profile[0]->database.comment != NULL) free(cdbw_server->profile[0]->database.comment);
				if(cdbw_server->profile[0]->database.encoding != NULL) free(cdbw_server->profile[0]->database.encoding);
				if(cdbw_server->profile[0]->database.template != NULL) free(cdbw_server->profile[0]->database.template);
				if(cdbw_server->profile[0]->database.tablespace != NULL) free(cdbw_server->profile[0]->database.tablespace);
				disconnect_db(cdbw_server->profile[0]->conn);
				free(cdbw_server->profile[0]);
			}
			free(cdbw_server->profile);
		}
		if(cdbw_server->host != NULL) free(cdbw_server->host);
 		if(cdbw_server->uname != NULL) free(cdbw_server->uname);
 		if(cdbw_server->passwd != NULL) free(cdbw_server->passwd);
 		if(cdbw_server->name != NULL) free(cdbw_server->name);		
  	free(cdbw_server);
	}
	cdbw_server = NULL;
}

gboolean
cdbw_connect_db(gchar * host, gchar * port, gchar * uname, gchar * passwd, gchar * SSL){
	PGconn *conn;
	/* Connection to database */
	char * conninfo = make_conninfo(host, port, "postgres", uname, passwd, (guint) atoi(SSL));

	// trying connection
	if((conn = connect_db(conninfo)) != NULL) ;
  else return FALSE;
  free(conninfo);
  
  // Create server struct
  if(cdbw_server != NULL){ // clear server struct
		if(cdbw_server->profile != NULL){ // clear database struct
		  if(cdbw_server->profile[0] != NULL){ // clear database struct	
		  	if(cdbw_server->profile[0]->dbname != NULL) free(cdbw_server->profile[0]->dbname);
		  	if(cdbw_server->profile[0]->database.owner != NULL) free(cdbw_server->profile[0]->database.owner);
				if(cdbw_server->profile[0]->database.comment != NULL) free(cdbw_server->profile[0]->database.comment);
				if(cdbw_server->profile[0]->database.encoding != NULL) free(cdbw_server->profile[0]->database.encoding);
				if(cdbw_server->profile[0]->database.template != NULL) free(cdbw_server->profile[0]->database.template);
				if(cdbw_server->profile[0]->database.tablespace != NULL) free(cdbw_server->profile[0]->database.tablespace);
				free(cdbw_server->profile[0]);
		  }
		  free(cdbw_server->profile);
		}
		if(cdbw_server->host != NULL) free(cdbw_server->host);
 		if(cdbw_server->uname != NULL) free(cdbw_server->uname);
 		if(cdbw_server->passwd != NULL) free(cdbw_server->passwd);
 		if(cdbw_server->name != NULL) free(cdbw_server->name); 		
  	free(cdbw_server);
	}
		
	if((cdbw_server = (T_Server*) malloc(sizeof(T_Server))) == NULL){
	  fprintf(stderr, "Memory allocation error: callbacks.c");
	  exit(0);
	}
	cdbw_server->n_profiles = 0;
	cdbw_server->profile = NULL;
  cdbw_server->host = NULL;
  cdbw_server->uname = NULL;
  cdbw_server->passwd = NULL;
  
  // Create profile struct
  if((cdbw_server->profile = (T_Profile **) malloc(sizeof(T_Profile*))) == NULL){
		 fprintf(stderr, "Memory allocation error.");
	   exit(0);
	}
		
	if((cdbw_server->profile[0] = (T_Profile*) malloc(sizeof(T_Profile))) == NULL){
		 fprintf(stderr, "Memory allocation error: callbacks.c");
		 exit(0);
	}
	cdbw_server->profile[0]->dbname = NULL;
	cdbw_server->profile[0]->database.owner =  NULL;
	cdbw_server->profile[0]->database.comment =  NULL;
	cdbw_server->profile[0]->database.encoding = NULL;
	cdbw_server->profile[0]->database.template = NULL;
	cdbw_server->profile[0]->database.tablespace = NULL;
				
	// init server properties
	cdbw_server->host = copy_string(host);
  cdbw_server->uname = copy_string(uname);
  cdbw_server->passwd = copy_string(passwd); 
  cdbw_server->port = atoi(port);
  cdbw_server->SSL = atoi(SSL);
  gint len = strlen(cdbw_server->uname) + strlen("@") + strlen(cdbw_server->host);
  cdbw_server->name = alloc_string(len + 1);
  cdbw_server->name[0] = '\0';
  strcat(cdbw_server->name, cdbw_server->uname); strcat(cdbw_server->name, "@"); strcat(cdbw_server->name, cdbw_server->host);
	// init profile connection
	cdbw_server->profile[0]->conn = conn;
  return TRUE;
}

void
cdbw_get_properties(T_Server * server){

	GtkWidget * wid = NULL;
	wid = glade_xml_get_widget(xml, "cdbw1_database_name");
	server->profile[0]->dbname = copy_string((gchar *)gtk_entry_get_text(GTK_ENTRY(wid)));

	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_database_wiz3")), "property_owner");
	if(wid == NULL) return;
	server->profile[0]->database.owner = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));

	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_database_wiz3")), "property_comment");
	if(wid == NULL) return;
	GtkTextBuffer * buffer;
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wid));
	GtkTextIter start, end;
  gtk_text_buffer_get_start_iter(buffer, &start);
  gtk_text_buffer_get_end_iter(buffer, &end);
	server->profile[0]->database.comment = copy_string(gtk_text_buffer_get_text(buffer, &start, &end, TRUE));

	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_database_wiz3")), "property_encoding");
	if(wid == NULL) return;
	server->profile[0]->database.encoding = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
	
	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_database_wiz3")), "property_template");
	if(wid == NULL) return;
	server->profile[0]->database.template = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
	
	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_database_wiz3")), "property_tablespace");
	if(wid == NULL) return;
	server->profile[0]->database.tablespace = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
	
	return;
}

gboolean
cdbw_properties_foreach_func (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data){
	gchar * property;
	T_Profile * profile = (T_Profile *) user_data;
	
	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
						 
	switch(depth){
	 case 2: 
	 	 switch(intpath[0]){
				case 0: // Common
					switch(intpath[1]){
						case 0: // Owner
								gtk_tree_model_get (model, iter, 2, &property, -1);
								profile->database.owner = get_text_from_markup(property);
								free(property);
             	break;
						case 1: // Comment
								gtk_tree_model_get (model, iter, 2, &property, -1);
								profile->database.comment = get_text_from_markup(property);
								free(property);
              break;
				  }
				  break;
				case 1: // Database
					switch(intpath[1]){
						case 0: // Encoding
								gtk_tree_model_get (model, iter, 2, &property, -1);
								profile->database.encoding = get_text_from_markup(property);
								free(property);
				  		break;
						case 1: // Template
								gtk_tree_model_get (model, iter, 2, &property, -1);
								profile->database.template = get_text_from_markup(property);
								free(property);
				  		break;
						case 2: // Tablespace
								gtk_tree_model_get (model, iter, 2, &property, -1);
								profile->database.tablespace = get_text_from_markup(property);
								free(property);
				  		break;
				  }
				  break;
	 	}
	}	
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Callbacks for create database wizard                                                                          */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void
connect_cdb_wiz_signals(GladeXML * source){
	GtkWidget * widget;
  
	// Check if database name not null
	widget = glade_xml_get_widget(source, "cdbw1_database_name");	
	g_signal_connect ((gpointer) widget, "changed",
                    G_CALLBACK (on_cdbw1_dbname_editing_done),
                    NULL);
                    
  // Check if host name not null
  widget = glade_xml_get_widget(source, "cdbw2_hostname");	                  
	g_signal_connect ((gpointer) widget, "changed",
                    G_CALLBACK (on_cdbw2_hostname_editing_done),
                    NULL);                    
}

void
on_cdbw1_dbname_editing_done(GtkEditable *editable, gpointer user_data)
{
	const gchar * db_name;
	db_name = gtk_entry_get_text( GTK_ENTRY( glade_xml_get_widget(xml,"cdbw1_database_name")));

	if(strcmp(db_name, "") == 0){
		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cdbw1_bt_next"), FALSE);
	}
	else{
		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cdbw1_bt_next"), TRUE);
	}
}

void
on_cdbw2_hostname_editing_done(GtkEditable *editable, gpointer user_data)
{
	const gchar * hname;
	hname = gtk_entry_get_text( GTK_ENTRY( glade_xml_get_widget(xml,"cdbw2_hostname")));

	if(strcmp(hname, "") == 0){
		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cdbw2_bt_next"), FALSE);
	}
	else{
		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cdbw2_bt_next"), TRUE);
	}
}
