/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "trigger_wizard.h"
#include "../functions.h"
#include "../database/db_connection.h"
#include "../database/create_db_objects.h"
#include "../database/get_db_objects.h"
#include "../otree/show_otree.h"
#include "../GUI/GUI_functions.h"
#include "../GUI/properties_widget.h"

// Structure for new trigger
T_Trigger * ctrw_trigger;
gint new_trigger_count = 0;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*						Create database profile wizard                                                                          */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void
init_c_trigger_wizard(T_Profile * profile){
	// Color settings
	GdkColor color;
	color.red = 255*257;
  color.green = 255*257;
  color.blue = 255*257;  
  gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctrw1_hr_bg"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctrw2_hr_bg"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctrw2_properties_bg2"), GTK_STATE_NORMAL, &color);
	
	color.red = 204*257;
  color.green = 0*257;
  color.blue = 0*257;  
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctrw1_hr"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctrw2_hr"), GTK_STATE_NORMAL, &color);

	color.red = 128*257;
  color.green = 128*257;
  color.blue = 128*257;  
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctrw2_properties_hr_bg"), GTK_STATE_NORMAL, &color);
	
	ctrw_trigger = NULL;
	
	// init trigger name
	GtkWidget * name = glade_xml_get_widget(xml, "ctrw1_name");
	gchar * new_trigger = append_string(NULL, "Trigger");
	gchar * count = itoa_n(new_trigger_count, 0);
	new_trigger = append_string(new_trigger, count);
	free(count);
	gtk_entry_set_text(GTK_ENTRY(name), new_trigger);
	free(new_trigger);
	
	// available tables
	GtkTreeModel      *model;
  if((model = gtk_combo_box_get_model(GTK_COMBO_BOX(glade_xml_get_widget(xml, "ctrw1_table")))) != NULL){
		gtk_list_store_clear(GTK_LIST_STORE(model));
	}
	else{
	  model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
		gtk_combo_box_set_model(GTK_COMBO_BOX(glade_xml_get_widget(xml, "ctrw1_table")), model);
		gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY(glade_xml_get_widget(xml, "ctrw1_table")), 0);
	}
	
	gtk_entry_set_editable(GTK_ENTRY(GTK_BIN(glade_xml_get_widget(xml, "ctrw1_table"))->child), FALSE);
	for(int i = 0; i < profile->database.n_schemas; i++){
		for(int j = 0; j < profile->database.schema[i]->n_tables; j++){
			gchar * table = NULL;
			table = append_string(table, profile->database.schema[i]->name);
			table = append_string(table, ".");			
			table = append_string(table, profile->database.schema[i]->table[j]->name);
			gtk_combo_box_append_text (GTK_COMBO_BOX (glade_xml_get_widget(xml, "ctrw1_table")), table);
			free(table);
		}
	}
	gtk_entry_set_text(GTK_ENTRY(GTK_BIN(glade_xml_get_widget(xml, "ctrw1_table"))->child), "");
	gtk_combo_box_set_active( GTK_COMBO_BOX (glade_xml_get_widget(xml, "ctrw1_table")), -1);
		
	// add properties
	GtkWidget * dialog = glade_xml_get_widget(xml, "c_trigger_wiz2");
		
	// add properties
	GtkWidget * properties_columns = glade_xml_get_widget(xml, "ctrw2_properties");
  GtkWidget * properties_column1 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column1, "properties_column1");
  gtk_widget_show (properties_column1);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column1, FALSE, TRUE, 0);
 
  GtkWidget * properties_column2 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column2, "properties_column2");
  gtk_widget_show (properties_column2);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column2, TRUE, TRUE, 0);
	
	gint height = 0;
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Common",  PIXMAP_DIRECTORY "/Common.png");
	height += 21;
	add_comment_entry_property("property_comment", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Comment", "");
	height += 41;
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Trigger",  PIXMAP_DIRECTORY "/Trigger.png");
	height += 21;
	add_toggle_property("property_enabled", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Enabled", TRUE, TRUE);
	height += 21;
	gchar *type[] = {"Before", "After"};
	gint id = 0;
	add_combo_property("property_type", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Type", type,  2, id);
	height += 21;
	gchar *for_each[] = {"Row", "Statement"};
	id = 0;
	add_combo_property("property_for_each", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "For Each", for_each,  2, id);
	height += 21;
	add_entry_property("property_events", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Event(s)", "");
	height += 21;
	GtkWidget * wid = get_widget_child_by_name(dialog, "property_events");	
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "changed",
                    G_CALLBACK (on_ctrw2_entries_changed),
                    wid);  
	}	
	// make list of available functions
	gchar ** list = NULL;
	gint n = 0;
	for(int i = 0; i < profile->database.n_schemas; i++){
		for(int j = 0; j < profile->database.schema[i]->n_functions; j++){
			T_Function * function = profile->database.schema[i]->function[j];
			if(strcmp(function->return_type, "trigger") == 0){
				 gchar * func = append_string(NULL, profile->database.schema[i]->name);
				 func = append_string(func, ".");
				 func = append_string(func, function->name);
				 list = append_string_to_array(list, &n ,func);
			}
		}
	}
	id = -1;
	add_combo_property("property_function", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Function", list,  n, id);
	height += 21;
	for(int i = 0; i < n; i++){
		free(list[i]);
	}
	free(list);
	wid = get_widget_child_by_name(dialog, "property_function");	
	if(wid != NULL){
	 g_signal_connect ((gpointer) GTK_BIN(wid)->child, "changed",
                    G_CALLBACK (on_ctrw2_entries_changed),
                    GTK_BIN(wid)->child);  
	}	
	add_entry_property("property_arguments", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Arguments", "");
	height += 21;
	gtk_widget_set_size_request (glade_xml_get_widget(xml, "ctrw2_scrolledwindow"), -1, height);
}

void 
run_c_trigger_wizard(T_Profile * profile){

	GtkWidget * widget = NULL;
	gint page = 1;
	gboolean not_end = TRUE;
	
	init_c_trigger_wizard(profile);

	not_end = TRUE;
	while(not_end){
		switch(page){
			case 1:	widget = glade_xml_get_widget(xml,"c_trigger_wiz1"); break;
			case 2: widget = glade_xml_get_widget(xml,"c_trigger_wiz2"); break;
			default:;
		};
		gint res = gtk_dialog_run(GTK_DIALOG(widget));
		
		if(res == GTK_RESPONSE_YES){
			if(page < 2){
				if(page == 1){ // get new db name
					page++;
					gtk_widget_hide(widget);	
				}
				else {
					 page++;
  				 gtk_widget_hide(widget);
				}
			}
			else{ // end the wizard
					not_end = FALSE;
					// get properties
					T_Trigger * new_trigger = NULL;
					if(( new_trigger = (T_Trigger *) malloc (sizeof(T_Trigger)) ) == NULL){	
						fprintf(stderr, "Memory allocation error: callbacks.c");
	    			exit(0);
					}
					
					// get properties
					new_trigger = ctrw_get_properties(new_trigger);

					gchar * parent = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(glade_xml_get_widget(xml, "ctrw1_table"))->child)));										
					new_trigger->name  = copy_string(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml, "ctrw1_name"))));			
												
					/* create trigger */
					gchar * sql =  create_new_trigger_SQL_query(new_trigger, parent, TRUE, FALSE);					
					// execute sql
					if(sql != NULL){
						if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)){
							// get server
							gboolean e = FALSE;
							gint i_server = 0;
							for(int k = 0; k < o_tree.n_servers; k++){
								for(int l = 0; l < o_tree.server[k]->n_profiles; l++){
									if(o_tree.server[k]->profile[l] == profile){
										i_server = k;
										e = TRUE;
										break;
									}
								}
								if(e) break;
							}
							gui_refresh( o_tree.server[i_server], profile);
							not_end = FALSE;
							new_trigger_count++;
							gtk_widget_hide(widget);
						}
						else{
							not_end=TRUE;
						}
						free(sql);
					}
					
					// free trigger structure
					free(new_trigger->name);
					free(new_trigger->comment);
					free(new_trigger->type);
					free(new_trigger->for_each);
					free(new_trigger->events);
					free(new_trigger->function);
					free(new_trigger->arguments);																					
  				free(parent);
  		
			}
		}
		else if(res == GTK_RESPONSE_NO){
			if(page > 1){
				page--;
				gtk_widget_hide(widget);
			}
		}
		else if(res == GTK_RESPONSE_CANCEL || res == GTK_RESPONSE_DELETE_EVENT){
				not_end = FALSE;
				gtk_widget_hide(widget);
		}
	}
	
	// clear properties
	GtkWidget * wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_trigger_wiz2")), "properties_column1");
	if(wid != NULL) gtk_widget_destroy(wid);
	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_trigger_wiz2")), "properties_column2");
	if(wid != NULL) gtk_widget_destroy(wid);
}

T_Trigger *
ctrw_get_properties(T_Trigger * trigger){
	GtkWidget * dialog = glade_xml_get_widget(xml, "c_trigger_wiz2");
	
	// comment
	GtkWidget * wid = NULL;
	wid = get_widget_child_by_name(dialog, "property_comment");
	if(wid != NULL){
		GtkTextBuffer * buffer;
		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wid));
		GtkTextIter start, end;
  	gtk_text_buffer_get_start_iter(buffer, &start);
  	gtk_text_buffer_get_end_iter(buffer, &end);
		trigger->comment = copy_string(gtk_text_buffer_get_text(buffer, &start, &end, TRUE));
	}
	else trigger->comment = copy_string("");
	
	// enabled
	wid = NULL;
	wid = get_widget_child_by_name(dialog, "property_enabled");
	if(wid != NULL){
		trigger->enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
	}
	else trigger->enabled = FALSE;
	
	// type
	wid = NULL;
	wid = get_widget_child_by_name(dialog, "property_type");
	if(wid != NULL){
		trigger->type = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
	}
	else trigger->type = copy_string("");
	
	// for each
	wid = NULL;
	wid = get_widget_child_by_name(dialog, "property_for_each");
	if(wid != NULL){
		trigger->for_each = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
	}
	else trigger->for_each = copy_string("");
	
	// events
	wid = NULL;
	wid = get_widget_child_by_name(dialog, "property_events");
	if(wid != NULL){
		trigger->events = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
	}
	else trigger->events = copy_string("");
	
	// function
	wid = NULL;
	wid = get_widget_child_by_name(dialog, "property_function");
	if(wid != NULL){
		trigger->function = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
	}
	else trigger->function = copy_string("");
	
	// arguments
	wid = NULL;
	wid = get_widget_child_by_name(dialog, "property_arguments");
	if(wid != NULL){
		trigger->arguments = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
	}
	else trigger->arguments = copy_string("");
		
	return trigger;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Callbacks for create database wizard                                                                          */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void
connect_ctrw_signals(GladeXML * source){
	GtkWidget * widget;
  
	// Check if database name not null
	widget = glade_xml_get_widget(source, "ctrw1_name");	
	g_signal_connect ((gpointer) widget, "changed",
                    G_CALLBACK (on_ctrw1_entries_changed),
                    NULL);
	widget = glade_xml_get_widget(source, "ctrw1_table");	
	g_signal_connect ((gpointer) widget, "changed",
                    G_CALLBACK (on_ctrw1_entries_changed),
                    NULL);                    
                                     
}

void
on_ctrw1_entries_changed(GtkEditable *editable, gpointer user_data)
{
	const gchar * name, *table;
	name = gtk_entry_get_text( GTK_ENTRY( glade_xml_get_widget(xml,"ctrw1_name")));
  table = gtk_entry_get_text( GTK_ENTRY( GTK_BIN(glade_xml_get_widget(xml,"ctrw1_table"))->child));
  
	if(strcmp(name, "") == 0 || strcmp(table, "") == 0){
		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"ctrw1_bt_next"), FALSE);
	}
	else{
		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"ctrw1_bt_next"), TRUE);
	}
}

void
on_ctrw2_entries_changed(GtkEditable *editable, gpointer user_data)
{
	GtkEntry * entry = GTK_ENTRY(user_data);
	gchar * text = copy_string(gtk_entry_get_text(entry));
	if(strcmp(text, "") == 0){
		gtk_widget_set_sensitive(glade_xml_get_widget(xml, "ctrw2_bt_ready"), FALSE);
	}
	else{
		GtkWidget * dialog = glade_xml_get_widget(xml, "c_trigger_wiz2");
		GtkWidget * wid1 = get_widget_child_by_name(dialog, "property_events");
		GtkWidget * wid2 = get_widget_child_by_name(dialog, "property_function");
		if(wid1 != NULL && wid2 != NULL){
			if(strcmp(gtk_entry_get_text(GTK_ENTRY(wid1)), "") != 0 && strcmp(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid2)->child)), "") != 0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "ctrw2_bt_ready"), TRUE);
		}
		
	}
}
