/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */
 
#include "drop_db_objects.h"

/*
 * Creates SQL query to drop database
 */
gchar *
create_drop_database_SQL_query(T_Profile * profile){
	gchar * sql = NULL;
	
	if(profile == NULL) return NULL;
	
	sql = append_string(sql, "DROP DATABASE \"");
	sql = append_string(sql, profile->dbname);
	sql = append_string(sql, "\";\n");
	
	
	return sql;
} // create_drop_database_SQL_query ()

/*
 * Creates SQL query to drop schema
 */
gchar *
create_drop_schema_SQL_query(T_Schema * schema, gboolean cascade){
	gchar * sql = NULL;
	
	if(schema == NULL) return NULL;
	
	sql = append_string(sql, "DROP SCHEMA \"");
	sql = append_string(sql, schema->name);
	sql = append_string(sql, "\"");
	
	if(cascade) sql = append_string(sql, " CASCADE");
	
	sql = append_string(sql, ";\n");
	
	return sql;
} // create_drop_schema_SQL_query ()

/*
 * Creates SQL query to drop table
 */
gchar *
create_drop_table_SQL_query(T_Table * table, gchar * schema, gboolean cascade){
	gchar * sql = NULL;
	
	if(table == NULL) return NULL;
	
	sql = append_string(sql, "DROP TABLE \"");
	
	if(schema != NULL){
	 	sql = append_string(sql, schema);
	 	sql = append_string(sql, "\".\"");
	}
	
	sql = append_string(sql, table->name);
	sql = append_string(sql, "\"");
	if(cascade) sql = append_string(sql, " CASCADE");
	
	sql = append_string(sql, ";\n");
	
	return sql;
} // create_drop_table_SQL_query ()

/*
 * Creates SQL query to drop trigger
 */
gchar *
create_drop_trigger_SQL_query(T_Trigger * trigger, gchar * parent_name, gboolean cascade){
	gchar * sql = NULL;
	
	if(trigger == NULL) return NULL;
	
	sql = append_string(sql, "DROP TRIGGER \"");
	sql = append_string(sql, trigger->name);
	sql = append_string(sql, "\"\n");
	sql = append_string(sql, "  ON \"");
	sql = append_string(sql, parent_name);
	sql = append_string(sql, "\"");
	
	if(cascade) sql = append_string(sql, " CASCADE");
	
	sql = append_string(sql, ";\n");
	
	return sql;
} // create_drop_trigger_SQL_query ()

/*
 * Creates SQL query to drop rule
 */
gchar *
create_drop_rule_SQL_query(T_Rule * rule, gchar * parent_name, gboolean cascade){
	gchar * sql = NULL;
	
	if(rule == NULL) return NULL;
	
	sql = append_string(sql, "DROP RULE \"");
	sql = append_string(sql, rule->name);
	sql = append_string(sql, "\"\n");
	sql = append_string(sql, "  ON \"");
	sql = append_string(sql, parent_name);
	sql = append_string(sql, "\"");
	
	if(cascade) sql = append_string(sql, " CASCADE");
	
	sql = append_string(sql, ";\n");
	
	return sql;
} // create_drop_rule_SQL_query ()

/*
 * Creates SQL query to drop field
 */
gchar * 
create_drop_field_SQL_query(T_Column * field, gchar * parent_name, gboolean cascade){
	gchar * sql = NULL;
	
	if(field == NULL) return NULL;
	
	sql = append_string(sql, "ALTER TABLE \"");
	sql = append_string(sql, parent_name);
	sql = append_string(sql, "\"\n");
	sql = append_string(sql, "  DROP COLUMN \"");
	sql = append_string(sql, field->name);
	sql = append_string(sql, "\"");
	
	if(cascade) sql = append_string(sql, " CASCADE");
	
	sql = append_string(sql, ";\n");
	
	return sql;
} // create_drop_field_SQL_query ()

/*
 * Creates SQL query to drop index
 */
gchar *
create_drop_index_SQL_query(T_Index * index, gchar * parent_name, gboolean cascade){
	gchar * sql = NULL;
	
	if(index == NULL) return NULL;
	
	if(index->pkey_constraint || index->unique_constraint){
		sql = append_string(sql, "ALTER TABLE \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n");
		sql = append_string(sql, "  DROP CONSTRAINT \"");
		sql = append_string(sql, index->name);
		sql = append_string(sql, "\"");
	}
	else{
		sql = append_string(sql, "DROP INDEX \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\".\"");
		sql = append_string(sql, index->name);
		sql = append_string(sql, "\"");
	}
	
	if(cascade) sql = append_string(sql, " CASCADE");
	
	sql = append_string(sql, ";\n");
	
	return sql;
} // create_drop_index_SQL_query ()

/*
 * Creates SQL query to drop foreign key
 */
gchar *
create_drop_fkey_SQL_query(T_FKey * fkey, gchar * parent_name, gboolean cascade){
	gchar * sql = NULL;
	
	if(fkey == NULL) return NULL;
	
	sql = append_string(sql, "ALTER TABLE \"");
	sql = append_string(sql, parent_name);
	sql = append_string(sql, "\"\n");
	sql = append_string(sql, "  DROP CONSTRAINT \"");
	sql = append_string(sql, fkey->name);
	sql = append_string(sql, "\"");
	
	if(cascade) sql = append_string(sql, " CASCADE");
	
	sql = append_string(sql, ";\n");
	
	return sql;
} // create_drop_fkey_SQL_query ()

/*
 * Creates SQL query to drop check
 */
gchar *
create_drop_check_SQL_query(T_Check * check, gchar * parent_name, gboolean cascade){
	gchar * sql = NULL;
	
	if(check == NULL) return NULL;
	
	sql = append_string(sql, "ALTER TABLE \"");
	sql = append_string(sql, parent_name);
	sql = append_string(sql, "\"\n");
	sql = append_string(sql, "  DROP CONSTRAINT \"");
	sql = append_string(sql, check->name);
	sql = append_string(sql, "\"");
	
	if(cascade) sql = append_string(sql, " CASCADE");
	
	sql = append_string(sql, ";\n");
	
	return sql;
} // create_drop_check_SQL_query ()
