/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */
#include "get_db_objects.h"
#include "db_connection.h"

/*
 * Connects to the database and gets db objects
 */
gboolean
get_db_objects (T_Server * server, T_Profile * profile)
{
	// Connecting to database
	gchar * conninfo = NULL;
	gchar * port = itoa_n(server->port, 0);

	if(profile->conn == NULL){ // If not connected yet
	  if(strcmp(server->passwd,"")==0){ // If no password saved
	    gchar * user_text = bold_text(server->uname, "#000000");
	    gchar * server_text = bold_text(server->host, "#000000");
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "password_dialog_password")), "");
	    gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml, "password_dialog_user")), user_text);
	    gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml, "password_dialog_server")), server_text);
	
	    free(user_text);
	    free(server_text);
	
	    // Show password dialog
	    gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "password_dialog")));
	    if(res == GTK_RESPONSE_OK){
	    	free(server->passwd);
	     	server->passwd = copy_string(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml, "password_dialog_password"))));
	  	  gtk_widget_hide(glade_xml_get_widget(xml, "password_dialog"));
	    }
	    else{
	    	gtk_widget_hide(glade_xml_get_widget(xml, "password_dialog"));
	  	  return FALSE;
	    }
	  }
	  // Connect to database
  	conninfo = make_conninfo(server->host, port, profile->dbname, server->uname, server->passwd, server->SSL);
  	free(port);
    profile->conn = connect_db(conninfo);
  	
  	if(profile->conn == NULL) return FALSE;
  	free(conninfo);
  }
	
	// Get database info
	if(!get_db_info(profile)) return FALSE;	
	
	// Get roles -- not solved
	server->n_roles = 0;
	server->role = NULL;
	
	// Tablespaces -- not solved
	server->n_tablespaces = 0;
	server->tablespace = NULL;
	
	// Get db objects
	// get schemas
  if(!get_schemas(profile, profile->conn)) return FALSE;
	else{
		for(int i = 0; i < profile->database.n_schemas; i++){
			for(int j = 0; j < profile->database.schema[i]->n_tables; j++){
				// Get foreign keys
				if(!get_table_foreign_keys( profile, profile->database.schema[i], profile->database.schema[i]->table[j], profile->conn)) return FALSE;
				// Get references
		  	if(!get_table_references( profile, profile->database.schema[i], profile->database.schema[i]->table[j], profile->conn)) return FALSE;
			}
		}
		
  }

	// get languages
	if(!get_languages(profile, profile->conn)) return FALSE;

	// get casts
	if(!get_casts(profile, profile->conn)) return FALSE;

	return TRUE;
} // get_db_objects()

/*
 * Gets inforamtions about database
 */
gboolean
get_db_info(T_Profile * profile){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT datname AS name, rolname AS owner, description AS comment, pg_database.oid, "
						           "             datconnlimit AS conn_limit, pg_encoding_to_char(encoding) AS encoding, CASE WHEN datistemplate THEN 'true' ELSE 'false' END AS is_template, spcname AS tablespace, datlastsysoid AS lastoid, datacl AS acl "
						           "FROM pg_database "
						           "					LEFT OUTER JOIN pg_authid ON pg_database.datdba = pg_authid.oid "
						           "					LEFT OUTER JOIN pg_tablespace ON pg_database.dattablespace = pg_tablespace.oid "
						           "					LEFT OUTER JOIN pg_description ON pg_database.oid = pg_description.objoid "
						           "WHERE datname = '";
	gchar sql2[]="';";
	guint len = strlen(sql1)+strlen(profile->dbname)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, profile->dbname); strcat(sql, sql2);
	
	// Execution
	PGresult *res = PQexec(profile->conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "owner");
	guint col2 = PQfnumber(res, "comment");
	guint col3 = PQfnumber(res, "oid");
	guint col4 = PQfnumber(res, "conn_limit");
	guint col5 = PQfnumber(res, "encoding");
	guint col6 = PQfnumber(res, "is_template");
	guint col7 = PQfnumber(res, "tablespace");
	guint col8 = PQfnumber(res, "lastoid");
	guint col9 = PQfnumber(res, "acl");
	
	// get values
	guint row;
	for (row = 0; row < PQntuples(res); row++){
		profile->database.owner = copy_string(PQgetvalue(res, row, col1));
		profile->database.comment = copy_string(PQgetvalue(res, row, col2));
		profile->database.OID = atoi(PQgetvalue(res, row, col3));
		profile->database.connection_limit = atoi(PQgetvalue(res, row, col4));
		profile->database.encoding = copy_string(PQgetvalue(res, row, col5));
		profile->database.is_template = strcmp(PQgetvalue(res, row, col6), "true") == 0 ? TRUE : FALSE;
		profile->database.tablespace = copy_string(PQgetvalue(res, row, col7));
		profile->database.lastOID = atoi(PQgetvalue(res, row, col8));
		profile->database.acl = copy_string(PQgetvalue(res, row, col9));
		profile->database.template = copy_string("");
		profile->modified = FALSE;
	}
	
	PQclear(res);
	return TRUE;	
}// get_db_info()

/*
 * Gets tablespaces
 */
gboolean
get_tablespaces(T_Server * server, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql[]="SELECT spcname AS name, rolname AS owner, pg_tablespace.oid AS oid, "
						           "						 spcacl as acl "
						           "FROM pg_tablespace "
						           "					LEFT OUTER JOIN pg_authid ON pg_tablespace.spcowner = pg_authid.oid "
						           "ORDER BY name;";
						           
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "owner");
	guint col3 = PQfnumber(res, "oid");
	guint col4 = PQfnumber(res, "acl");
	
	// Get values
	guint row;
	for (row = 0; row < PQntuples(res); row++){
			guint n = server->n_tablespaces;
			
			// Create new tablespace struct
			if(server->tablespace == NULL){
				 if((server->tablespace = (T_Tablespace **) malloc(sizeof(T_Tablespace*))) == NULL){
	 		    fprintf(stderr, "Memory allocation error.");
	 	     exit(0);
	 	   }
			}
			else{
	 		 	if((server->tablespace = (T_Tablespace **) realloc((server->tablespace), sizeof(T_Tablespace*)*(n+1))) == NULL){
	 		    fprintf(stderr, "Memory allocation error.");
	    	  exit(0);
	   		}
			}
	
			if((server->tablespace[n] = (T_Tablespace*) malloc(sizeof(T_Tablespace))) == NULL){
		 	  fprintf(stderr, "Memory allocation error: callbacks.c");
	  	  exit(0);
			}

		server->tablespace[n]->name = copy_string(PQgetvalue(res, row, col1));
		server->tablespace[n]->owner = copy_string(PQgetvalue(res, row, col2));
		server->tablespace[n]->OID = atoi(PQgetvalue(res, row, col3));
		server->tablespace[n]->acl = copy_string(PQgetvalue(res, row, col4));
		
		server->n_tablespaces++;
	}
	
	PQclear(res);
	return TRUE;	
} // get_tablespaces()

/*
 * Gets schemas in database
 */
gboolean
get_schemas(T_Profile * profile, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql[]="SELECT CASE WHEN nspname LIKE 'pg\\_temp\\_%%' THEN 1 WHEN (nspname LIKE 'pg\\_%' OR nspname = 'information_schema') THEN 0 ELSE 3 END AS nsptyp, "
			                 "              nspname AS name, rolname AS owner, description AS comment, pg_namespace.oid AS oid, "
			                 "			        nspacl AS acl "
			                 "FROM pg_namespace "
			                 "					LEFT OUTER JOIN pg_authid ON pg_namespace.nspowner = pg_authid.oid "
			                 "					LEFT OUTER JOIN pg_description on pg_namespace.oid = pg_description.objoid "
			                 "ORDER BY name;";

	// Execution
	PGresult *res = PQexec(conn, sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "nsptyp");
	guint col2 = PQfnumber(res, "name");
	guint col3 = PQfnumber(res, "owner");
	guint col4 = PQfnumber(res, "comment");
	guint col5 = PQfnumber(res, "oid");
	guint col6 = PQfnumber(res, "acl");
	
	// Get values
	guint row;
	for (row = 0; row < PQntuples(res); row++){

			if(strcmp(PQgetvalue(res, row, col1), "3")!=0) continue;
			
			guint n = profile->database.n_schemas;
			
			// Create new schema struct
			if(profile->database.schema == NULL){
				 if((profile->database.schema = (T_Schema **) malloc(sizeof(T_Schema*))) == NULL){
	 		    fprintf(stderr, "Memory allocation error.");
	 	     exit(0);
	 	   }
			}
			else{
	 		 	if((profile->database.schema = (T_Schema **) realloc((profile->database.schema), sizeof(T_Schema*)*(n+1))) == NULL){
	 		    fprintf(stderr, "Memory allocation error.");
	    	  exit(0);
	   		}
			}
	
			if((profile->database.schema[n] = (T_Schema*) malloc(sizeof(T_Schema))) == NULL){
		 	  fprintf(stderr, "Memory allocation error: callbacks.c");
	  	  exit(0);
			}
			
			profile->database.schema[n]->name = copy_string(PQgetvalue(res, row, col2));
			profile->database.schema[n]->owner = copy_string(PQgetvalue(res, row, col3));
			profile->database.schema[n]->comment = copy_string(PQgetvalue(res, row, col4));
			profile->database.schema[n]->OID = atoi(PQgetvalue(res, row, col5));
			profile->database.schema[n]->acl = copy_string(PQgetvalue(res, row, col6));
			
			// Init objects
			profile->database.schema[n]->table = NULL;          // tables
			profile->database.schema[n]->n_tables = 0;
			profile->database.schema[n]->view = NULL;          // views
			profile->database.schema[n]->n_views = 0;
			profile->database.schema[n]->function = NULL;     // functions
			profile->database.schema[n]->n_functions = 0;
			profile->database.schema[n]->domain = NULL;      // domains
			profile->database.schema[n]->n_domains = 0;			
			profile->database.schema[n]->aggregate = NULL;  // aggregates
			profile->database.schema[n]->n_aggregates = 0;
			profile->database.schema[n]->sequence = NULL;   // sequences 
			profile->database.schema[n]->n_sequences = 0;
			profile->database.schema[n]->type = NULL;            // types
			profile->database.schema[n]->n_types = 0;
			profile->database.schema[n]->operator = NULL;     // operators
			profile->database.schema[n]->n_operators = 0;
			profile->database.schema[n]->trigger = NULL;         // triggers
			profile->database.schema[n]->n_triggers = 0;
			profile->database.schema[n]->rule = NULL;             // rules
			profile->database.schema[n]->n_rules = 0;
			profile->database.schema[n]->dep_on.dep = NULL;// dependencies
			profile->database.schema[n]->dep_on.n_deps = 0;
			profile->database.schema[n]->ref_by.dep = NULL;
			profile->database.schema[n]->ref_by.n_deps = 0;
			// Get tables
			if(!get_tables(profile, profile->database.schema[n],conn)) return FALSE;
			
			// Get Views
			if(!get_views(profile->database.schema[n],conn)) return FALSE;
			
			// Get Functions
			if(!get_functions(profile->database.schema[n],conn)) return FALSE;
			
			// Get Domains
			if(!get_domains(profile->database.schema[n],conn)) return FALSE;
			
			// Get Aggregates
			if(!get_aggregates(profile->database.schema[n],conn)) return FALSE;
			
			// Get Sequences
			if(!get_sequences(profile->database.schema[n],conn)) return FALSE;
			
			// Get Types
			if(!get_types(profile->database.schema[n],conn)) return FALSE;
			
			// Get Operators
			if(!get_operators(profile->database.schema[n],conn)) return FALSE;
			
			// Get dependencies
			if(!get_dep_ons(profile->database.schema[n]->OID, &(profile->database.schema[n]->dep_on), conn)) return FALSE;
			if(!get_ref_bys(profile->database.schema[n]->OID, &(profile->database.schema[n]->ref_by), conn)) return FALSE;

			// Increment count
			profile->database.n_schemas++;
	}
	
	PQclear(res);
	return TRUE;
} // get_schemas()

/*
 * Gets languages in database
 */
gboolean
get_languages(T_Profile * profile, PGconn * conn){
 	GtkWidget * mess;
	
	// SQL query
	gchar sql[]="SELECT lanname AS name, description AS comment, pg_language.oid AS oid, "
			                "						 CASE WHEN lanpltrusted THEN 'true' ELSE 'false' END AS trusted, han.proname AS handler, val.proname AS validator, lanacl AS acl "
			                "FROM pg_language "
			                "					LEFT OUTER JOIN pg_proc han ON pg_language.lanplcallfoid = han.oid " 
			                "					LEFT OUTER JOIN pg_proc val ON pg_language.lanvalidator = val.oid "
			                "					LEFT OUTER JOIN pg_description ON pg_language.oid = pg_description.objoid "
			                "WHERE lanispl=true "
			                "ORDER BY name;";


	// Execution
	PGresult *res = PQexec(conn, sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "comment");
	guint col3 = PQfnumber(res, "oid");
	guint col4 = PQfnumber(res, "trusted");
	guint col5 = PQfnumber(res, "handler");
	guint col6 = PQfnumber(res, "validator");
	guint col7 = PQfnumber(res, "acl");
	
	guint row;
	for (row = 0; row < PQntuples(res); row++){
	
			guint n = profile->database.n_languages;
			// Create new language
			if(profile->database.language == NULL){
				 if((profile->database.language = (T_Language **) malloc(sizeof(T_Language*))) == NULL){
	 		    fprintf(stderr, "Memory allocation error.");
	 	     exit(0);
	 	   }
			}
			else{
	 		 	if((profile->database.language = (T_Language **) realloc((profile->database.language), sizeof(T_Language*)*(n+1))) == NULL){
	 		    fprintf(stderr, "Memory allocation error.");
	    	  exit(0);
	   		}
			}
	
			if((profile->database.language[n] = (T_Language*) malloc(sizeof(T_Language))) == NULL){
		 	  fprintf(stderr, "Memory allocation error: callbacks.c");
	  	  exit(0);
			}
			
			profile->database.language[n]->name = copy_string(PQgetvalue(res, row, col1));
			profile->database.language[n]->comment = copy_string(PQgetvalue(res, row, col2));
			profile->database.language[n]->OID = atoi(PQgetvalue(res, row, col3));
			profile->database.language[n]->trusted = strcmp(PQgetvalue(res, row, col4), "true") == 0 ? TRUE : FALSE;
			profile->database.language[n]->handler_func = copy_string(PQgetvalue(res, row, col5));
			profile->database.language[n]->validator_func = copy_string(PQgetvalue(res, row, col6));
			profile->database.language[n]->acl = copy_string(PQgetvalue(res, row, col7));
			
			profile->database.language[n]->dep_on.dep = NULL;// dependencies
			profile->database.language[n]->dep_on.n_deps = 0;
			profile->database.language[n]->ref_by.dep = NULL;
			profile->database.language[n]->ref_by.n_deps = 0;
			
			// Get dependencies
			if(!get_dep_ons(profile->database.language[n]->OID, &(profile->database.language[n]->dep_on), conn)) return FALSE;
			if(!get_ref_bys(profile->database.language[n]->OID, &(profile->database.language[n]->ref_by), conn)) return FALSE;

			// Increment count
			profile->database.n_languages++;
  }

	PQclear(res);
	return TRUE;	
} // get_languages ()

/*
 * Gets casts in database
 */
gboolean
get_casts(T_Profile * profile, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT description AS comment, pg_cast.oid AS oid, "
							         " 						stype.typname AS source_type, ttype.typname AS target_type, func.proname AS function, "
						           "							CASE WHEN castcontext='a' THEN 'Assignment' WHEN castcontext='e' THEN 'Explicit' WHEN castcontext='i' THEN 'Implicit' END AS context "
						           "FROM pg_cast "
						           "					LEFT OUTER JOIN pg_type stype ON pg_cast.castsource = stype.oid "
						           "					LEFT OUTER JOIN pg_type ttype ON pg_cast.casttarget = ttype.oid "
						           "					LEFT OUTER JOIN pg_proc func ON pg_cast.castfunc = func.oid "
						           "					LEFT OUTER JOIN pg_description ON pg_cast.oid = pg_description.objoid "
						           "WHERE pg_cast.oid > ";
	gchar sql2[]=" ORDER BY source_type, target_type;";
	
	gchar * lastoid = itoa_n(profile->database.lastOID, 0);
	guint len = strlen(sql1)+strlen(lastoid)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql,lastoid); strcat(sql, sql2);
	free(lastoid);

	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "comment");
	guint col2 = PQfnumber(res, "oid");
	guint col3 = PQfnumber(res, "source_type");
	guint col4 = PQfnumber(res, "target_type");
	guint col5 = PQfnumber(res, "function");
	guint col6 = PQfnumber(res, "context");
	
	// Get values
	guint row;
	for (row = 0; row < PQntuples(res); row++){
	
			guint n = profile->database.n_casts;
			// Create new cast
			if(profile->database.cast == NULL){
				 if((profile->database.cast = (T_Cast **) malloc(sizeof(T_Cast*))) == NULL){
	 		    fprintf(stderr, "Memory allocation error.");
	 	     exit(0);
	 	   }
			}
			else{
	 		 	if((profile->database.cast = (T_Cast **) realloc((profile->database.cast), sizeof(T_Cast*)*(n+1))) == NULL){
	 		    fprintf(stderr, "Memory allocation error.");
	    	  exit(0);
	   		}
			}
	
			if((profile->database.cast[n] = (T_Cast*) malloc(sizeof(T_Cast))) == NULL){
		 	  fprintf(stderr, "Memory allocation error: callbacks.c");
	  	  exit(0);
			}
			
			profile->database.cast[n]->comment = copy_string(PQgetvalue(res, row, col1));
			profile->database.cast[n]->OID = atoi(PQgetvalue(res, row, col2));
			profile->database.cast[n]->source_type = copy_string(PQgetvalue(res, row, col3));
			profile->database.cast[n]->target_type = copy_string(PQgetvalue(res, row, col4));
			profile->database.cast[n]->function = copy_string(PQgetvalue(res, row, col5));
			profile->database.cast[n]->context = copy_string(PQgetvalue(res, row, col6));			
			
			gchar * name = alloc_string(strlen(profile->database.cast[n]->source_type)+strlen(" -> ")+strlen(profile->database.cast[n]->target_type)+1);
			strcpy(name, profile->database.cast[n]->source_type); strcat(name," -> "); strcat(name,profile->database.cast[n]->target_type);
			profile->database.cast[n]->name = name;
			
			// Increment count
			profile->database.n_casts++;
  }

	PQclear(res);
	return TRUE;	
} // get_casts ()

/*
 * Gets tables in schema
 */
gboolean
get_tables(T_Profile * profile, T_Schema * schema, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT tablename AS name, tableowner AS owner, description AS comment, class.oid AS oid, "
						           "						 CASE WHEN class.relhasoids THEN 'true' ELSE 'false' END AS with_oids, inhf.relname AS inherited_from, inhfsch.nspname AS inherited_from_schema, tablespace, class.relacl AS acl "
						           "FROM pg_tables "
						           "					LEFT OUTER JOIN pg_class class ON pg_tables.tablename = class.relname "
						           "					LEFT OUTER JOIN pg_inherits inher ON inher.inhrelid = class.oid "
						           "					LEFT OUTER JOIN pg_class inhf ON inher.inhparent = inhf.oid "
						           "          LEFT OUTER JOIN pg_namespace inhfsch ON inhf.relnamespace = inhfsch.oid "
						           "					LEFT OUTER JOIN pg_description ON class.oid = pg_description.objoid AND pg_description.objsubid = 0 "
						           "WHERE schemaname = '";
	gchar sql2[]="' ORDER BY tablename;";
	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "owner");
	guint col3 = PQfnumber(res, "comment");
	guint col4 = PQfnumber(res, "oid");
	guint col5 = PQfnumber(res, "with_oids");
	guint col6 = PQfnumber(res, "inherited_from");
	guint col7 = PQfnumber(res, "tablespace");
	guint col8 = PQfnumber(res, "acl");
	guint col9 = PQfnumber(res, "inherited_from_schema");
	
	// Get values
	guint row;
	gchar * previous = NULL;
	for(row = 0; row < PQntuples(res); row++){
		guint n = schema->n_tables;
		
		if(previous != NULL){
			if(strcmp(PQgetvalue(res, row, col1), previous) == 0){
				schema->table[n-1]->inherited_from = append_string(schema->table[n-1]->inherited_from, ", ");
				schema->table[n-1]->inherited_from = append_string(schema->table[n-1]->inherited_from, PQgetvalue(res, row, col9));
				schema->table[n-1]->inherited_from = append_string(schema->table[n-1]->inherited_from, ".");
				schema->table[n-1]->inherited_from = append_string(schema->table[n-1]->inherited_from, PQgetvalue(res, row, col6));
				continue;
			}
		}
		
		// Create new schema struct
		if(schema->table == NULL){
			if((schema->table = (T_Table **) malloc(sizeof(T_Table*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((schema->table = (T_Table **) realloc((schema->table), sizeof(T_Table*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((schema->table[n] = (T_Table*) malloc(sizeof(T_Table))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		schema->table[n]->name = copy_string(PQgetvalue(res, row, col1));
		previous = schema->table[n]->name;
		
		schema->table[n]->owner = copy_string(PQgetvalue(res, row, col2));
		schema->table[n]->comment = copy_string(PQgetvalue(res, row, col3));
		schema->table[n]->OID = atoi(PQgetvalue(res, row, col4));
		schema->table[n]->WithOIDs = strcmp(PQgetvalue(res, row, col5), "true") == 0 ? TRUE : FALSE;
		
		schema->table[n]->inherited_from = append_string(NULL, PQgetvalue(res, row, col9));
		if(strcmp(schema->table[n]->inherited_from, "") != 0){
			schema->table[n]->inherited_from = append_string(schema->table[n]->inherited_from, ".");
			schema->table[n]->inherited_from = append_string(schema->table[n]->inherited_from, PQgetvalue(res, row, col6));
		}
		schema->table[n]->tablespace = copy_string(PQgetvalue(res, row, col7));
		schema->table[n]->acl = copy_string(PQgetvalue(res, row, col8));
			
		// Init table objects
		schema->table[n]->column = NULL;     // fields
		schema->table[n]->n_columns = 0;
		schema->table[n]->index = NULL;        // indexes 
		schema->table[n]->n_indexes = 0;
		schema->table[n]->check = NULL;       // checks
		schema->table[n]->n_checks = 0;
		schema->table[n]->trigger = NULL;       // triggers
		schema->table[n]->n_triggers = 0;
		schema->table[n]->fkey = NULL;          // fkeys
		schema->table[n]->n_fkeys = 0;
		schema->table[n]->rule = NULL;          // rules
		schema->table[n]->n_rules = 0;
		schema->table[n]->reference = NULL; // references
		schema->table[n]->n_references = 0;
		schema->table[n]->dep_on.dep = NULL;// dependencies
		schema->table[n]->dep_on.n_deps = 0;
		schema->table[n]->ref_by.dep = NULL;
		schema->table[n]->ref_by.n_deps = 0;
			
		// Get dependencies
		if(!get_dep_ons(schema->table[n]->OID, &(schema->table[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(schema->table[n]->OID, &(schema->table[n]->ref_by), conn)) return FALSE;
			
		// Get fields
		if(!get_table_fields(schema, schema->table[n], conn)) return FALSE;
		// Get indexes
		if(!get_table_indexes(schema, schema->table[n], conn)) return FALSE;		
		// Get checks
		if(!get_table_checks(schema, schema->table[n], conn)) return FALSE;
		// Get triggers
		if(!get_table_triggers(schema, schema->table[n], conn)) return FALSE;
		// Get rules
		if(!get_table_rules(schema, schema->table[n], conn)) return FALSE;
		
    
		schema->n_tables++;
 }
 
 PQclear(res);
 return TRUE;
} // get_tables ()

/*
 * Get views in schema
 */
gboolean
get_views(T_Schema * schema, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT viewname AS name, viewowner AS owner, description AS comment, class.oid AS oid, "
						           "						 definition AS body, class.relacl AS acl "
						           "FROM pg_views "
						           "					LEFT OUTER JOIN pg_class class ON pg_views.viewname = class.relname "
						           "					LEFT OUTER JOIN pg_description ON class.oid = pg_description.objoid "
						           "WHERE schemaname = '";
	gchar sql2[]="' ORDER BY viewname;";
	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "owner");
	guint col3 = PQfnumber(res, "comment");
	guint col4 = PQfnumber(res, "oid");
	guint col5 = PQfnumber(res, "body");
	guint col6 = PQfnumber(res, "acl");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = schema->n_views;
		
		// Create new schema struct
		if(schema->view == NULL){
			if((schema->view = (T_View **) malloc(sizeof(T_View*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((schema->view = (T_View **) realloc((schema->view), sizeof(T_View*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((schema->view[n] = (T_View*) malloc(sizeof(T_View))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		schema->view[n]->name = copy_string(PQgetvalue(res, row, col1));
		schema->view[n]->owner = copy_string(PQgetvalue(res, row, col2));
		schema->view[n]->comment = copy_string(PQgetvalue(res, row, col3));
		schema->view[n]->OID = atoi(PQgetvalue(res, row, col4));
		schema->view[n]->body = copy_string(PQgetvalue(res, row, col5));
		schema->view[n]->acl = copy_string(PQgetvalue(res, row, col6));
			
		// Init view objects
		schema->view[n]->column = NULL;     // fields
		schema->view[n]->n_columns = 0;
		schema->view[n]->rule = NULL;           // rules
		schema->view[n]->n_rules = 0;
		schema->view[n]->dep_on.dep = NULL;// dependencies
		schema->view[n]->dep_on.n_deps = 0;
		schema->view[n]->ref_by.dep = NULL;
		schema->view[n]->ref_by.n_deps = 0;
			
		// Get dependencies
		if(!get_dep_ons(schema->view[n]->OID, &(schema->view[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(schema->view[n]->OID, &(schema->view[n]->ref_by), conn)) return FALSE;
		
		// Get fields
		if(!get_view_fields(schema, schema->view[n], conn)) return FALSE;
		// Get rules
		if(!get_view_rules(schema, schema->view[n], conn)) return FALSE;
		
		schema->n_views++;
 }
 
 PQclear(res);
 return TRUE;
} // get_views()

/*
 * Get functions in schema
 */
gboolean
get_functions(T_Schema * schema, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT proname AS name,  rolname AS owner, description AS comment, pg_proc.oid as oid, "
						           "						 CASE WHEN proretset THEN 'true' ELSE 'false' END AS return_set, rettype.typname AS return_type, lan.lanname AS language, "
						           "						 CASE WHEN proisstrict THEN 'true' ELSE 'false' END AS strict, CASE WHEN prosecdef THEN 'Definer' ELSE 'Invoker' END AS execution_privilegies, "
						           "						 CASE WHEN provolatile='i' THEN 'Immutable' WHEN provolatile='s' THEN 'Stable' WHEN provolatile='v' THEN 'Volatile' END AS stability, "
						           "						 pronargs AS n_args, proargtypes, proallargtypes, proargmodes, proargnames, "
						           "						 prosrc AS definition, "
						           "						 proacl AS acl "
						           "FROM pg_proc "
						           "					LEFT OUTER JOIN pg_authid ON pg_proc.proowner = pg_authid.oid "
						           "					LEFT OUTER JOIN pg_type rettype ON pg_proc.prorettype = rettype.oid "
						           "					LEFT OUTER JOIN pg_language lan ON pg_proc.prolang = lan.oid "
						           "					LEFT OUTER JOIN pg_namespace ON pg_proc.pronamespace = pg_namespace.oid "
						           "					LEFT OUTER JOIN pg_description ON pg_proc.oid = pg_description.objoid "
						           "WHERE nspname = '";
	gchar sql2[]="' AND NOT proisagg ORDER BY name;";
	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "owner");
	guint col3 = PQfnumber(res, "comment");
	guint col4 = PQfnumber(res, "oid");
	guint col5 = PQfnumber(res, "return_set");
	guint col6 = PQfnumber(res, "return_type");
	guint col7 = PQfnumber(res, "language");
	guint col8 = PQfnumber(res, "strict");
	guint col9 = PQfnumber(res, "execution_privilegies");
	guint col10 = PQfnumber(res, "stability");
	guint col11 = PQfnumber(res, "proargtypes");
	guint col12 = PQfnumber(res, "proallargtypes");
	guint col13 = PQfnumber(res, "proargmodes");
	guint col14 = PQfnumber(res, "proargnames");
	guint col15 = PQfnumber(res, "definition");
	guint col16 = PQfnumber(res, "acl");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = schema->n_functions;
		
		// Create new schema struct
		if(schema->function == NULL){
			if((schema->function = (T_Function **) malloc(sizeof(T_Function*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((schema->function = (T_Function **) realloc((schema->function), sizeof(T_Function*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((schema->function[n] = (T_Function*) malloc(sizeof(T_Function))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		schema->function[n]->name = copy_string(PQgetvalue(res, row, col1));
		schema->function[n]->owner = copy_string(PQgetvalue(res, row, col2));
		schema->function[n]->comment = copy_string(PQgetvalue(res, row, col3));
		schema->function[n]->OID = atoi(PQgetvalue(res, row, col4));
		schema->function[n]->return_set = strcmp(PQgetvalue(res, row, col5), "true") == 0 ? TRUE : FALSE;
		schema->function[n]->return_type = copy_string(PQgetvalue(res, row, col6));
		schema->function[n]->language = copy_string(PQgetvalue(res, row, col7));
		schema->function[n]->strict = strcmp(PQgetvalue(res, row, col8), "true") == 0 ? TRUE : FALSE;
		schema->function[n]->execution_privilegies = copy_string(PQgetvalue(res, row, col9));
		schema->function[n]->stability = copy_string(PQgetvalue(res, row, col10));
		schema->function[n]->definition = copy_string(PQgetvalue(res, row, col15));
		schema->function[n]->acl = copy_string(PQgetvalue(res, row, col16));
		
		// Init function objects
		schema->function[n]->parameter = NULL;     // parameters
		schema->function[n]->n_parameters = 0;
		schema->function[n]->dep_on.dep = NULL;     // dependencies
		schema->function[n]->dep_on.n_deps = 0;
		schema->function[n]->ref_by.dep = NULL;
		schema->function[n]->ref_by.n_deps = 0;
		schema->function[n]->dep_on.dep = NULL;// dependencies
		schema->function[n]->dep_on.n_deps = 0;
		schema->function[n]->ref_by.dep = NULL;
		schema->function[n]->ref_by.n_deps = 0;
			
		// Get dependencies
		if(!get_dep_ons(schema->function[n]->OID, &(schema->function[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(schema->function[n]->OID, &(schema->function[n]->ref_by), conn)) return FALSE;
		
		// Get parameters
		if(!get_function_parameters(PQgetvalue(res, row, col11), PQgetvalue(res, row, col12), PQgetvalue(res, row, col13), PQgetvalue(res, row, col14), schema->function[n], conn)) return FALSE;
		
		int i, len=0;
		for(i=0; i < schema->function[n]->n_parameters; i++){
			if(strcmp(schema->function[n]->parameter[i]->scope, "Input") != 0) len += strlen(schema->function[n]->parameter[i]->scope) + 1;
			len += strlen(schema->function[n]->parameter[i]->name) + 1 + strlen(schema->function[n]->parameter[i]->type) + 2;
		}
		gchar * params = alloc_string(len);
		params[0]='\0';
		for(i=0; i < schema->function[n]->n_parameters; i++){
			if(strcmp(schema->function[n]->parameter[i]->scope, "Input") != 0){
			  strcat(params, schema->function[n]->parameter[i]->scope);
			  strcat(params, " ");
			}
			strcat(params, schema->function[n]->parameter[i]->name);
			strcat(params, " ");
			strcat(params, schema->function[n]->parameter[i]->type);
			if(i < (schema->function[n]->n_parameters-1)) strcat(params, ", ");
		}
		schema->function[n]->parameters = params;
		
		// Get dependencies
		if(!get_dep_ons(schema->function[n]->OID, &(schema->function[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(schema->function[n]->OID, &(schema->function[n]->ref_by), conn)) return FALSE;
						
		schema->n_functions++;
 }
 
 PQclear(res);
 return TRUE;
} // get_functions ()

/*
 * Get domains in schema
 */
gboolean
get_domains(T_Schema * schema, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT dom.typname AS name, rolname AS owner, description AS comment, dom.oid AS oid, "
						           "						 dtype.typname AS data_type, CASE WHEN dom.typnotnull THEN 'true' ELSE 'false' END AS not_null, dom.typdefault AS default_value "
						           "FROM pg_type dom "
						           "					LEFT OUTER JOIN pg_type dtype ON dom.typbasetype = dtype.oid "
						           "					LEFT OUTER JOIN pg_authid ON dom.typowner = pg_authid.oid "
						           "					LEFT OUTER JOIN pg_namespace nsp ON nsp.oid = dom.typnamespace "
						           "					LEFT OUTER JOIN pg_description ON dom.oid = pg_description.objoid "
						           "WHERE nsp.nspname='";
	gchar sql2[]="' AND dom.typtype = 'd' ORDER BY dom.typname;";
	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}

	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "owner");
	guint col3 = PQfnumber(res, "comment");
	guint col4 = PQfnumber(res, "oid");
	guint col5 = PQfnumber(res, "data_type");
	guint col6 = PQfnumber(res, "not_null");
	guint col7 = PQfnumber(res, "default_value");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = schema->n_domains;
		
		// Create new schema struct
		if(schema->domain == NULL){
			if((schema->domain = (T_Domain **) malloc(sizeof(T_Domain*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((schema->domain = (T_Domain **) realloc((schema->domain), sizeof(T_Domain*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((schema->domain[n] = (T_Domain*) malloc(sizeof(T_Domain))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		schema->domain[n]->name = copy_string(PQgetvalue(res, row, col1));
		schema->domain[n]->owner = copy_string(PQgetvalue(res, row, col2));
		schema->domain[n]->comment = copy_string(PQgetvalue(res, row, col3));
		schema->domain[n]->OID = atoi(PQgetvalue(res, row, col4));
		schema->domain[n]->data_type = copy_string(PQgetvalue(res, row, col5));
		schema->domain[n]->not_null = strcmp(PQgetvalue(res, row, col6), "true") == 0 ? TRUE : FALSE;
		schema->domain[n]->default_value = copy_string(PQgetvalue(res, row, col7));
			
		// Init view objects
		schema->domain[n]->check = NULL;     // checks
		schema->domain[n]->n_checks = 0;
		schema->domain[n]->dep_on.dep = NULL;// dependencies
		schema->domain[n]->dep_on.n_deps = 0;
		schema->domain[n]->ref_by.dep = NULL;
		schema->domain[n]->ref_by.n_deps = 0;
			
		// Get dependencies
		if(!get_dep_ons(schema->domain[n]->OID, &(schema->domain[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(schema->domain[n]->OID, &(schema->domain[n]->ref_by), conn)) return FALSE;
		
		// Get checks
		if(!get_domain_checks(schema, schema->domain[n], conn)) return FALSE;
		
		schema->n_domains++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_schemas ()

/*
 * Get aggregates in schema
 */
gboolean
get_aggregates(T_Schema * schema, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT agg.proname AS name,  rolname AS owner, description AS comment, agg.oid as oid, "
						           " 						 btype.typname AS base_type, stype.typname AS state_type, sfunc.proname AS state_function, "
						           "               ftype.typname AS final_type, ffunc.proname AS final_function, agginitval AS initial_condition "
						           "FROM pg_proc agg "
						           "					LEFT OUTER JOIN pg_authid ON agg.proowner = pg_authid.oid "
						           "					LEFT OUTER JOIN pg_type btype ON agg.proargtypes[0] = btype.oid "
						           "					LEFT OUTER JOIN pg_type ftype ON agg.prorettype = ftype.oid "
						           "					LEFT OUTER JOIN pg_aggregate ON agg.oid = pg_aggregate.aggfnoid "
						           "					LEFT OUTER JOIN pg_type stype ON aggtranstype = stype.oid "
						           "					LEFT OUTER JOIN pg_proc sfunc ON aggtransfn = sfunc.oid "
						           "					LEFT OUTER JOIN pg_proc ffunc ON aggfinalfn = ffunc.oid "
						           "					LEFT OUTER JOIN pg_namespace ON agg.pronamespace = pg_namespace.oid "
						           "					LEFT OUTER JOIN pg_description ON agg.oid = pg_description.objoid "
						           "WHERE nspname = '";
	gchar sql2[]="' AND agg.proisagg ORDER BY name;";
	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);

	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "owner");
	guint col3 = PQfnumber(res, "comment");
	guint col4 = PQfnumber(res, "oid");
	guint col5 = PQfnumber(res, "base_type");
	guint col6 = PQfnumber(res, "state_type");
	guint col7 = PQfnumber(res, "state_function");
	guint col8 = PQfnumber(res, "final_type");
	guint col9 = PQfnumber(res, "final_function");
	guint col10 = PQfnumber(res, "initial_condition");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = schema->n_aggregates;
		
		// Create new aggregate struct
		if(schema->aggregate == NULL){
			if((schema->aggregate = (T_Aggregate **) malloc(sizeof(T_Aggregate*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((schema->aggregate = (T_Aggregate **) realloc((schema->aggregate), sizeof(T_Aggregate*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((schema->aggregate[n] = (T_Aggregate*) malloc(sizeof(T_Aggregate))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		schema->aggregate[n]->name = copy_string(PQgetvalue(res, row, col1));
		schema->aggregate[n]->owner = copy_string(PQgetvalue(res, row, col2));
		schema->aggregate[n]->comment = copy_string(PQgetvalue(res, row, col3));
		schema->aggregate[n]->OID = atoi(PQgetvalue(res, row, col4));
		schema->aggregate[n]->base_type = copy_string(PQgetvalue(res, row, col5));
		schema->aggregate[n]->state_type = copy_string(PQgetvalue(res, row, col6));
		schema->aggregate[n]->state_function = copy_string(PQgetvalue(res, row, col7));
		schema->aggregate[n]->final_type = copy_string(PQgetvalue(res, row, col8));
		schema->aggregate[n]->final_function = copy_string(PQgetvalue(res, row, col9));
		schema->aggregate[n]->initial_condition = copy_string(PQgetvalue(res, row, col10));
		
		schema->aggregate[n]->dep_on.dep = NULL;// dependencies
		schema->aggregate[n]->dep_on.n_deps = 0;
		schema->aggregate[n]->ref_by.dep = NULL;
		schema->aggregate[n]->ref_by.n_deps = 0;
			
		// Get dependencies
		if(!get_dep_ons(schema->aggregate[n]->OID, &(schema->aggregate[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(schema->aggregate[n]->OID, &(schema->aggregate[n]->ref_by), conn)) return FALSE;
		
					
		schema->n_aggregates++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_aggregates ()

/*
 * Get sequences in schema
 */
gboolean
get_sequences(T_Schema * schema, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT relname AS name, rolname AS owner, description AS comment, pg_class.oid, "
						           " 					   relacl AS acl "
						           "FROM pg_class "
						           "					LEFT OUTER JOIN pg_authid ON pg_class.relowner = pg_authid.oid "
						           "					LEFT OUTER JOIN pg_namespace ON pg_class.relnamespace = pg_namespace.oid "
						           "					LEFT OUTER JOIN pg_description ON pg_class.oid = pg_description.objoid "
						           "WHERE nspname = '";
	gchar sql2[]="' AND relkind = 'S' ORDER BY name;";
	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "owner");
	guint col3 = PQfnumber(res, "comment");
	guint col4 = PQfnumber(res, "oid");
	guint col5 = PQfnumber(res, "acl");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = schema->n_sequences;
		
		// Create new sequence struct
		if(schema->sequence == NULL){
			if((schema->sequence = (T_Sequence **) malloc(sizeof(T_Sequence*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((schema->sequence = (T_Sequence **) realloc((schema->sequence), sizeof(T_Sequence*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((schema->sequence[n] = (T_Sequence*) malloc(sizeof(T_Sequence))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		schema->sequence[n]->name = copy_string(PQgetvalue(res, row, col1));
		schema->sequence[n]->owner = copy_string(PQgetvalue(res, row, col2));
		schema->sequence[n]->comment = copy_string(PQgetvalue(res, row, col3));
		schema->sequence[n]->OID = atoi(PQgetvalue(res, row, col4));
		schema->sequence[n]->acl = copy_string(PQgetvalue(res, row, col5));
		
		/*-------------- get aditional info --------------------*/
			// SQL query
			gchar sql3[]="SELECT last_value AS value,increment_by AS increment, max_value, min_value, cache_value AS cache , CASE WHEN is_cycled THEN 'true' ELSE 'false' END AS cycle from ";
			gchar sql4[]=";";
			guint sub_len = strlen(sql3)+strlen(schema->sequence[n]->name)+strlen(sql4);
			gchar * sub_sql;
			sub_sql = alloc_string(sub_len+1);
			strcpy(sub_sql, sql3); strcat(sub_sql, schema->sequence[n]->name); strcat(sub_sql, sql4);
	
			// Execution
			PGresult *sub_res = PQexec(conn, sub_sql);
			
			free(sub_sql);
			
			if (PQresultStatus(sub_res) != PGRES_TUPLES_OK){
	  		mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
     		                                                      GTK_BUTTONS_OK,  "Error during retrieving data.");
				gint resp = gtk_dialog_run(GTK_DIALOG(mess));
				if(resp == GTK_RESPONSE_OK);
				gtk_widget_destroy (mess);
				PQclear(sub_res);
				PQclear(res);
				return FALSE;	                                                           
			}
	
			// Determine column nubers
			guint sub_col1 = PQfnumber(sub_res, "value");
			guint sub_col2 = PQfnumber(sub_res, "increment");
			guint sub_col3 = PQfnumber(sub_res, "max_value");
			guint sub_col4 = PQfnumber(sub_res, "min_value");
			guint sub_col5 = PQfnumber(sub_res, "cache");
			guint sub_col6 = PQfnumber(sub_res, "cycle");
			schema->sequence[n]->value = copy_string(PQgetvalue(sub_res, 0, sub_col1));
			schema->sequence[n]->increment = copy_string(PQgetvalue(sub_res, 0, sub_col2));
			schema->sequence[n]->max_value = copy_string(PQgetvalue(sub_res, 0, sub_col3));
			schema->sequence[n]->min_value = copy_string(PQgetvalue(sub_res, 0, sub_col4));
			schema->sequence[n]->cache = copy_string(PQgetvalue(sub_res, 0, sub_col5));
			schema->sequence[n]->cycle = strcmp(PQgetvalue(sub_res, 0, sub_col6), "true") == 0 ? TRUE : FALSE;
			
			PQclear(sub_res);
		/*---------------------------------------------------------*/	
		
		schema->sequence[n]->dep_on.dep = NULL;// dependencies
		schema->sequence[n]->dep_on.n_deps = 0;
		schema->sequence[n]->ref_by.dep = NULL;
		schema->sequence[n]->ref_by.n_deps = 0;
			
		// Get dependencies
		if(!get_dep_ons(schema->sequence[n]->OID, &(schema->sequence[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(schema->sequence[n]->OID, &(schema->sequence[n]->ref_by), conn)) return FALSE;
		
		schema->n_sequences++;
 }
 
 PQclear(res);
 return TRUE; 	
} // get_sequences ()

/*
 * Get types in schema
 */
gboolean
get_types(T_Schema * schema, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT typ.typname AS name, rolname AS owner, description AS comment, typ.oid AS oid, "
						           "						 ifunc.proname AS input_function, ofunc.proname AS output_function, rfunc.proname AS receive_function, sfunc.proname AS send_function, afunc.proname AS analyze_function, "
							         "	  				 typ.typlen AS internal_length, typ.typdefault AS Default, elem.typname AS element, typ.typdelim AS delimiter, "
						           "						 CASE WHEN typ.typalign='c' THEN 'Char' WHEN typ.typalign='s' THEN 'Short' WHEN typ.typalign='i' THEN 'Int' WHEN typ.typalign='d' THEN 'Double' END AS alignment, "
						           "						 CASE WHEN typ.typbyval THEN 'true' ELSE 'false' END AS passed_by_value, "
						           "						 CASE WHEN typ.typstorage='p' THEN 'Plain' WHEN typ.typstorage='e' THEN 'Secondary relation' WHEN typ.typstorage='m' THEN 'Compressed inline' WHEN typ.typstorage='x' THEN 'Compressed inline or secondary relation' END AS storage "
						           "FROM pg_type typ "
						           "					LEFT OUTER JOIN pg_proc ifunc ON typ.typinput = ifunc.oid "
						           "					LEFT OUTER JOIN pg_proc ofunc ON typ.typoutput = ofunc.oid "
						           "					LEFT OUTER JOIN pg_proc rfunc ON typ.typreceive = rfunc.oid "
						           "					LEFT OUTER JOIN pg_proc sfunc ON typ.typsend = sfunc.oid "
						           "					LEFT OUTER JOIN pg_proc afunc ON typ.typanalyze = afunc.oid "
						           "					LEFT OUTER JOIN pg_type elem ON typ.typelem = elem.oid "
						           "					LEFT OUTER JOIN pg_authid ON typ.typowner = pg_authid.oid "
						           "					LEFT OUTER JOIN pg_namespace ON typ.typnamespace = pg_namespace.oid "
						           "					LEFT OUTER JOIN pg_description ON typ.oid = pg_description.objoid "
						           "WHERE typ.typtype='b' AND nspname='";
	gchar sql2[]="' ORDER BY typ.typname;";
	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "owner");
	guint col3 = PQfnumber(res, "comment");
	guint col4 = PQfnumber(res, "oid");
	guint col5 = PQfnumber(res, "input_function");
	guint col6 = PQfnumber(res, "output_function");
	guint col7 = PQfnumber(res, "receive_function");
	guint col8 = PQfnumber(res, "send_function");
	guint col9 = PQfnumber(res, "analyze_function");
	guint col10 = PQfnumber(res, "internal_length");
	guint col11 = PQfnumber(res, "Default");
	guint col12 = PQfnumber(res, "element");
	guint col13 = PQfnumber(res, "delimiter");
	guint col14 = PQfnumber(res, "alignment");
	guint col15 = PQfnumber(res, "passed_by_value");
	guint col16 = PQfnumber(res, "storage");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = schema->n_types;
		
		// Create new schema struct
		if(schema->type == NULL){
			if((schema->type = (T_Type **) malloc(sizeof(T_Type*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((schema->type = (T_Type **) realloc((schema->type), sizeof(T_Type*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((schema->type[n] = (T_Type*) malloc(sizeof(T_Type))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		schema->type[n]->name = copy_string(PQgetvalue(res, row, col1));
		schema->type[n]->owner = copy_string(PQgetvalue(res, row, col2));
		schema->type[n]->comment = copy_string(PQgetvalue(res, row, col3));
		schema->type[n]->OID = atoi(PQgetvalue(res, row, col4));
		schema->type[n]->input_function = copy_string(PQgetvalue(res, row, col5));
		schema->type[n]->output_function = copy_string(PQgetvalue(res, row, col6));
		schema->type[n]->receive_function = copy_string(PQgetvalue(res, row, col7));
		schema->type[n]->send_function = copy_string(PQgetvalue(res, row, col8));
		schema->type[n]->analyze_function = copy_string(PQgetvalue(res, row, col9));
		schema->type[n]->internal_length = copy_string(PQgetvalue(res, row, col10));
		schema->type[n]->Default = copy_string(PQgetvalue(res, row, col11));
		schema->type[n]->element = copy_string(PQgetvalue(res, row, col12));
		schema->type[n]->delimiter = copy_string(PQgetvalue(res, row, col13));
		schema->type[n]->alignment = copy_string(PQgetvalue(res, row, col14));
		schema->type[n]->passed_by_value = strcmp(PQgetvalue(res, row, col15), "true") == 0 ? TRUE : FALSE;
		schema->type[n]->storage = copy_string(PQgetvalue(res, row, col16));
		
		schema->type[n]->dep_on.dep = NULL;// dependencies
		schema->type[n]->dep_on.n_deps = 0;
		schema->type[n]->ref_by.dep = NULL;
		schema->type[n]->ref_by.n_deps = 0;
			
		// Get dependencies
		if(!get_dep_ons(schema->type[n]->OID, &(schema->type[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(schema->type[n]->OID, &(schema->type[n]->ref_by), conn)) return FALSE;
					
		schema->n_types++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_types ()

/*
 * Get operators in schema
 */
gboolean
get_operators(T_Schema * schema, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT oper.oprname AS name, rolname AS owner, description AS comment, oper.oid, "
						           "							CASE WHEN oper.oprkind='b' THEN 'Infix both' WHEN oper.oprkind='l' THEN 'Prefix left' WHEN oper.oprkind='r' THEN 'Postfix right' WHEN oper.oprkind='d' THEN 'Double' END AS kind, "
						           "							func.proname AS function, leftop.typname AS left_operand_type, rightop.typname AS right_operand_type, com.oprname AS commutator, neg.oprname AS negator, "
						           "							rfunc.proname AS restrict_function, jfunc.proname AS join_function, lsort.oprname AS left_sort_operator, rsort.oprname AS right_sort_operator, lcmp.oprname AS less_than_operator, rcmp.oprname AS grater_than_operator, "
						           "							CASE WHEN oper.oprcanhash THEN 'true' ELSE 'false' END AS supports_hash_joins "
						           "FROM pg_operator oper "
						           "					LEFT OUTER JOIN pg_proc func ON oper.oprcode = func.oid "
						           "					LEFT OUTER JOIN pg_type leftop ON oper.oprleft = leftop.oid "
						           "					LEFT OUTER JOIN pg_type rightop ON oper.oprright = rightop.oid "
						           "					LEFT OUTER JOIN pg_operator com ON oper.oprcom = com.oid "
						           "					LEFT OUTER JOIN pg_operator neg ON oper.oprnegate = com.oid "
						           "					LEFT OUTER JOIN pg_proc rfunc ON oper.oprrest = rfunc.oid "
						           "					LEFT OUTER JOIN pg_proc jfunc ON oper.oprjoin = jfunc.oid "
						           "					LEFT OUTER JOIN pg_operator lsort ON oper.oprlsortop = com.oid "
						           "					LEFT OUTER JOIN pg_operator rsort ON oper.oprrsortop = com.oid "
						           "					LEFT OUTER JOIN pg_operator lcmp ON oper.oprltcmpop = com.oid "
						           "					LEFT OUTER JOIN pg_operator rcmp ON oper.oprgtcmpop = com.oid "
						           "					LEFT OUTER JOIN pg_authid ON oper.oprowner = pg_authid.oid "
						           "					LEFT OUTER JOIN pg_namespace ON oper.oprnamespace = pg_namespace.oid "
						           "					LEFT OUTER JOIN pg_description ON oper.oid = pg_description.objoid "
						           "WHERE nspname='";
	gchar sql2[]="' ORDER BY oper.oprname;";
	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "owner");
	guint col3 = PQfnumber(res, "comment");
	guint col4 = PQfnumber(res, "oid");
	guint col5 = PQfnumber(res, "kind");
	guint col6 = PQfnumber(res, "function");
	guint col7 = PQfnumber(res, "left_operand_type");
	guint col8 = PQfnumber(res, "right_operand_type");
	guint col9 = PQfnumber(res, "commutator");
	guint col10 = PQfnumber(res, "negator");
	guint col11 = PQfnumber(res, "restrict_function");
	guint col12 = PQfnumber(res, "join_function");
	guint col13 = PQfnumber(res, "left_sort_operator");
	guint col14 = PQfnumber(res, "right_sort_operator");
	guint col15 = PQfnumber(res, "less_than_operator");
	guint col16 = PQfnumber(res, "grater_than_operator");
	guint col17 = PQfnumber(res, "supports_hash_joins");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = schema->n_operators;
		
		// Create new schema struct
		if(schema->operator == NULL){
			if((schema->operator = (T_Operator **) malloc(sizeof(T_Operator*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((schema->operator = (T_Operator **) realloc((schema->operator), sizeof(T_Operator*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((schema->operator[n] = (T_Operator*) malloc(sizeof(T_Operator))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		schema->operator[n]->name = copy_string(PQgetvalue(res, row, col1));
		schema->operator[n]->owner = copy_string(PQgetvalue(res, row, col2));
		schema->operator[n]->comment = copy_string(PQgetvalue(res, row, col3));
		schema->operator[n]->OID = atoi(PQgetvalue(res, row, col4));
		schema->operator[n]->kind = copy_string(PQgetvalue(res, row, col5));
		schema->operator[n]->function = copy_string(PQgetvalue(res, row, col6));
		schema->operator[n]->left_operand_type = copy_string(PQgetvalue(res, row, col7));
		schema->operator[n]->right_operand_type = copy_string(PQgetvalue(res, row, col8));
		schema->operator[n]->commutator = copy_string(PQgetvalue(res, row, col9));
		schema->operator[n]->negator = copy_string(PQgetvalue(res, row, col10));
		schema->operator[n]->restrict_function = copy_string(PQgetvalue(res, row, col11));
		schema->operator[n]->join_function = copy_string(PQgetvalue(res, row, col12));
		schema->operator[n]->left_sort_operator = copy_string(PQgetvalue(res, row, col13));
		schema->operator[n]->right_sort_operator = copy_string(PQgetvalue(res, row, col14));
		schema->operator[n]->less_than_operator = copy_string(PQgetvalue(res, row, col15));
		schema->operator[n]->grater_than_operator = copy_string(PQgetvalue(res, row, col16));
		schema->operator[n]->supports_hash_joins = strcmp(PQgetvalue(res,  row, col17), "true") == 0 ? TRUE : FALSE;
		
		schema->operator[n]->dep_on.dep = NULL;// dependencies
		schema->operator[n]->dep_on.n_deps = 0;
		schema->operator[n]->ref_by.dep = NULL;
		schema->operator[n]->ref_by.n_deps = 0;
			
		// Get dependencies
		if(!get_dep_ons(schema->operator[n]->OID, &(schema->operator[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(schema->operator[n]->OID, &(schema->operator[n]->ref_by), conn)) return FALSE;
								
		schema->n_operators++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_operators ()

//////////////////////////////////////////////// Table objects /////////////////////////////////////////////////////////////////////
/*
 * Get fields in table
 */
gboolean 
get_table_fields(T_Schema * schema, T_Table * table, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT fields.attname AS name, description AS comment, "
												"						 CASE WHEN cols.udt_name LIKE '\\\\_%' THEN substring(cols.udt_name from '_#\"%#\"' for '#') ELSE cols.udt_name END AS type, "
												"              CASE WHEN cols.udt_name LIKE '\\\\_%' THEN 'true' ELSE 'false' END AS is_array, "
												"              cols.character_maximum_length AS size, cols.numeric_precision AS precision, "
												"						 CASE WHEN cons_p.contype ='p' THEN 'true' ELSE 'false' END AS pkey, CASE WHEN cons_u.contype ='u' THEN 'true' ELSE 'false' END AS unique, "
												"              CASE WHEN attnotnull THEN 'true' ELSE 'false' END AS not_null, "
												"						 fieldef.adsrc AS default_value, fields.attnum AS column_number, fields.attndims AS dimensions, "
												"              CASE WHEN fields.attinhcount > 0 THEN 'true' ELSE 'false' END AS inherited "
												"FROM pg_attribute fields "
												"					LEFT OUTER JOIN pg_class tab ON tab.oid = fields.attrelid "
												"					LEFT OUTER JOIN pg_attrdef fieldef ON fields.attrelid = fieldef.adrelid AND fields.attnum = fieldef.adnum "
												"					LEFT OUTER JOIN pg_constraint cons_p ON tab.oid = cons_p.conrelid AND fields.attnum = ANY (cons_p.conkey) AND cons_p.contype = 'p' "
												"					LEFT OUTER JOIN pg_constraint cons_u ON tab.oid = cons_u.conrelid AND fields.attnum = ANY (cons_u.conkey) AND cons_u.contype = 'u' "
												"					LEFT OUTER JOIN pg_namespace ON tab.relnamespace = pg_namespace.oid "
												"					LEFT OUTER JOIN information_schema.columns cols ON nspname = cols.table_schema AND tab.relname = cols.table_name AND fields.attname = cols.column_name "
												"					LEFT OUTER JOIN pg_description ON tab.oid = pg_description.objoid AND pg_description.objsubid = fields.attnum "
												"WHERE nspname='";
	gchar sql2[]="' AND tab.relname='";
	gchar sql3[]="' AND fields.attnum > 0 AND attisdropped = FALSE ORDER BY fields.attnum;";

	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2)+strlen(table->name)+strlen(sql3);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2); strcat(sql, table->name); strcat(sql, sql3);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "comment");
	guint col3 = PQfnumber(res, "type");
	guint col4 = PQfnumber(res, "size");
	guint col5 = PQfnumber(res, "precision");
	guint col6 = PQfnumber(res, "pkey");
	guint col7 = PQfnumber(res, "unique");
	guint col8 = PQfnumber(res, "not_null");
	guint col9 = PQfnumber(res, "default_value");
	guint col10 = PQfnumber(res, "column_number");
	guint col11 = PQfnumber(res, "is_array");
	guint col12 = PQfnumber(res, "dimensions");
	guint col13 = PQfnumber(res, "inherited");
	
	// Get values	
	guint row;
	T_Data_types * dtypes = get_data_types(conn);
	
	for(row = 0; row < PQntuples(res); row++){
		guint n = table->n_columns;
		
		// Create new table struct
		if(table->column == NULL){
			if((table->column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((table->column = (T_Column **) realloc((table->column), sizeof(T_Column*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((table->column[n] = (T_Column*) malloc(sizeof(T_Column))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		table->column[n]->name = copy_string(PQgetvalue(res, row, col1));
		table->column[n]->comment = copy_string(PQgetvalue(res, row, col2));
		gchar * name = copy_string(PQgetvalue(res, row, col3));
		table->column[n]->type = (strcmp(PQgetvalue(res, row, col11), "true") == 0) ? append_string(name, "[]") : name;
		if(dtypes != NULL){
			gint id=0;
			for(int i = 0; i < dtypes->n_data_types; i++){
				if(strcmp(dtypes->name[i], table->column[n]->type) == 0){
					id = i;
					break;
				}
			}
			table->column[n]->resizeable = dtypes->varlen[id];
		}
		else table->column[n]->resizeable = FALSE;
		
		table->column[n]->size = copy_string(PQgetvalue(res, row, col4));
		table->column[n]->precision = copy_string(PQgetvalue(res, row, col5));
		table->column[n]->pkey = strcmp(PQgetvalue(res,  row, col6), "true") == 0 ? TRUE : FALSE;
		table->column[n]->unique = strcmp(PQgetvalue(res,  row, col7), "true") == 0 ? TRUE : FALSE;
		table->column[n]->not_null = strcmp(PQgetvalue(res,  row, col8), "true") == 0 ? TRUE : FALSE;
		table->column[n]->default_value = copy_string(PQgetvalue(res, row, col9));
		table->column[n]->column_number = atoi(PQgetvalue(res, row, col10));
		table->column[n]->dimensions = atoi(PQgetvalue(res, row, col12));
		table->column[n]->inherited = strcmp(PQgetvalue(res,  row, col13), "true") == 0 ? TRUE : FALSE;
		
		table->n_columns++;
 }
 
 if(dtypes->varlen != NULL) free(dtypes->varlen);
 if(dtypes != NULL) free(dtypes);
 
 PQclear(res);
 return TRUE; 
} // get_table_fields ()

/*
 * Get indexes in table
 */
gboolean
get_table_indexes(T_Schema * schema, T_Table * table, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT  ind.relname AS name, description AS comment, ind.oid AS oid, "
												"							indkey AS columns, "
												"							CASE WHEN indisprimary THEN 'true' ELSE 'false' END AS pkey_constraint, "
												"							CASE WHEN con.contype ='u' THEN 'true' ELSE 'false' END AS unique_constraint, "
												"							CASE WHEN (indisprimary OR con.contype='u' OR indisunique=false) THEN 'false' ELSE 'true' END AS unique, "
												"							CASE WHEN indisclustered THEN 'true' ELSE 'false' END AS clustered, "
												"							am.amname AS access_method, "
												"							spcname AS tablespace, "
												"							indpred AS predicate "
												"FROM pg_index "
												"					LEFT OUTER JOIN pg_class ind ON pg_index.indexrelid = ind.oid "
												"					LEFT OUTER JOIN pg_class tab ON pg_index.indrelid = tab.oid "
												"					LEFT OUTER JOIN pg_namespace ON tab.relnamespace = pg_namespace.oid "
												"					LEFT OUTER JOIN pg_tablespace ON ind.reltablespace = pg_tablespace.oid "
												"           LEFT OUTER JOIN pg_constraint con ON indrelid = con.conrelid AND ind.relname = con.conname "
												"					LEFT OUTER JOIN pg_am am ON ind.relam = am.oid "
												"					LEFT OUTER JOIN pg_description ON ind.oid = pg_description.objoid "
												"WHERE nspname = '";
	gchar sql2[]="' AND  tab.relname = '";
	gchar sql3[]="' ORDER BY ind.relname;";

	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2)+strlen(table->name)+strlen(sql3);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2); strcat(sql, table->name); strcat(sql, sql3);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "comment");
	guint col3 = PQfnumber(res, "oid");
	guint col4 = PQfnumber(res, "columns");
	guint col5 = PQfnumber(res, "pkey_constraint");
	guint col6 = PQfnumber(res, "unique_constraint");
	guint col7 = PQfnumber(res, "unique");
	guint col8 = PQfnumber(res, "clustered");
	guint col9 = PQfnumber(res, "access_method");
	guint col10 = PQfnumber(res, "tablespace");
	guint col11 = PQfnumber(res, "predicate");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = table->n_indexes;
		
		// Create new index struct
		if(table->index == NULL){
			if((table->index = (T_Index **) malloc(sizeof(T_Index*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((table->index = (T_Index **) realloc((table->index), sizeof(T_Index*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((table->index[n] = (T_Index*) malloc(sizeof(T_Index))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		table->index[n]->name = copy_string(PQgetvalue(res, row, col1));
		table->index[n]->comment = copy_string(PQgetvalue(res, row, col2));
		table->index[n]->OID = atoi(PQgetvalue(res, row, col3));
		
		// Getting columns
		table->index[n]->n_columns = 0;
		table->index[n]->column = NULL;
				
		gchar * columns = copy_string(PQgetvalue(res, row, col4));
		
		guint l = strlen(columns);
		gboolean num = FALSE;
		gchar temp[l+1];
		temp[0]='\0';
		gint j=0;
		for(int i=0; i < l; i++){
			if(columns[i] >= '0' && columns[i] <= '9' && num){
				temp[j] = columns[i];
				j++;				
			}
			else if(((columns[i] >= '0' && columns[i] <= '9') || columns[i] == '-') && !num){
				num = TRUE;
				j=0;
				temp[j] = columns[i];
				j++;				
			}
			else if(num){
				temp[j] = '\0'; // end number
				gint number = atoi(temp);
				for(int k = 0; k < table->n_columns; k++){
					if(number == table->column[k]->column_number){
						gint m = table->index[n]->n_columns;
						
						if(table->index[n]->column == NULL){
							if((table->index[n]->column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  						 fprintf(stderr, "Memory allocation error.");
	   						 exit(0);
	   						}
						}
						else{
		 						if((table->index[n]->column = (T_Column **) realloc((table->index[n]->column), sizeof(T_Column*)*(m+1))) == NULL){
		    					fprintf(stderr, "Memory allocation error.");
	  	 						exit(0);
	 							}
						}
						table->index[n]->column[m] = table->column[k];
						table->index[n]->n_columns++;
						break;
					}
				}
				num=FALSE;
			}
			
		}
		if(num){
			temp[j] = '\0'; // end number
			gint number = atoi(temp);
			for(int k = 0; k < table->n_columns; k++){
				if(number == table->column[k]->column_number){
					gint m = table->index[n]->n_columns;
					
					if(table->index[n]->column == NULL){
						if((table->index[n]->column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  					 fprintf(stderr, "Memory allocation error.");
	   					 exit(0);
	   					}
					}
					else{
		 					if((table->index[n]->column = (T_Column **) realloc((table->index[n]->column), sizeof(T_Column*)*(m+1))) == NULL){
		  					fprintf(stderr, "Memory allocation error.");
	  						exit(0);
	 						}
					}
					table->index[n]->column[m] = table->column[k];
					table->index[n]->n_columns++;
					break;
				}
			}
		}
		free(columns);
		
		table->index[n]->pkey_constraint = strcmp(PQgetvalue(res, row, col5), "true") == 0 ? TRUE : FALSE;
		table->index[n]->unique_constraint = strcmp(PQgetvalue(res, row, col6), "true") == 0 ? TRUE : FALSE;
		table->index[n]->unique_index = strcmp(PQgetvalue(res, row, col7), "true") == 0 ? TRUE : FALSE;
		table->index[n]->clustered = strcmp(PQgetvalue(res, row, col8), "true") == 0 ? TRUE : FALSE;
		table->index[n]->access_method = copy_string(PQgetvalue(res, row, col9));
		table->index[n]->tablespace = copy_string(PQgetvalue(res, row, col10));
		table->index[n]->predicate = copy_string(PQgetvalue(res, row, col11));
		
		table->n_indexes++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_table_indexes ()

/*
 * Get foreign keys in table
 */
gboolean
get_table_foreign_keys(T_Profile * profile, T_Schema * schema, T_Table * table, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT con.conname AS name, description AS comment, con.oid AS oid, "
												"						 con.conkey AS columns, "
												"						 ref_nsp.nspname AS ref_schema, ref_tab.relname AS ref_table, con.confkey AS ref_columns, "
												"						 CASE WHEN con.confupdtype='a' THEN 'No Action' WHEN con.confupdtype='r' THEN 'Restrict' WHEN con.confupdtype='c' THEN 'Cascade' WHEN con.confupdtype='n' THEN 'Set Null' WHEN con.confupdtype='d' THEN 'Set Default' ELSE 'Unknown' END AS on_update_rule, "
												"						 CASE WHEN con.confdeltype='a' THEN 'No Action' WHEN con.confdeltype='r' THEN 'Restrict' WHEN con.confdeltype='c' THEN 'Cascade' WHEN con.confdeltype='n' THEN 'Set Null' WHEN con.confdeltype='d' THEN 'Set Default' ELSE 'Unknown' END AS on_delete_rule, "
												"						 CASE WHEN con.confmatchtype='u' THEN 'Simple' WHEN con.confmatchtype='f' THEN 'Full' ELSE 'Unknown' END AS match_type, "
												"						 CASE WHEN con.condeferrable THEN 'true' ELSE 'false' END AS deferrable, "
												" 						 CASE WHEN con.condeferred THEN 'At the end of the transaction' ELSE 'After each statement' END AS check_time "
												"FROM pg_constraint con "
												"					LEFT OUTER JOIN pg_class tab ON con.conrelid = tab.oid "
												"					LEFT OUTER JOIN pg_namespace nsp ON tab.relnamespace = nsp.oid "
												"					LEFT OUTER JOIN pg_class ref_tab ON con.confrelid = ref_tab.oid "
												"					LEFT OUTER JOIN pg_namespace ref_nsp ON ref_tab.relnamespace = ref_nsp.oid "
												"					LEFT OUTER JOIN pg_description ON con.oid = pg_description.objoid "
												"WHERE con.contype = 'f' AND nsp.nspname = '";
	gchar sql2[]="' AND tab.relname = '";
	gchar sql3[]="' ORDER BY name;";

	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2)+strlen(table->name)+strlen(sql3);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2); strcat(sql, table->name); strcat(sql, sql3);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "comment");
	guint col3 = PQfnumber(res, "oid");
	guint col4 = PQfnumber(res, "columns");
	guint col5 = PQfnumber(res, "ref_schema");
	guint col6 = PQfnumber(res, "ref_table");
	guint col7 = PQfnumber(res, "ref_columns");
	guint col8 = PQfnumber(res, "on_update_rule");
	guint col9 = PQfnumber(res, "on_delete_rule");
	guint col10 = PQfnumber(res, "match_type");
	guint col11 = PQfnumber(res, "deferrable");
	guint col12 = PQfnumber(res, "check_time");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = table->n_fkeys;
		
		// Create new table struct
		if(table->fkey == NULL){
			if((table->fkey = (T_FKey **) malloc(sizeof(T_FKey*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((table->fkey = (T_FKey **) realloc((table->fkey), sizeof(T_FKey*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((table->fkey[n] = (T_FKey*) malloc(sizeof(T_FKey))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		table->fkey[n]->name = copy_string(PQgetvalue(res, row, col1));
		table->fkey[n]->comment = copy_string(PQgetvalue(res, row, col2));
		table->fkey[n]->OID = atoi(PQgetvalue(res, row, col3));
		
		// Getting columns
		table->fkey[n]->n_columns = 0;
		table->fkey[n]->column = NULL;
				
		gchar * columns = copy_string(PQgetvalue(res, row, col4));
		
		guint l = strlen(columns);
		gboolean num = FALSE;
		gchar temp[l+1];
		temp[0]='\0';
		gint j=0;
		for(int i=0; i < l; i++){
			if(columns[i] >= '0' && columns[i] <= '9' && num){
				temp[j] = columns[i];
				j++;				
			}
			else if(((columns[i] >= '0' && columns[i] <= '9') || columns[i] == '-') && !num){
				num = TRUE;
				j=0;
				temp[j] = columns[i];
				j++;				
			}
			else if(num){
				temp[j] = '\0'; // end number
				gint number = atoi(temp);
				for(int k = 0; k < table->n_columns; k++){
					if(number == table->column[k]->column_number){
						gint m = table->fkey[n]->n_columns;
						
						if(table->fkey[n]->column == NULL){
							if((table->fkey[n]->column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  						 fprintf(stderr, "Memory allocation error.");
	   						 exit(0);
	   						}
						}
						else{
		 						if((table->fkey[n]->column = (T_Column **) realloc((table->fkey[n]->column), sizeof(T_Column*)*(m+1))) == NULL){
		    					fprintf(stderr, "Memory allocation error.");
	  	 						exit(0);
	 							}
						}
						table->fkey[n]->column[m] = table->column[k];
						table->fkey[n]->n_columns++;
						break;
					}
				}
				num=FALSE;
			}
			
		}
		if(num){
			temp[j] = '\0'; // end number
			gint number = atoi(temp);
			for(int k = 0; k < table->n_columns; k++){
				if(number == table->column[k]->column_number){
					gint m = table->fkey[n]->n_columns;
					
					if(table->fkey[n]->column == NULL){
						if((table->fkey[n]->column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  					 fprintf(stderr, "Memory allocation error.");
	   					 exit(0);
	   					}
					}
					else{
		 					if((table->fkey[n]->column = (T_Column **) realloc((table->fkey[n]->column), sizeof(T_Column*)*(m+1))) == NULL){
		  					fprintf(stderr, "Memory allocation error.");
	  						exit(0);
	 						}
					}
					table->fkey[n]->column[m] = table->column[k];
					table->fkey[n]->n_columns++;
					break;
				}
			}
		}
		free(columns);
		
		table->fkey[n]->ref_schema = copy_string(PQgetvalue(res, row, col5));
		table->fkey[n]->ref_table = copy_string(PQgetvalue(res, row, col6));
		
		T_Table * ref_table = NULL;
		
		for(int i = 0; i < profile->database.n_schemas; i++){
			if(strcmp(profile->database.schema[i]->name, table->fkey[n]->ref_schema) == 0){
				for(int j = 0; j < profile->database.schema[i]->n_tables; j++){
					if(strcmp(profile->database.schema[i]->table[j]->name, table->fkey[n]->ref_table) == 0){
						ref_table =  profile->database.schema[i]->table[j];
						break;
					}
				}
				break;
			}
		}
		
		if(ref_table == NULL) return FALSE;
		
		// Getting referenced columns
		
		table->fkey[n]->n_ref_columns = 0;
		table->fkey[n]->ref_column = NULL;
				
		gchar * ref_columns = copy_string(PQgetvalue(res, row, col7));
		
		l = strlen(ref_columns);
		gboolean ref_num = FALSE;
		gchar ref_temp[l+1];
		ref_temp[0]='\0';
		j=0;
		for(int i=0; i < l; i++){
			if(ref_columns[i] >= '0' && ref_columns[i] <= '9' && ref_num){
				ref_temp[j] = ref_columns[i];
				j++;				
			}
			else if(((ref_columns[i] >= '0' && ref_columns[i] <= '9') || ref_columns[i] == '-') && !ref_num){
				ref_num = TRUE;
				j=0;
				ref_temp[j] = ref_columns[i];
				j++;				
			}
			else if(ref_num){
				ref_temp[j] = '\0'; // end number
				gint number = atoi(ref_temp);
				for(int k = 0; k < ref_table->n_columns; k++){
					if(number == ref_table->column[k]->column_number){
						gint m = table->fkey[n]->n_ref_columns;
						
						if(table->fkey[n]->ref_column == NULL){
							if((table->fkey[n]->ref_column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  						 fprintf(stderr, "Memory allocation error.");
	   						 exit(0);
	   						}
						}
						else{
		 						if((table->fkey[n]->ref_column = (T_Column **) realloc((table->fkey[n]->ref_column), sizeof(T_Column*)*(m+1))) == NULL){
		    					fprintf(stderr, "Memory allocation error.");
	  	 						exit(0);
	 							}
						}
						table->fkey[n]->ref_column[m] = ref_table->column[k];
						table->fkey[n]->n_ref_columns++;
						break;
					}
				}
				ref_num=FALSE;
			}
			
		}
		if(ref_num){
			ref_temp[j] = '\0'; // end number
			gint number = atoi(ref_temp);
			for(int k = 0; k < ref_table->n_columns; k++){
				if(number == ref_table->column[k]->column_number){
					gint m = table->fkey[n]->n_ref_columns;
					
					if(table->fkey[n]->ref_column == NULL){
						if((table->fkey[n]->ref_column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  					 fprintf(stderr, "Memory allocation error.");
	   					 exit(0);
	   					}
					}
					else{
		 					if((table->fkey[n]->ref_column = (T_Column **) realloc((table->fkey[n]->ref_column), sizeof(T_Column*)*(m+1))) == NULL){
		  					fprintf(stderr, "Memory allocation error.");
	  						exit(0);
	 						}
					}
					table->fkey[n]->ref_column[m] = ref_table->column[k];
					table->fkey[n]->n_ref_columns++;
					break;
				}
			}
		}
		free(ref_columns);
		
		
		table->fkey[n]->on_update_rule = copy_string(PQgetvalue(res, row, col8));
		table->fkey[n]->on_delete_rule = copy_string(PQgetvalue(res, row, col9));		
		table->fkey[n]->match_type = copy_string(PQgetvalue(res, row, col10));
		table->fkey[n]->deferrable = strcmp(PQgetvalue(res, row, col11), "true") == 0 ? TRUE : FALSE;
		table->fkey[n]->check_time = copy_string(PQgetvalue(res, row, col12));
	
		table->n_fkeys++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_table_foreign_keys ()

/*
 * Get checks in table
 */
gboolean
get_table_checks(T_Schema * schema, T_Table * table, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT conname AS name, description AS comment, pg_constraint.oid, "
												"						 consrc AS condition "
												"FROM pg_constraint "
												"					LEFT OUTER JOIN pg_class ON conrelid = pg_class.oid "
												"					LEFT OUTER JOIN pg_namespace ON connamespace = pg_namespace.oid "
												"					LEFT OUTER JOIN pg_description ON pg_constraint.oid = pg_description.objoid "
												"WHERE contype='c' AND nspname = '";
	gchar sql2[]="' AND relname='";
	gchar sql3[]="' ORDER BY name;";

	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2)+strlen(table->name)+strlen(sql3);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2); strcat(sql, table->name); strcat(sql, sql3);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "comment");
	guint col3 = PQfnumber(res, "oid");
	guint col4 = PQfnumber(res, "condition");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = table->n_checks;
		
		// Create new table struct
		if(table->check == NULL){
			if((table->check = (T_Check **) malloc(sizeof(T_Check*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((table->check = (T_Check **) realloc((table->check), sizeof(T_Check*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((table->check[n] = (T_Check*) malloc(sizeof(T_Check))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		table->check[n]->name = copy_string(PQgetvalue(res, row, col1));
		table->check[n]->comment = copy_string(PQgetvalue(res, row, col2));
		table->check[n]->OID = atoi(PQgetvalue(res, row, col3));
		table->check[n]->condition = copy_string(PQgetvalue(res, row, col4));
				
		table->n_checks++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_table_checks ()

/*
 * Get triggers in table
 */
gboolean
get_table_triggers(T_Schema * schema, T_Table * table, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT tgname AS name, description AS comment, pg_trigger.oid AS oid, "
						           "						 CASE WHEN tgenabled THEN 'true' ELSE 'false' END AS enabled, "
						           "						 CASE WHEN instg.condition_timing='BEFORE' THEN 'Before' ELSE 'After' END AS type, "
						           "						 CASE WHEN instg.action_orientation='ROW' THEN 'Row' ELSE 'Statement' END AS for_each, "
						           "						 CASE WHEN instg.event_manipulation='INSERT' THEN 'Insert' ELSE '' END AS insert_event, "
						           "						 CASE WHEN deltg.event_manipulation='DELETE' THEN 'Delete' ELSE '' END AS delete_event, "
						           "						 CASE WHEN updtg.event_manipulation='UPDATE' THEN 'Update' ELSE '' END AS update_event, "
						           "						 proname AS function, tgnargs, tgargs AS args, pnsp.nspname AS func_nsp "
						           "FROM pg_trigger "
						           "					LEFT OUTER JOIN (SELECT * FROM information_schema.triggers WHERE event_manipulation='INSERT') as instg ON tgname = instg.trigger_name "
						           "					LEFT OUTER JOIN (SELECT * FROM information_schema.triggers WHERE event_manipulation='DELETE') as deltg ON tgname = deltg.trigger_name "
						           "					LEFT OUTER JOIN (SELECT * FROM information_schema.triggers WHERE event_manipulation='UPDATE') as updtg ON tgname = updtg.trigger_name "
						           "					LEFT OUTER JOIN pg_class class ON pg_trigger.tgrelid = class.oid "
						           "					LEFT OUTER JOIN pg_namespace nsp ON class.relnamespace = nsp.oid "
						           "					LEFT OUTER JOIN pg_proc ON pg_trigger.tgfoid = pg_proc.oid "
						           "          LEFT OUTER JOIN pg_namespace pnsp ON pronamespace = pnsp.oid "
						           "					LEFT OUTER JOIN pg_description ON pg_trigger.oid = pg_description.objoid "
						           "WHERE nsp.nspname='";
	gchar sql2[]="' AND class.relname ='";
	gchar sql3[]="' AND NOT tgisconstraint ORDER BY name;";

	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2)+strlen(table->name)+strlen(sql3);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2); strcat(sql, table->name); strcat(sql, sql3);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "comment");
	guint col3 = PQfnumber(res, "oid");
	guint col4 = PQfnumber(res, "enabled");
	guint col5 = PQfnumber(res, "type");
	guint col6 = PQfnumber(res, "for_each");
	guint col7 = PQfnumber(res, "insert_event");
	guint col8 = PQfnumber(res, "delete_event");
	guint col9 = PQfnumber(res, "update_event");
	guint col10 = PQfnumber(res, "function");
	guint col11 = PQfnumber(res, "args");		
	guint col12 = PQfnumber(res, "func_nsp");		
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = table->n_triggers;
		
		// Create new schema struct
		if(table->trigger == NULL){
			if((table->trigger = (T_Trigger **) malloc(sizeof(T_Trigger*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((table->trigger = (T_Trigger **) realloc((table->trigger), sizeof(T_Trigger*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((table->trigger[n] = (T_Trigger*) malloc(sizeof(T_Trigger))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}

		table->trigger[n]->name = copy_string(PQgetvalue(res, row, col1));
		table->trigger[n]->comment = copy_string(PQgetvalue(res, row, col2));
		table->trigger[n]->OID = atoi(PQgetvalue(res, row, col3));
		table->trigger[n]->enabled = strcmp(PQgetvalue(res,  row, col4), "true") == 0 ? TRUE : FALSE;
		table->trigger[n]->type = copy_string(PQgetvalue(res, row, col5));
		table->trigger[n]->for_each = copy_string(PQgetvalue(res, row, col6));
		
		gchar * input = copy_string(PQgetvalue(res, row, col7));
		gchar * delete = copy_string(PQgetvalue(res, row, col8));
		gchar * update = copy_string(PQgetvalue(res, row, col9));
		gchar events[26];
		events[0]='\0';
		gboolean prev=FALSE;
		if(strcmp(input, "") != 0){
		  strcat(events, input);
		  prev=TRUE;
		}
		if(strcmp(delete, "") != 0){
			if(prev) strcat(events, " OR ");
			strcat(events, delete);
			prev=TRUE;
		}
		if(strcmp(update, "") != 0){
			if(prev) strcat(events, " OR ");
			strcat(events, update);
		}
		table->trigger[n]->events = copy_string(events);
		
		table->trigger[n]->function = append_string(NULL, PQgetvalue(res, row, col12));
		table->trigger[n]->function = append_string(table->trigger[n]->function, ".");
		table->trigger[n]->function = append_string(table->trigger[n]->function, PQgetvalue(res, row, col10));
		
		// parse \000 in arguments list
		gchar * args = copy_string(PQgetvalue(res, row, col11));
		gint h = 0;
		gint s = 0;
		gint p = strlen(args);
		table->trigger[n]->arguments = NULL;
		while(h < (p-3)){
			if(args[h] == '\\' && args[h+1] == '0' && args[h+2] == '0' && args[h+3] == '0'){
				args[h] = '\0';
				table->trigger[n]->arguments = append_string(table->trigger[n]->arguments, args+s);
				args[h] = '\\';
				h = h+4;
				if(h < p) table->trigger[n]->arguments = append_string(table->trigger[n]->arguments, ", ");
				s = h;
			}
			else h++;
		}
		free(args);
		//table->trigger[n]->arguments = copy_string(PQgetvalue(res, row, col11));
		table->trigger[n]->table_name = copy_string(table->name);
		
		// Add trigger to global list
		guint m = schema->n_triggers;
		
		// Create new schema struct
		if(schema->trigger == NULL){
			if((schema->trigger = (T_Trigger **) malloc(sizeof(T_Trigger*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((schema->trigger = (T_Trigger **) realloc((schema->trigger), sizeof(T_Trigger*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
		schema->trigger[m] = table->trigger[n];
		schema->n_triggers++;
		
		table->trigger[n]->dep_on.dep = NULL;// dependencies
		table->trigger[n]->dep_on.n_deps = 0;
		table->trigger[n]->ref_by.dep = NULL;
		table->trigger[n]->ref_by.n_deps = 0;
			
		// Get dependencies
		if(!get_dep_ons(table->trigger[n]->OID, &(table->trigger[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(table->trigger[n]->OID, &(table->trigger[n]->ref_by), conn)) return FALSE;
		
		
		table->n_triggers++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_table_triggers ()

/*
 * Get rules in table
 */
gboolean
get_table_rules(T_Schema * schema, T_Table * table, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT rul.rulename AS name, description AS comment, rew.oid AS oid, "
						           "						 CASE WHEN rew.ev_type='1' THEN 'Select' WHEN rew.ev_type='2' THEN 'Update' WHEN rew.ev_type='3' THEN 'Insert' WHEN rew.ev_type='4' THEN 'Delete' END AS event, "
						           "						 CASE WHEN rew.is_instead THEN 'true' ELSE 'false' END AS instead, CASE WHEN rew.is_instead THEN substring(pg_get_ruledef(rew.oid, true) from '%INSTEAD #\"%#\";' for '#') ELSE substring(pg_get_ruledef(rew.oid, true) from '%DO #\"%#\";' for '#')END AS definition, "
						           "						 substring(pg_get_ruledef(rew.oid, true) from '%WHERE #\"%#\" DO%' for '#') AS condition "
						           "FROM pg_rules rul "
						           "					LEFT OUTER JOIN pg_rewrite rew ON rul.rulename = rew.rulename "
						           "          LEFT OUTER JOIN pg_class rew_tab ON rew.ev_class = rew_tab.oid "
						           "					LEFT OUTER JOIN pg_description ON rew.oid = pg_description.objoid "
						           "WHERE rul.schemaname='";
	gchar sql2[]="' AND rul.tablename = '";
	gchar sql4[]="' AND rew_tab.relname = '";
	gchar sql3[]="' ORDER BY name;";

	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2)+strlen(table->name)+strlen(sql4)+strlen(table->name)+strlen(sql3);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2); strcat(sql, table->name); strcat(sql, sql4); strcat(sql, table->name); strcat(sql, sql3);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "comment");
	guint col3 = PQfnumber(res, "oid");
	guint col4 = PQfnumber(res, "event");
	guint col5 = PQfnumber(res, "instead");
	guint col6 = PQfnumber(res, "condition");
	guint col7 = PQfnumber(res, "definition");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = table->n_rules;
		
		// Create new schema struct
		if(table->rule == NULL){
			if((table->rule = (T_Rule **) malloc(sizeof(T_Rule*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((table->rule = (T_Rule **) realloc((table->rule), sizeof(T_Rule*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((table->rule[n] = (T_Rule*) malloc(sizeof(T_Rule))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}

		table->rule[n]->name = copy_string(PQgetvalue(res, row, col1));
		table->rule[n]->comment = copy_string(PQgetvalue(res, row, col2));
		table->rule[n]->OID = atoi(PQgetvalue(res, row, col3));
		table->rule[n]->event = copy_string(PQgetvalue(res, row, col4));
		table->rule[n]->instead = strcmp(PQgetvalue(res,  row, col5), "true") == 0 ? TRUE : FALSE;
		table->rule[n]->condition = copy_string(PQgetvalue(res, row, col6));
		table->rule[n]->definition = copy_string(PQgetvalue(res, row, col7));
		table->rule[n]->parent_name = copy_string(table->name);
		
		// Add rule to global list
		guint m = schema->n_rules;
		
		// Create new schema struct
		if(schema->rule == NULL){
			if((schema->rule = (T_Rule **) malloc(sizeof(T_Rule*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((schema->rule = (T_Rule **) realloc((schema->rule), sizeof(T_Rule*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
		schema->rule[m] = table->rule[n];
		schema->n_rules++;
		
		table->rule[n]->dep_on.dep = NULL;// dependencies
		table->rule[n]->dep_on.n_deps = 0;
		table->rule[n]->ref_by.dep = NULL;
		table->rule[n]->ref_by.n_deps = 0;
			
		// Get dependencies
		if(!get_dep_ons(table->rule[n]->OID, &(table->rule[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(table->rule[n]->OID, &(table->rule[n]->ref_by), conn)) return FALSE;
		
		table->n_rules++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_table_rules ()

/*
 * Get references of table
 */
gboolean
get_table_references(T_Profile * profile, T_Schema * schema, T_Table * table, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT con.conname AS name, "
												"						 con.confkey AS columns, "
												"						 nsp.nspname AS ref_schema, tab.relname AS ref_table, con.conkey AS ref_columns "
												"FROM pg_constraint con "
												"					LEFT OUTER JOIN pg_class tab ON con.conrelid = tab.oid "
												"					LEFT OUTER JOIN pg_namespace nsp ON tab.relnamespace = nsp.oid "
												"					LEFT OUTER JOIN pg_class ref_tab ON con.confrelid = ref_tab.oid "
												"					LEFT OUTER JOIN pg_namespace ref_nsp ON ref_tab.relnamespace = ref_nsp.oid "
												"					LEFT OUTER JOIN pg_description ON con.oid = pg_description.objoid "
												"WHERE con.contype = 'f' AND ref_nsp.nspname = '";
	gchar sql2[]="' AND ref_tab.relname = '";
	gchar sql3[]="' ORDER BY name;";

	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2)+strlen(table->name)+strlen(sql3);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2); strcat(sql, table->name); strcat(sql, sql3);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "columns");
	guint col3 = PQfnumber(res, "ref_schema");
	guint col4 = PQfnumber(res, "ref_table");
	guint col5 = PQfnumber(res, "ref_columns");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = table->n_references;
		
		// Create new table struct
		if(table->reference == NULL){
			if((table->reference = (T_Reference **) malloc(sizeof(T_Reference*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((table->reference = (T_Reference **) realloc((table->reference), sizeof(T_Reference*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((table->reference[n] = (T_Reference*) malloc(sizeof(T_Reference))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		table->reference[n]->name = copy_string(PQgetvalue(res, row, col1));
		
		// Getting columns
		table->reference[n]->n_columns = 0;
		table->reference[n]->column = NULL;
				
		gchar * columns = copy_string(PQgetvalue(res, row, col2));
		
		guint l = strlen(columns);
		gboolean num = FALSE;
		gchar temp[l+1];
		temp[0]='\0';
		gint j=0;
		for(int i=0; i < l; i++){
			if(columns[i] >= '0' && columns[i] <= '9' && num){
				temp[j] = columns[i];
				j++;				
			}
			else if(((columns[i] >= '0' && columns[i] <= '9') || columns[i] == '-') && !num){
				num = TRUE;
				j=0;
				temp[j] = columns[i];
				j++;				
			}
			else if(num){
				temp[j] = '\0'; // end number
				gint number = atoi(temp);
				for(int k = 0; k < table->n_columns; k++){
					if(number == table->column[k]->column_number){
						gint m = table->reference[n]->n_columns;
						
						if(table->reference[n]->column == NULL){
							if((table->reference[n]->column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  						 fprintf(stderr, "Memory allocation error.");
	   						 exit(0);
	   						}
						}
						else{
		 						if((table->reference[n]->column = (T_Column **) realloc((table->reference[n]->column), sizeof(T_Column*)*(m+1))) == NULL){
		    					fprintf(stderr, "Memory allocation error.");
	  	 						exit(0);
	 							}
						}
						table->reference[n]->column[m] = table->column[k];
						table->reference[n]->n_columns++;
						break;
					}
				}
				num=FALSE;
			}
			
		}
		if(num){
			temp[j] = '\0'; // end number
			gint number = atoi(temp);
			for(int k = 0; k < table->n_columns; k++){
				if(number == table->column[k]->column_number){
					gint m = table->reference[n]->n_columns;
					
					if(table->reference[n]->column == NULL){
						if((table->reference[n]->column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  					 fprintf(stderr, "Memory allocation error.");
	   					 exit(0);
	   					}
					}
					else{
		 					if((table->reference[n]->column = (T_Column **) realloc((table->reference[n]->column), sizeof(T_Column*)*(m+1))) == NULL){
		  					fprintf(stderr, "Memory allocation error.");
	  						exit(0);
	 						}
					}
					table->reference[n]->column[m] = table->column[k];
					table->reference[n]->n_columns++;
					break;
				}
			}
		}
		free(columns);
		
		table->reference[n]->ref_schema = copy_string(PQgetvalue(res, row, col3));
		table->reference[n]->ref_table = copy_string(PQgetvalue(res, row, col4));
		
		T_Table * ref_table = NULL;
		
		for(int i = 0; i < profile->database.n_schemas; i++){
			if(strcmp(profile->database.schema[i]->name, table->reference[n]->ref_schema) == 0){
				for(int j = 0; j < profile->database.schema[i]->n_tables; j++){
					if(strcmp(profile->database.schema[i]->table[j]->name, table->reference[n]->ref_table) == 0){
						ref_table =  profile->database.schema[i]->table[j];
						break;
					}
				}
				break;
			}
		}
		
		if(ref_table == NULL) return FALSE;
		
		// Getting referenced columns
		
		table->reference[n]->n_ref_columns = 0;
		table->reference[n]->ref_column = NULL;
				
		gchar * ref_columns = copy_string(PQgetvalue(res, row, col5));
		
		l = strlen(ref_columns);
		gboolean ref_num = FALSE;
		gchar ref_temp[l+1];
		ref_temp[0]='\0';
		j=0;
		for(int i=0; i < l; i++){
			if(ref_columns[i] >= '0' && ref_columns[i] <= '9' && ref_num){
				ref_temp[j] = ref_columns[i];
				j++;				
			}
			else if(((ref_columns[i] >= '0' && ref_columns[i] <= '9') || ref_columns[i] == '-') && !ref_num){
				ref_num = TRUE;
				j=0;
				ref_temp[j] = ref_columns[i];
				j++;				
			}
			else if(ref_num){
				ref_temp[j] = '\0'; // end number
				gint number = atoi(ref_temp);
				for(int k = 0; k < ref_table->n_columns; k++){
					if(number == ref_table->column[k]->column_number){
						gint m = table->reference[n]->n_ref_columns;
						
						if(table->reference[n]->ref_column == NULL){
							if((table->reference[n]->ref_column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  						 fprintf(stderr, "Memory allocation error.");
	   						 exit(0);
	   						}
						}
						else{
		 						if((table->reference[n]->ref_column = (T_Column **) realloc((table->reference[n]->ref_column), sizeof(T_Column*)*(m+1))) == NULL){
		    					fprintf(stderr, "Memory allocation error.");
	  	 						exit(0);
	 							}
						}
						table->reference[n]->ref_column[m] = ref_table->column[k];
						table->reference[n]->n_ref_columns++;
						break;
					}
				}
				ref_num=FALSE;
			}
			
		}
		if(ref_num){
			ref_temp[j] = '\0'; // end number
			gint number = atoi(ref_temp);
			for(int k = 0; k < ref_table->n_columns; k++){
				if(number == ref_table->column[k]->column_number){
					gint m = table->reference[n]->n_ref_columns;
					
					if(table->reference[n]->ref_column == NULL){
						if((table->reference[n]->ref_column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  					 fprintf(stderr, "Memory allocation error.");
	   					 exit(0);
	   					}
					}
					else{
		 					if((table->reference[n]->ref_column = (T_Column **) realloc((table->reference[n]->ref_column), sizeof(T_Column*)*(m+1))) == NULL){
		  					fprintf(stderr, "Memory allocation error.");
	  						exit(0);
	 						}
					}
					table->reference[n]->ref_column[m] = ref_table->column[k];
					table->reference[n]->n_ref_columns++;
					break;
				}
			}
		}
		free(ref_columns);
		
		table->n_references++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_table_references ()

////////////////////////////////// view objects ////////////////////////////////////////////////////////////////////////////////////
/*
 * Get fields in view
 */
gboolean
get_view_fields(T_Schema * schema, T_View * view, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT fields.attname AS name, description AS comment, "
												"						 CASE WHEN cols.udt_name LIKE '\\\\_%' THEN substring(cols.udt_name from '_#\"%#\"' for '#') ELSE cols.udt_name END AS type, "
												"              CASE WHEN cols.udt_name LIKE '\\\\_%' THEN 'true' ELSE 'false' END AS is_array, "
												"              cols.character_maximum_length AS size, cols.numeric_precision AS precision, "
												"						 CASE WHEN cons_p.contype ='p' THEN 'true' ELSE 'false' END AS pkey, CASE WHEN cons_u.contype ='u' THEN 'true' ELSE 'false' END AS unique, "
												"              CASE WHEN attnotnull THEN 'true' ELSE 'false' END AS not_null, "
												"						 fieldef.adsrc AS default_value, fields.attnum AS column_number "
												"FROM pg_attribute fields "
												"					LEFT OUTER JOIN pg_class tab ON tab.oid = fields.attrelid "
												"					LEFT OUTER JOIN pg_attrdef fieldef ON fields.attrelid = fieldef.adrelid AND fields.attnum = fieldef.adnum "
												"					LEFT OUTER JOIN pg_constraint cons_p ON tab.oid = cons_p.conrelid AND fields.attnum = ANY (cons_p.conkey) AND cons_p.contype = 'p' "
												"					LEFT OUTER JOIN pg_constraint cons_u ON tab.oid = cons_u.conrelid AND fields.attnum = ANY (cons_u.conkey) AND cons_u.contype = 'u' "
												"					LEFT OUTER JOIN pg_namespace ON tab.relnamespace = pg_namespace.oid "
												"					LEFT OUTER JOIN information_schema.columns cols ON nspname = cols.table_schema AND tab.relname = cols.table_name AND fields.attname = cols.column_name "
												"					LEFT OUTER JOIN pg_description ON tab.oid = pg_description.objoid AND pg_description.objsubid = fields.attnum "
												"WHERE nspname='";
	gchar sql2[]="' AND tab.relname='";
	gchar sql3[]="' AND fields.attnum > 0 AND attisdropped = FALSE ORDER BY fields.attnum;";

	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2)+strlen(view->name)+strlen(sql3);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2); strcat(sql, view->name); strcat(sql, sql3);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "comment");
	guint col3 = PQfnumber(res, "type");
	guint col4 = PQfnumber(res, "size");
	guint col5 = PQfnumber(res, "precision");
	guint col6 = PQfnumber(res, "pkey");
	guint col7 = PQfnumber(res, "unique");
	guint col8 = PQfnumber(res, "not_null");
	guint col9 = PQfnumber(res, "default_value");
	guint col10 = PQfnumber(res, "column_number");
	guint col11 = PQfnumber(res, "is_array");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = view->n_columns;
		
		// Create new view struct
		if(view->column == NULL){
			if((view->column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((view->column = (T_Column **) realloc((view->column), sizeof(T_Column*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((view->column[n] = (T_Column*) malloc(sizeof(T_Column))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		view->column[n]->name = copy_string(PQgetvalue(res, row, col1));
		view->column[n]->comment = copy_string(PQgetvalue(res, row, col2));
		view->column[n]->type = (strcmp(PQgetvalue(res, row, col11), "true") == 0) ? append_string(copy_string(PQgetvalue(res, row, col3)), "[]") : copy_string(PQgetvalue(res, row, col3));
		view->column[n]->size = copy_string(PQgetvalue(res, row, col4));
		view->column[n]->precision = copy_string(PQgetvalue(res, row, col5));
		view->column[n]->pkey = strcmp(PQgetvalue(res,  row, col6), "true") == 0 ? TRUE : FALSE;
		view->column[n]->unique = strcmp(PQgetvalue(res,  row, col7), "true") == 0 ? TRUE : FALSE;
		view->column[n]->not_null = strcmp(PQgetvalue(res,  row, col8), "true") == 0 ? TRUE : FALSE;
		view->column[n]->default_value = copy_string(PQgetvalue(res, row, col9));
		view->column[n]->column_number = atoi(PQgetvalue(res, row, col10));
		
		view->n_columns++;
 }
 
 PQclear(res);
 return TRUE;
} // get_view_fields ()

/*
 * Get rules in view
 */
gboolean
get_view_rules(T_Schema * schema, T_View * view, PGconn * conn){
 	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT rul.rulename AS name, description AS comment, rew.oid AS oid, "
						           "						 CASE WHEN rew.ev_type='1' THEN 'Select' WHEN rew.ev_type='2' THEN 'Update' WHEN rew.ev_type='3' THEN 'Insert' WHEN rew.ev_type='4' THEN 'Delete' END AS event, "
						           "						 CASE WHEN rew.is_instead THEN 'true' ELSE 'false' END AS instead, CASE WHEN rew.is_instead THEN substring(pg_get_ruledef(rew.oid, true) from '%INSTEAD #\"%#\";' for '#') ELSE substring(pg_get_ruledef(rew.oid, true) from '%DO #\"%#\";' for '#')END AS definition, "
						           "						 substring(pg_get_ruledef(rew.oid, true) from '%WHERE #\"%#\" DO%' for '#') AS condition "
						           "FROM pg_rules rul "
						           "					LEFT OUTER JOIN pg_rewrite rew ON rul.rulename = rew.rulename "
						           "          LEFT OUTER JOIN pg_class rew_tab ON rew.ev_class = rew_tab.oid "
						           "					LEFT OUTER JOIN pg_description ON rew.oid = pg_description.objoid "
						           "WHERE rul.schemaname='";
	gchar sql2[]="' AND rul.tablename = '";
	gchar sql4[]="' AND rew_tab.relname = '";
	gchar sql3[]="' ORDER BY name;";

	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2)+strlen(view->name)+strlen(sql4)+strlen(view->name)+strlen(sql3);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2); strcat(sql, view->name); strcat(sql, sql4); strcat(sql, view->name); strcat(sql, sql3);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "comment");
	guint col3 = PQfnumber(res, "oid");
	guint col4 = PQfnumber(res, "event");
	guint col5 = PQfnumber(res, "instead");
	guint col6 = PQfnumber(res, "condition");
	guint col7 = PQfnumber(res, "definition");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = view->n_rules;
		
		// Create new schema struct
		if(view->rule == NULL){
			if((view->rule = (T_Rule **) malloc(sizeof(T_Rule*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((view->rule = (T_Rule **) realloc((view->rule), sizeof(T_Rule*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((view->rule[n] = (T_Rule*) malloc(sizeof(T_Rule))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		//schema->rule[n]->schema_name = copy_string(schema->name);
		view->rule[n]->name = copy_string(PQgetvalue(res, row, col1));
		view->rule[n]->comment = copy_string(PQgetvalue(res, row, col2));
		view->rule[n]->OID = atoi(PQgetvalue(res, row, col3));
		view->rule[n]->event = copy_string(PQgetvalue(res, row, col4));
		view->rule[n]->instead = strcmp(PQgetvalue(res,  row, col5), "true") == 0 ? TRUE : FALSE;
		view->rule[n]->condition = copy_string(PQgetvalue(res, row, col6));
		view->rule[n]->definition = copy_string(PQgetvalue(res, row, col7));
		view->rule[n]->parent_name = copy_string(view->name);
		
		// Add rule to global list
		guint m = schema->n_rules;
		
		// Create new schema struct
		if(schema->rule == NULL){
			if((schema->rule = (T_Rule **) malloc(sizeof(T_Rule*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((schema->rule = (T_Rule **) realloc((schema->rule), sizeof(T_Rule*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
		schema->rule[m] = view->rule[n];
		schema->n_rules++;
		
		view->rule[n]->dep_on.dep = NULL;// dependencies
		view->rule[n]->dep_on.n_deps = 0;
		view->rule[n]->ref_by.dep = NULL;
		view->rule[n]->ref_by.n_deps = 0;
			
		// Get dependencies
		if(!get_dep_ons(view->rule[n]->OID, &(view->rule[n]->dep_on), conn)) return FALSE;
		if(!get_ref_bys(view->rule[n]->OID, &(view->rule[n]->ref_by), conn)) return FALSE;
		
		view->n_rules++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_view_rules

///////////////////////////////// domain objects ////////////////////////////////////////////////////////////////////////////////
/*
 * Get checks in domain
 */
gboolean
get_domain_checks(T_Schema * schema, T_Domain * domain, PGconn * conn){
	GtkWidget * mess;
	
	// SQL query
	gchar sql1[]="SELECT conname AS name, description AS comment, pg_constraint.oid, "
												"						 consrc AS condition "
												"FROM pg_constraint "
												"					LEFT OUTER JOIN pg_type ON conrelid = pg_type.oid "
												"					LEFT OUTER JOIN pg_namespace ON connamespace = pg_namespace.oid "
												"					LEFT OUTER JOIN pg_description ON pg_constraint.oid = pg_description.objoid "
												"WHERE contype='c' AND nspname = '";
	gchar sql2[]="' AND typname='";
	gchar sql3[]="' ORDER BY name;";

	guint len = strlen(sql1)+strlen(schema->name)+strlen(sql2)+strlen(domain->name)+strlen(sql3);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, schema->name); strcat(sql, sql2); strcat(sql, domain->name); strcat(sql, sql3);
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "comment");
	guint col3 = PQfnumber(res, "oid");
	guint col4 = PQfnumber(res, "condition");
	
	// Get values
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = domain->n_checks;
		
		// Create new domain struct
		if(domain->check == NULL){
			if((domain->check = (T_Check **) malloc(sizeof(T_Check*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((domain->check = (T_Check **) realloc((domain->check), sizeof(T_Check*)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}
	
		if((domain->check[n] = (T_Check*) malloc(sizeof(T_Check))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
		}
		
		domain->check[n]->name = copy_string(PQgetvalue(res, row, col1));
		domain->check[n]->comment = copy_string(PQgetvalue(res, row, col2));
		domain->check[n]->OID = atoi(PQgetvalue(res, row, col3));
		domain->check[n]->condition = copy_string(PQgetvalue(res, row, col4));
				
		domain->n_checks++;
 }
 
 PQclear(res);
 return TRUE; 
} // get_domain_checks ()

//////////////////////////////// function parameters /////////////////////////////////////////////////////////////////////////
/*
 * Get function parameters
 */
gboolean
get_function_parameters(gchar * argtypes, gchar * allargtypes, gchar * argmodes, gchar * argnames, T_Function * function, PGconn * conn){
	// Get arguments names
	function->n_parameters = 0;
	function->parameter = NULL;

	gint l = strlen(argnames);
	gboolean in_name = FALSE;
	gchar temp[l+1];
	temp[0]='\0';
	gint j=0;
	for(int i=0; i < l; i++){
		if(argnames[i] != '{' && argnames[i] != '}' && argnames[i] != ',' && in_name){
			temp[j] = argnames[i];
			j++;				
		}
		else if(argnames[i] != '{' && argnames[i] != '}' && argnames[i] != ',' && !in_name){
			in_name = TRUE;
			j=0;
			temp[j] = argnames[i];
			j++;				
		}
		else if(in_name){
			temp[j] = '\0'; // end name
			gint n = function->n_parameters;
			
			// Add new parameter		
			if(function->parameter == NULL){
				if((function->parameter = (T_Parameter **) malloc(sizeof(T_Parameter*))) == NULL){
	  			 fprintf(stderr, "Memory allocation error.");
	   			 exit(0);
	   		}
			}
			else{
		 		if((function->parameter = (T_Parameter **) realloc((function->parameter), sizeof(T_Parameter*)*(n+1))) == NULL){
		  		fprintf(stderr, "Memory allocation error.");
	  			exit(0);
	 			}
			}
			if((function->parameter[n] = (T_Parameter*) malloc(sizeof(T_Parameter))) == NULL){
		  	fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
			}
			
			function->parameter[n]->name = copy_string(temp);
			function->n_parameters++;
			
			in_name=FALSE;
		}
	}
	if(in_name){
		temp[j] = '\0'; // end name
		gint n = function->n_parameters;
			
		// Add new parameter		
		if(function->parameter == NULL){
			if((function->parameter = (T_Parameter **) malloc(sizeof(T_Parameter*))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	  		 exit(0);
	  	}
		}
		else{
			if((function->parameter = (T_Parameter **) realloc((function->parameter), sizeof(T_Parameter*)*(n+1))) == NULL){
				fprintf(stderr, "Memory allocation error.");
	  		exit(0);
	 		}
		}
		if((function->parameter[n] = (T_Parameter*) malloc(sizeof(T_Parameter))) == NULL){
		 	fprintf(stderr, "Memory allocation error: callbacks.c");
	   	exit(0);
		}
		
		function->parameter[n]->name = copy_string(temp);
		function->n_parameters++;
	}

	// Get arguments types
	gchar * types;
	if(allargtypes == NULL || strcmp(allargtypes, "") == 0){ // All Input
		types = copy_string(argtypes);
	}
	else{
		types = copy_string(allargtypes);
	}

	l = strlen(types);
	in_name = FALSE;
	gchar typ_temp[l+1];
	typ_temp[0]='\0';
	j=0;
	gint n = 0;
	for(int i=0; i < l; i++){
		if(types[i] >= '0' && types[i] <= '9' && in_name){
			typ_temp[j] = types[i];
			j++;				
		}
		else if(((types[i] >= '0' && types[i] <= '9') || types[i] == '-') && !in_name){
			in_name = TRUE;
			j=0;
			typ_temp[j] = types[i];
			j++;				
		}
		else if(in_name){
			typ_temp[j] = '\0'; // end number
			// Get type name
			// SQL query
			gchar sql3[]="SELECT typname AS name FROM pg_type WHERE oid =";
			gchar sql4[]=";";
			guint sub_len = strlen(sql3)+strlen(typ_temp)+strlen(sql4);
			gchar * sub_sql;
			sub_sql = alloc_string(sub_len+1);
			strcpy(sub_sql, sql3); strcat(sub_sql, typ_temp); strcat(sub_sql, sql4);
	
			// Execution
			PGresult *sub_res = PQexec(conn, sub_sql);
			
			free(sub_sql);
			
			if (PQresultStatus(sub_res) != PGRES_TUPLES_OK){
	  		GtkWidget * mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
     		                                                      GTK_BUTTONS_OK,  "Error during retrieving data.");
				gint resp = gtk_dialog_run(GTK_DIALOG(mess));
				if(resp == GTK_RESPONSE_OK);
				gtk_widget_destroy (mess);
				PQclear(sub_res);
				return FALSE;	                                                           
			}
	
			// Determine column nubers
			guint sub_col1 = PQfnumber(sub_res, "name");
			function->parameter[n]->type = copy_string(PQgetvalue(sub_res, 0, sub_col1));
			n++;			
			PQclear(sub_res);				
			in_name=FALSE;
		}
		
	}
	if(in_name){
		typ_temp[j] = '\0'; // end number
		// Get type name
		// SQL query
			gchar sql3[]="SELECT typname AS name FROM pg_type WHERE oid =";
			gchar sql4[]=";";
			guint sub_len = strlen(sql3)+strlen(typ_temp)+strlen(sql4);
			gchar * sub_sql;
			sub_sql = alloc_string(sub_len+1);
			strcpy(sub_sql, sql3); strcat(sub_sql, typ_temp); strcat(sub_sql, sql4);
	
			// Execution
			PGresult *sub_res = PQexec(conn, sub_sql);
			
			free(sub_sql);
			
			if (PQresultStatus(sub_res) != PGRES_TUPLES_OK){
	  		GtkWidget * mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
     		                                                      GTK_BUTTONS_OK,  "Error during retrieving data.");
				gint resp = gtk_dialog_run(GTK_DIALOG(mess));
				if(resp == GTK_RESPONSE_OK);
				gtk_widget_destroy (mess);
				PQclear(sub_res);
				return FALSE;	                                                           
			}
	
			// Determine column nubers
			guint sub_col1 = PQfnumber(sub_res, "name");
			function->parameter[n]->type = copy_string(PQgetvalue(sub_res, 0, sub_col1));
			n++;			
			PQclear(sub_res);				
	}
	free(types);
		
	// Get arguments modes
	if(argmodes == NULL || strcmp(argmodes, "") == 0){ // All Input
		for(int k=0; k < function->n_parameters; k++){
			function->parameter[k]->scope = copy_string("Input");
		}
	}
	else{ // Different types
		l = strlen(argmodes);
		in_name = FALSE;
		gchar mode_temp[l+1];
		mode_temp[0]='\0';
		j=0;
		n = 0;
		for(int i=0; i < l; i++){
			if(argmodes[i] != '{' && argmodes[i] != '}' && argmodes[i] != ',' && in_name){
				mode_temp[j] = argnames[i];
				j++;				
			}
			else if(argmodes[i] != '{' && argmodes[i] != '}' && argmodes[i] != ',' && !in_name){
				in_name = TRUE;
				j=0;
				mode_temp[j] = argmodes[i];
				j++;				
			}
			else if(in_name){
				mode_temp[j] = '\0'; // end name
				if(strcmp(mode_temp, "i") == 0)	function->parameter[n]->scope = copy_string("Input");
				else if(strcmp(mode_temp, "o") == 0) function->parameter[n]->scope = copy_string("Output");
				else if(strcmp(mode_temp, "b") == 0) function->parameter[n]->scope = copy_string("Input/Output");
				else function->parameter[n]->scope = copy_string("Unknown");
				n++;
				in_name=FALSE;
			}
		}
		if(in_name){
			mode_temp[j] = '\0'; // end name
			if(strcmp(mode_temp, "i") == 0)	function->parameter[n]->scope = copy_string("Input");
			else if(strcmp(mode_temp, "o") == 0) function->parameter[n]->scope = copy_string("Output");
			else if(strcmp(mode_temp, "b") == 0) function->parameter[n]->scope = copy_string("Input/Output");
			else function->parameter[n]->scope = copy_string("Unknown");
			n++;
		}
	}
		
	return TRUE;
} // get_function_parameters ()

//////////////////////////////// dependencies ///////////////////////////////////////////////////////////////////////////////////
/*
 * Get dependencies of object with OID
 */
gboolean
get_dep_ons(gint OID, T_Dependencies * deps, PGconn * conn){
	GtkWidget * mess;
	gchar * oid = itoa_n( OID,0);

	// SQL query
	gchar sql1[]="SELECT DISTINCT refobjid AS dep_on_OID, CASE WHEN deptype='n' THEN 'Normal' WHEN deptype='a' THEN 'Auto' WHEN deptype='i' THEN 'Internal' WHEN deptype='p' THEN'Pin' ELSE '' END AS type "
												"FROM pg_depend "	
												"WHERE objid=";
	gchar sql2[]=";";
	guint len = strlen(sql1)+strlen(oid)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, oid); strcat(sql, sql2);

	// Execution
	PGresult *res = PQexec(conn, sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving dependencies.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}

	// Determine column nubers
	guint col1 = PQfnumber(res, "dep_on_OID");
	guint col2 = PQfnumber(res, "type");
	
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = deps->n_deps;

		// Create new schema struct
		if(deps->dep == NULL){
			if((deps->dep = (T_Depend *) malloc(sizeof(T_Depend))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((deps->dep = (T_Depend*) realloc((deps->dep), sizeof(T_Depend)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}

		deps->dep[n].OID = atoi(PQgetvalue(res, row, col1));	
		deps->dep[n].type = copy_string(PQgetvalue(res, row, col2));
		deps->n_deps++;
 }
 
 free(oid);
 PQclear(res);
 return TRUE; 
} // get_dep_ons ()

/*
 * Get references of object with OID
 */
gboolean
get_ref_bys(gint OID, T_Dependencies * refs, PGconn * conn){
	GtkWidget * mess;
	gchar * oid = itoa_n( OID,0);

	// SQL query
	gchar sql1[]="SELECT DISTINCT objid AS ref_by_OID, CASE WHEN deptype='n' THEN 'Normal' WHEN deptype='a' THEN 'Auto' WHEN deptype='i' THEN 'Internal' WHEN deptype='p' THEN'Pin' ELSE '' END AS type "
												"FROM pg_depend "	
												"WHERE refobjid=";
	gchar sql2[]=";";
	guint len = strlen(sql1)+strlen(oid)+strlen(sql2);
	gchar * sql;
	sql = alloc_string(len+1);
	strcpy(sql, sql1); strcat(sql, oid); strcat(sql, sql2);
	
	
	// Execution
	PGresult *res = PQexec(conn, sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving dependencies.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "ref_by_OID");
	guint col2 = PQfnumber(res, "type");
	
	guint row;
	for(row = 0; row < PQntuples(res); row++){
		guint n = refs->n_deps;

		// Create new schema struct
		if(refs->dep == NULL){
			if((refs->dep = (T_Depend *) malloc(sizeof(T_Depend))) == NULL){
	  		 fprintf(stderr, "Memory allocation error.");
	   		 exit(0);
	   		}
		}
		else{
		 		if((refs->dep = (T_Depend*) realloc((refs->dep), sizeof(T_Depend)*(n+1))) == NULL){
		    	fprintf(stderr, "Memory allocation error.");
	  	 		exit(0);
	 			}
		}

		refs->dep[n].OID = atoi(PQgetvalue(res, row, col1));	
		refs->dep[n].type = copy_string(PQgetvalue(res, row, col2));
					
		refs->n_deps++;
 }
 
 free(oid);
 PQclear(res);
 return TRUE; 
} // get_ref_bys ()

//////////////////////////////// other functions //////////////////////////////////////////////////////////////////////////////////
/*
 * Get user roles
 */
T_Roles *
get_roles(T_Profile * profile, gboolean can_create_database){
	GtkWidget * mess;
	T_Roles * roles;
	
	// SQL query
	gchar * sql;
	sql = append_string(NULL, "SELECT rolname AS name FROM pg_roles ");
	if(can_create_database){
		sql = append_string(sql, "WHERE rolcreatedb = TRUE ");
	}
	sql = append_string(sql, "ORDER BY name;");
						           
	// Execution
	PGresult *res = PQexec(profile->conn, sql);
	free(sql);
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return NULL;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	
	guint row;
	gint rows = PQntuples(res);
	// alloc new struct
	if((roles = (T_Roles *) malloc(sizeof(T_Roles))) == NULL){
	    fprintf(stderr, "Memory allocation error.");
	    exit(0);
	}
	roles->n_roles= rows;
	roles->name=NULL;
	if((roles->name = (gchar **) malloc(sizeof(gchar*)*(rows))) == NULL){
	   fprintf(stderr, "Memory allocation error.");
	   exit(0);
	}
		
	for (row = 0; row < rows; row++){
		roles->name[row] = copy_string(PQgetvalue(res, row, col1));
	}
	
	PQclear(res);
	return roles;		
} // get_roles ()

/*
 * Get database tablespaces
 */
T_Tablespaces *
get_Tablespaces(T_Profile * profile){
	GtkWidget * mess;
	T_Tablespaces * tspces;
	
	// SQL query
	gchar sql[]="SELECT spcname AS name "
						           "FROM pg_tablespace "
						           "ORDER BY name;";
						           
	// Execution
	PGresult *res = PQexec(profile->conn, sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return NULL;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	
	guint row;
	gint rows = PQntuples(res);
	// alloc new struct
	if((tspces = (T_Tablespaces *) malloc(sizeof(T_Tablespaces))) == NULL){
	    fprintf(stderr, "Memory allocation error.");
	    exit(0);
	}
	tspces->n_tablespaces= rows;
	tspces->name=NULL;
	if((tspces->name = (gchar **) malloc(sizeof(gchar*)*(rows))) == NULL){
	   fprintf(stderr, "Memory allocation error.");
	   exit(0);
	}
		
	for (row = 0; row < rows; row++){
		tspces->name[row] = copy_string(PQgetvalue(res, row, col1));
	}
	
	PQclear(res);
	return tspces;		
} // get_tablespaces()

/*
 * Get database data types 
 */
T_Data_types *
get_data_types(PGconn * profile_conn){
	GtkWidget * mess;
	T_Data_types * dtypes;
	
	// SQL query
	gchar sql[]="SELECT  CASE WHEN t.typname LIKE '\\\\_%' THEN substring(t.typname from '\\_#\"%#\"' for '#') ELSE t.typname END AS name, elem.typname, CASE WHEN t.typname LIKE '\\\\_%' THEN 'true' ELSE 'false' END AS is_array, "
							        "CASE WHEN elem.typname = 'bit' OR (elem.typname = 'char' AND  t.typname LIKE '\\\\_%') OR elem.typname = 'varchar' OR elem.typname = 'numeric' OR "
							        "																	elem.typname = 'bpchar' OR elem.typname = 'interval' OR elem.typname = 'time' OR elem.typname = 'timetz' OR "
							        "																	elem.typname = 'timestamp' OR elem.typname = 'timestamptz'	THEN 'true' "
							        "												WHEN t.typname = 'bit' OR t.typname = 'char' OR t.typname = 'varchar' OR t.typname = 'numeric' OR "
							        "																	t.typname = 'bpchar' OR t.typname = 'interval' OR t.typname = 'time' OR t.typname = 'timetz' OR "
							        "																	t.typname = 'timestamp' OR t.typname = 'timestamptz'	THEN 'true' "
							        "											 ELSE 'false' END AS varlen "
							        "FROM pg_type t "
							        "	LEFT OUTER JOIN pg_type elem ON t.typelem = elem.oid "
							        "WHERE t.typtype='b' OR t.typtype='d' ORDER BY is_array, name;";
						           
	// Execution
	PGresult *res = PQexec(profile_conn, sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return NULL;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "name");
	guint col2 = PQfnumber(res, "is_array");
	guint col3 = PQfnumber(res, "varlen");
	
	guint row;
	gint rows = PQntuples(res);
	// alloc new struct
	if((dtypes = (T_Data_types *) malloc(sizeof(T_Data_types))) == NULL){
	    fprintf(stderr, "Memory allocation error.");
	    exit(0);
	}
	dtypes->n_data_types= rows;
	dtypes->name=NULL;
	dtypes->varlen=NULL;
	if((dtypes->name = (gchar **) malloc(sizeof(gchar*)*(rows))) == NULL){
	   fprintf(stderr, "Memory allocation error.");
	   exit(0);
	}
	if((dtypes->varlen = (gboolean *) malloc(sizeof(gboolean)*(rows))) == NULL){
	   fprintf(stderr, "Memory allocation error.");
	   exit(0);
	}
	for (row = 0; row < rows; row++){
		gchar * name = copy_string(PQgetvalue(res, row, col1));
		dtypes->name[row] = (strcmp(PQgetvalue(res, row, col2), "true") == 0) ? append_string(name, "[]") : name;
		dtypes->varlen[row] = (strcmp(PQgetvalue(res, row, col3), "true") == 0) ? TRUE : FALSE;
	}
	
	PQclear(res);
	return dtypes;		
} // get_data_types ()

/*
 * Get list of databases
 */
gchar **
get_database_list(PGconn * conn, gboolean can_connect, gboolean first_free, gint * n_rows){
	gchar ** list = NULL;
	GtkWidget * mess;
	
	// SQL query
	gchar *sql = NULL;
	sql = append_string(sql, "SELECT datname FROM pg_database");
	if(can_connect) sql = append_string(sql, " WHERE datallowconn = true");
	sql = append_string(sql, ";");
							           
	// Execution
	PGresult *res = PQexec(conn, sql);
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                                           GTK_BUTTONS_OK,  "Error during retrieving data.");
		gint resp = gtk_dialog_run(GTK_DIALOG(mess));
		if(resp == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);
		PQclear(res);
		return NULL;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "datname");
	
	guint row;
	gint rows = PQntuples(res);
	*n_rows = rows + (first_free? 1 : 0);
	
	
	if((list = (gchar **) malloc(sizeof(gchar*)*(rows+(first_free? 1 : 0)))) == NULL){
	   fprintf(stderr, "Memory allocation error.");
	   exit(0);
	}
	
	if(first_free)	list[0] = copy_string("");
	
	for (row = 0; row < rows; row++){
		list[row+(first_free? 1 : 0)] = copy_string(PQgetvalue(res, row, col1));
	}
	
	PQclear(res);
	return list;	
} // get_database_list ()
