/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */
#include "otree_callbacks.h"
#include "show_otree.h"
#include "../GUI/GUI_functions.h"
#include "../GUI/sensitivity.h"
#include "../functions.h"
#include "../database/get_db_objects.h"
#include "../GUI_db_objects/database.h"
#include "../GUI_db_objects/schema.h"
#include "../GUI_db_objects/language.h"
#include "../GUI_db_objects/cast.h"
#include "../GUI_db_objects/table.h"
#include "../GUI_db_objects/view.h"
#include "../GUI_db_objects/function.h"
#include "../GUI_db_objects/domain.h"
#include "../GUI_db_objects/trigger.h"
#include "../GUI_db_objects/rule.h"
#include "../GUI_db_objects/sequence.h"
#include "../GUI_db_objects/aggregate.h"
#include "../GUI_db_objects/type.h"
#include "../GUI_db_objects/operator.h"
#include "../GUI_db_objects/other_objects.h"
#include "../tools/db_profile_manager.h"
#include "../tools/sql_editor.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Callbacks for object tree                                                                                              */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void
connect_object_tree_signals(GladeXML * source)
{
	GtkWidget * widget;

	// When closing the window
	widget = glade_xml_get_widget(source, "object_tree");
	g_signal_connect((gpointer) widget,"row-activated", (GCallback) view_onRowActivated, NULL);
	g_signal_connect(widget, "popup-menu", (GCallback) view_onPopupMenu, NULL);
  g_signal_connect(widget, "button-press-event", (GCallback) view_onButtonPressed, NULL);
  
  GtkTreeSelection  *selection;
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
  gtk_tree_selection_set_select_function(selection, view_onRowSelected, NULL, NULL);
  
}

gboolean
view_onRowSelected            (GtkTreeSelection *selection,
                                                         GtkTreeModel *model,
                                                         GtkTreePath *path,
                                                         gboolean path_currently_selected,
                                                         gpointer data)
{
	if (!path_currently_selected)
  {
    	gint depth = gtk_tree_path_get_depth(path);
			gint * intpath = gtk_tree_path_get_indices(path);
			
			// basic sensitivity
			if(depth < 3){
			  basic_sensitivity();
			}
			
			// connected profile sensitivity
			if(depth == 3 && intpath[1]==0 && o_tree.server[intpath[0]]->profile[intpath[2]]->connected){
			  connected_profile_sensitivity();
			}
			// disconnected profile sensitivity
			else if (depth == 3 && intpath[1]==0 && !o_tree.server[intpath[0]]->profile[intpath[2]]->connected){
			  disconnected_profile_sensitivity();			
			}
			
			// normal sensitivity
			if(depth > 3 && intpath[1]==0 && o_tree.server[intpath[0]]->profile[intpath[2]]->connected){ 
				normal_sensitivity();
			}
			
			// set info about connected profile and some profile connected sensitivity
			if(depth >= 3 && intpath[1]==0 && o_tree.server[intpath[0]]->profile[intpath[2]]->connected){
			  some_profile_connected();
			  gchar *text = bold_text(o_tree.server[intpath[0]]->profile[intpath[2]]->alias, "#FFFFFF");
			  gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"info_database")), text);
			  current_profile = o_tree.server[intpath[0]]->profile[intpath[2]];
			  free(text);		
			}
			else if(!some_db_connected){
				gchar *text = normal_text_with_color("no database connected", "#FFFFFF");
			  gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"info_database")), text);
			  free(text);		
			}
	}			
	return TRUE;
}

void
view_onRowActivated (GtkTreeView        *treeview,
                                     GtkTreePath        *path,
                                     GtkTreeViewColumn  *col,
                                     gpointer            userdata)
{
  GtkTreeModel *model;
  
	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	switch(depth){
		case 1: // server name
			break;
		case 2:
		case 3:
			switch(intpath[1]){
				case 0: // databases
					if(depth == 3){
						if(!o_tree.server[intpath[0]]->profile[intpath[2]]->connected){
							if(gui_connect_db(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]])){
								refresh_profile_manager();
								refresh_sql_editor();
								show_object_tree();
								gtk_tree_view_expand_to_path (treeview, path);
								GtkTreeSelection * selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
								gtk_tree_selection_select_path(selection, path);
							}
						}
						else{
							create_db_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]], gtk_tree_path_to_string(path));
						}
					}
				break;
				case 1: // roles
				break;
				case 2: // tablespaces
				break;
			}
			break;
		case 4:
		case 5:
			switch(intpath[1]){
				case 0: // databases
					switch(intpath[3]){
						case 0: // schemas
							if(depth == 5){
								create_schema_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]], gtk_tree_path_to_string(path));
							}
							break;
						case 1: // languages
							if(depth == 5){
								create_language_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.language[intpath[4]], gtk_tree_path_to_string(path));
							}
							break;
						case 2: // casts
							if(depth == 5){
								create_cast_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.cast[intpath[4]], gtk_tree_path_to_string(path));
							}
							break;
					}
					break;
				case 1: // roles
				break;
				case 2: // tablespaces
				break;
			}
			break;
	 case 6:
	 case 7:
			switch(intpath[5]){
				case 0: // tables
					if(depth == 7){
						create_table_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]], gtk_tree_path_to_string(path));
					}
					break;
				case 1: // views
					if(depth == 7){
						create_view_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->view[intpath[6]], gtk_tree_path_to_string(path));
					}
				break;
				case 2: // functions
							if(depth == 7){
								create_function_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->function[intpath[6]], gtk_tree_path_to_string(path));
							}
				break;
				case 3: // domains
							if(depth == 7){
								create_domain_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->domain[intpath[6]], gtk_tree_path_to_string(path));
							}
				break;
				case 4: // aggregates
							if(depth == 7){
								create_aggregate_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->aggregate[intpath[6]], gtk_tree_path_to_string(path));
							}				
				break;
				case 5: // sequences
							if(depth == 7){
								create_sequence_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->sequence[intpath[6]], gtk_tree_path_to_string(path));
							}				
				break;
				case 6: // types
							if(depth == 7){
								create_type_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->type[intpath[6]], gtk_tree_path_to_string(path));
							}				
				break;
				case 7: // operators
							if(depth == 7){
								create_operator_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->operator[intpath[6]], gtk_tree_path_to_string(path));
							}				
				break;
				case 8: // triggers
							if(depth == 7){
								create_trigger_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->trigger[intpath[6]], gtk_tree_path_to_string(path));
							}				
				break;
				case 9: // rules
							if(depth == 7){
								create_rule_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->rule[intpath[6]], gtk_tree_path_to_string(path));
							}				
				break;
			}
			break;
	 case 8:
	 case 9:
			switch(intpath[5]){
				case 0: // tables
						switch(intpath[7]){
							case 0: // fields
								if(depth == 9){
									run_field_dialog(o_tree.server[intpath[0]]->profile[intpath[2]], 
									                          o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									                          o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]],
																						o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->column[intpath[8]], 
																						FALSE);
								}
								break;
							case 1: // indexes
								if(depth == 9){
									run_index_dialog(o_tree.server[intpath[0]]->profile[intpath[2]], 
									                          o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									                          o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]],
																						o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->index[intpath[8]], 
																						FALSE);
								}
							break;
							case 2: // foreign keys
								if(depth == 9){
									run_fkey_dialog(o_tree.server[intpath[0]]->profile[intpath[2]], 
									                          o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									                          o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]],
																						o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->fkey[intpath[8]], 
																						FALSE);
								}
							break;
							case 3: // checks
								if(depth == 9){
									run_check_dialog(o_tree.server[intpath[0]]->profile[intpath[2]], 
									                             o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									                             o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]], 
									                             o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->check[intpath[8]], 
									                             FALSE);
								}
							break;
							case 4: // triggers
								if(depth == 9){
									create_trigger_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->trigger[intpath[8]], gtk_tree_path_to_string(path));
								}				
							break;
							case 5: // rules
								if(depth == 9){
									create_rule_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->rule[intpath[8]], gtk_tree_path_to_string(path));
								}	
							break;
							case 6: // references
							break;
						}
					break;
				case 1: // views
					switch(intpath[7]){
							case 0: // fields
								break;
							case 1: // rules
							break;
					}
				break;
				case 2: // functions
				break;
				case 3: // domains
					switch(intpath[7]){
							case 0: // checks
								break;
					}		
				break;
				case 4: // aggregates
				break;
				case 5: // sequences
				break;
				case 6: // types
				break;
				case 7: // operators
				break;
				case 8: // triggers
				break;
				case 9: // rules
				break;
			}
		break;
		case 10:
		case 11:
			switch(intpath[5]){
				case 0: // tables
						switch(intpath[7]){		
							case 2: // foreign keys
							case 6: // references
							break;
						}
				break;
			}							
		break;
  }
  model = gtk_tree_view_get_model(treeview);
}


void
view_popup_menu (GtkWidget *treeview, GdkEventButton *event, gpointer userdata)
{
  GtkWidget *menu = NULL;
  GtkTreeSelection *selection;
  GtkTreeIter iter;
  GtkTreePath *path;
	
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
  
  GtkTreeModel * treemodel = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
  
  if(!gtk_tree_selection_get_selected(selection, NULL, &iter)) return;
	path = gtk_tree_model_get_path(treemodel, &iter);
	
	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);

	switch(depth){
		case 1: // server name
			menu = glade_xml_get_widget(xml,"popup_menu_databases");
			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_edit_db_profile"),FALSE);
			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_rm_db_profile"),FALSE);
			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_conn_db"),FALSE);
			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_disconn_db"),FALSE);			
			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_drop_db"),FALSE);
			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_refresh"),FALSE);			
			break;
		case 2:
		case 3:
			switch(intpath[1]){
				case 0: // databases
				menu = glade_xml_get_widget(xml,"popup_menu_databases");
				if(depth == 2){
					gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_edit_db_profile"),FALSE);
					gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_rm_db_profile"),FALSE);
					gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_conn_db"),FALSE);
					gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_disconn_db"),FALSE);			
					gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_drop_db"),FALSE);
					gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_refresh"),FALSE);
				}
				else{
					if(o_tree.server[intpath[0]]->profile[intpath[2]]->connected){ // connected database
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_edit_db_profile"),TRUE);
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_rm_db_profile"),TRUE);
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_conn_db"),FALSE);
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_disconn_db"),TRUE);			
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_drop_db"),TRUE);
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_refresh"),TRUE);
					}
					else{ // disconnected database
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_edit_db_profile"),TRUE);
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_rm_db_profile"),TRUE);
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_conn_db"),TRUE);
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_disconn_db"),FALSE);			
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_drop_db"),FALSE);
						gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_db_refresh"),FALSE);
					}
				}
				break;
				case 1: // roles
					menu = glade_xml_get_widget(xml,"popup_menu_roles");
				break;
				case 2: // tablespaces
					menu = glade_xml_get_widget(xml,"popup_menu_tablespaces");
				break;
			}
			break;
		case 4:
		case 5:
			switch(intpath[1]){
				case 0: // databases
					switch(intpath[3]){
						case 0: // schemas
						  if(depth == 4){
						  	menu = glade_xml_get_widget(xml,"popup_menu_schemas");
  						  gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_schs_edit"),FALSE);	 
   						  gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_schs_drop"),FALSE);	 
						  }
							else{
							  menu = glade_xml_get_widget(xml,"popup_menu_schema");
  						  gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_sch_edit"),TRUE);	 
   						  gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_sch_drop"),TRUE);	 
							}
							break;
						case 1: // languages
							menu = glade_xml_get_widget(xml,"popup_menu_languages");
							if(depth == 4){
								gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_lang_create"),FALSE);	 
								gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_lang_edit"),FALSE);	 
   							gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_lang_drop"),FALSE);	
   						}
   						else{
   							gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_lang_create"),FALSE);	 
								gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_lang_edit"),TRUE);	 
   							gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_lang_drop"),FALSE);	
   						}
							break;
						case 2: // casts
							menu = glade_xml_get_widget(xml,"popup_menu_casts");
							if(depth == 4){
								gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_cast_create"),FALSE);	 
								gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_cast_edit"),FALSE);	 
   							gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_cast_drop"),FALSE);	
   						}
   						else{
   							gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_cast_create"),FALSE);	 
								gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_cast_edit"),TRUE);	 
   							gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_cast_drop"),FALSE);	
   						}
							break;
					}
					break;
				case 1: // roles
					menu = glade_xml_get_widget(xml,"popup_menu_roles");
				break;
				case 2: // tablespaces
					menu = glade_xml_get_widget(xml,"popup_menu_tablespaces");
				break;
			}
			break;
	 case 6:
	 case 7:
			switch(intpath[5]){
				case 0: // tables
						if(depth == 6){
							menu = glade_xml_get_widget(xml,"popup_menu_tables");
						  gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_tabs_edit"),FALSE);	 
 						  gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_tabs_drop"),FALSE);	 
						}
						else{
						  menu = glade_xml_get_widget(xml,"popup_menu_table");
						  gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_tab_edit"),TRUE);	 
 						  gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_tab_drop"),TRUE);	 
						}
					break;
				case 1: // views
					menu = glade_xml_get_widget(xml,"popup_menu_views");
				  if(depth == 6){
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_view_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_view_edit"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_view_drop"),FALSE);
				  }
				  else{
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_view_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_view_edit"),TRUE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_view_drop"),FALSE);
				  }
				break;
				case 2: // functions
					menu = glade_xml_get_widget(xml,"popup_menu_functions");
				  if(depth == 6){
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_func_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_func_edit"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_func_drop"),FALSE);
				  }
				  else{
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_func_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_func_edit"),TRUE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_func_drop"),FALSE);
				  }					
				break;
				case 3: // domains
					menu = glade_xml_get_widget(xml,"popup_menu_domains");
				  if(depth == 6){
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_domain_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_domain_edit"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_domain_drop"),FALSE);
				  }
				  else{
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_domain_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_domain_edit"),TRUE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_domain_drop"),FALSE);
				  }					
				break;
				case 4: // aggregates
					menu = glade_xml_get_widget(xml,"popup_menu_aggregates");
				  if(depth == 6){
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_aggreg_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_aggreg_edit"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_aggreg_drop"),FALSE);
				  }
				  else{
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_aggreg_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_aggreg_edit"),TRUE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_aggreg_drop"),FALSE);
				  }					
				break;
				case 5: // sequences
					menu = glade_xml_get_widget(xml,"popup_menu_sequences");
				  if(depth == 6){
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_seq_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_seq_edit"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_seq_drop"),FALSE);
				  }
				  else{
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_seq_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_seq_edit"),TRUE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_seq_drop"),FALSE);
				  }					
				break;
				case 6: // types
					menu = glade_xml_get_widget(xml,"popup_menu_types");
				  if(depth == 6){
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_type_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_type_edit"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_type_drop"),FALSE);
				  }
				  else{
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_type_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_type_edit"),TRUE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_type_drop"),FALSE);
				  }					
				break;
				case 7: // operators
					menu = glade_xml_get_widget(xml,"popup_menu_operators");
				  if(depth == 6){
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_oper_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_oper_edit"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_oper_drop"),FALSE);
				  }
				  else{
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_oper_create"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_oper_edit"),TRUE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_oper_drop"),FALSE);
				  }					
				break;
				case 8: // triggers
					menu = glade_xml_get_widget(xml,"popup_menu_triggers");
				  if(depth == 6){
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_trigg_edit"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_trigg_drop"),FALSE);
				  }
				  else{
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_trigg_edit"),TRUE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_trigg_drop"),TRUE);
				  }					
				break;
				case 9: // rules
					menu = glade_xml_get_widget(xml,"popup_menu_rules");
				  if(depth == 6){
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_create"),TRUE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_edit"),FALSE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_drop"),FALSE);
				  }
				  else{
				  	gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_create"),TRUE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_edit"),TRUE);
				    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_drop"),TRUE);
				  }					
				break;
			}
			break;
	 case 8:
	 case 9:
			switch(intpath[5]){
				case 0: // tables
						switch(intpath[7]){
							case 0: // fields
									menu = glade_xml_get_widget(xml,"popup_menu_fields");
									if(depth == 8){
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_add"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_drop"),FALSE);
				  				}
				  				else{
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_add"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_edit"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_drop"),TRUE);
				  				}					
								break;
							case 1: // indexes
									menu = glade_xml_get_widget(xml,"popup_menu_indexes");
									if(depth == 8){
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_index_add"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_index_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_index_drop"),FALSE);
				  				}
				  				else{
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_index_add"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_index_edit"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_index_drop"),TRUE);
				  				}														
							break;
							case 2: // foreign keys
									menu = glade_xml_get_widget(xml,"popup_menu_fkeys");
									if(depth == 8){
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_fkey_add"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_fkey_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_fkey_drop"),FALSE);
				  				}
				  				else{
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_fkey_add"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_fkey_edit"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_fkey_drop"),TRUE);
				  				}									
							break;
							case 3: // checks
								 	menu = glade_xml_get_widget(xml,"popup_menu_checks");
									if(depth == 8){
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_add"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_drop"),FALSE);
				  				}
				  				else{
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_add"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_edit"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_drop"),TRUE);
				  				}																	
							break;
							case 4: // triggers
								menu = glade_xml_get_widget(xml,"popup_menu_triggers");
								if(depth == 8){
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_trigg_create"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_trigg_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_trigg_drop"),FALSE);
				  				}
				  				else{
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_trigg_create"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_trigg_edit"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_trigg_drop"),TRUE);
				  				}							
							break;
							case 5: // rules
								menu = glade_xml_get_widget(xml,"popup_menu_rules");
								if(depth == 8){
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_create"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_drop"),FALSE);
				  				}
				  				else{
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_create"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_edit"),TRUE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_drop"),TRUE);
				  				}															
							break;
							case 6: // references
								menu = glade_xml_get_widget(xml,"popup_menu_fkeys");
								gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_fkey_add"),FALSE);
		    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_fkey_edit"),FALSE);
		    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_fkey_drop"),FALSE);
							break;
						}
					break;
				case 1: // views
					switch(intpath[7]){
							case 0: // fields
									menu = glade_xml_get_widget(xml,"popup_menu_fields");
									if(depth == 8){
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_add"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_drop"),FALSE);
				  				}
				  				else{
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_add"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_drop"),FALSE);
				  				}					
								break;
							case 1: // rules
									menu = glade_xml_get_widget(xml,"popup_menu_rules");
								  if(depth == 8){
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_create"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_drop"),FALSE);
				  				}
				  				else{
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_create"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_rule_drop"),FALSE);
				  				}																				
							break;
					}
				break;
				case 3: // domains
					switch(intpath[7]){
							case 0: // checks
									menu = glade_xml_get_widget(xml,"popup_menu_checks");
									if(depth == 8){
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_add"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_drop"),FALSE);
				  				}
				  				else{
										gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_add"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_edit"),FALSE);
				    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_check_drop"),FALSE);
				  				}																		
								break;
					}		
				break;
			}
		break;
		case 10:
		case 11:
			switch(intpath[5]){
				case 0: // tables
						switch(intpath[7]){		
							case 2: // foreign keys
							case 6: // references
									menu = glade_xml_get_widget(xml,"popup_menu_fields");
									gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_add"),FALSE);
			    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_edit"),FALSE);
			    				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pop_field_drop"),FALSE);
							break;
						}
				break;
			}							
		break;
	} 
	
	
  gtk_widget_show_all(menu);


  gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
                 (event != NULL) ? event->button : 0,
                 gdk_event_get_time((GdkEvent*)event));
}

gboolean
view_onPopupMenu (GtkWidget *treeview, gpointer userdata)
{
    view_popup_menu(treeview, NULL, userdata);

    return TRUE; 
}
  
gboolean
view_onButtonPressed (GtkWidget *treeview, GdkEventButton *event, gpointer userdata)
{
    // check if single click with the right mouse button
    if (event->type == GDK_BUTTON_PRESS  &&  event->button == 3)
    {
      GtkTreeSelection *selection;

      selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));

      if (gtk_tree_selection_count_selected_rows(selection)  <= 1)
      {
         GtkTreePath *path;
         // Get tree path for row that was clicked
         if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(treeview),
                                           (gint) event->x, 
                                           (gint) event->y,
                                           &path, NULL, NULL, NULL))
         {
           gtk_tree_selection_unselect_all(selection);
           gtk_tree_selection_select_path(selection, path);
           gtk_tree_path_free(path);
         }
      }            
		
		// No row is selected, do not show popupmenu
		if(gtk_tree_selection_count_selected_rows(selection)  != 1){
			return FALSE;
		}

    view_popup_menu(treeview, event, userdata);

    return TRUE; 
  }
		
	return FALSE; 
}
