/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */
 
#include "show_otree.h"
#include "../functions.h"
#include "../GUI/sensitivity.h"


void
show_object_tree(void)
{
	GtkTreeStore  * store;
	GtkTreeModel * model;
	GtkTreeView * tree;
	GtkTreeIter      root, parent, child, prechild;
  int n; gchar * text;
	tree = GTK_TREE_VIEW(glade_xml_get_widget(xml,"object_tree"));
	model = gtk_tree_view_get_model (tree);
	
	GtkTreeSelection * selection;
	GtkTreeIter sel_iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(tree);
	if(!gtk_tree_selection_get_selected(selection, &model, &sel_iter)) path = NULL;
	else path = gtk_tree_model_get_path(model, &sel_iter);
	
	T_Exp_rows * exp_rows;
	if((exp_rows = (T_Exp_rows *) malloc(sizeof(T_Exp_rows))) == NULL){
		fprintf(stderr, "Memory allocation error.");
		exit(0);
	}
	exp_rows->list = NULL; 
	if(path!= NULL)exp_rows->selected_path = gtk_tree_path_copy(path);
	else exp_rows->selected_path = NULL;
	
	// map expanded rows
	gtk_tree_view_map_expanded_rows (tree, map_expanded_rows, &exp_rows);

	// clear list
	store= GTK_TREE_STORE(model);
	gtk_tree_store_clear(store);

 	// Show servers
 	for(int i = 0; i < o_tree.n_servers; i++){
		
 		gtk_tree_store_append(store, &root, NULL);
		text = normal_text(o_tree.server[i]->name);
		gtk_tree_store_set(store, &root, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Server.png"), COL_TEXT, text, -1);
		free(text);
	  n = o_tree.server[i]->n_profiles;
	  gtk_tree_store_append(store, &child, &root);
	  if(n != 0){
	  	text = bold_text_with_count("Databases",n);
	  	gtk_tree_store_set(store, &child, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Databases.png"), COL_TEXT, text, -1);
	  }
		else{
		 text = normal_text("Databases");
		 gtk_tree_store_set(store, &child, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Databases.png"), COL_TEXT, text, -1);
		}
		free(text);
		
		parent = root;
	  root = child;
	 
    // Show Profiles                       
		for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
			if(o_tree.server[i]->profile[j]->connected){			// Connected
				text = bold_text(o_tree.server[i]->profile[j]->alias, "#000000");
		  	gtk_tree_store_append(store, &child, &root);
	    	gtk_tree_store_set(store, &child, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Database.png"), COL_TEXT, text, -1);
	    	free(text);
	    	 
	    	T_Database * db = &o_tree.server[i]->profile[j]->database;
	    	
	    	// Schemas
	    	gtk_tree_store_append(store, &prechild, &child);
	    	if(db->n_schemas != 0){
	    		text = bold_text_with_count("Schemas",db->n_schemas);
	  			gtk_tree_store_set(store, &prechild, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Schemas.png"), COL_TEXT, text, -1);
	    	}
	    	else{
      	 text = normal_text("Schemas");
				 gtk_tree_store_set(store, &prechild, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Schemas.png"), COL_TEXT, text, -1);
	    	}
	    	free(text);
	    	for(int i = 0; i < db->n_schemas; i++){
					GtkTreeIter iter1, iter2;
					gtk_tree_store_append(store, &iter1, &prechild);
					text = normal_text(db->schema[i]->name);
				 	gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Schema.png"), COL_TEXT, text, -1);
					free(text);
					
					// Tables
					gtk_tree_store_append(store, &iter2, &iter1);
					if(db->schema[i]->n_tables != 0){					  
					  text = bold_text_with_count("Tables", db->schema[i]->n_tables);
				 	  gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Tables.png"), COL_TEXT, text, -1);
				 	}
				 	else{
  					text = normal_text("Tables");
  				 	gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Tables.png"), COL_TEXT, text, -1);
				 	}  
					free(text);
					for(int j = 0; j < db->schema[i]->n_tables; j++){
						GtkTreeIter iter1, iter3;
						gtk_tree_store_append(store, &iter1, &iter2);
						text = normal_text(db->schema[i]->table[j]->name);
				 		gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Table.png"), COL_TEXT, text, -1);
						free(text);
							// Fields
							gtk_tree_store_append(store, &iter3, &iter1);
							if(db->schema[i]->table[j]->n_columns != 0){					  
					  		text = bold_text_with_count("Fields", db->schema[i]->table[j]->n_columns);
				 	  		gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Fields.png"), COL_TEXT, text, -1);
				 			}
				 			else{
  							text = normal_text("Fields");
  				 			gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Fields.png"), COL_TEXT, text, -1);
				 			}  
							free(text);
							for(int l = 0; l < db->schema[i]->table[j]->n_columns; l++){
								GtkTreeIter iter1;
								gtk_tree_store_append(store, &iter1, &iter3);
								text = normal_text(db->schema[i]->table[j]->column[l]->name);
				 				if(db->schema[i]->table[j]->column[l]->pkey) gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/FieldPK.png"), COL_TEXT, text, -1);
				 				else gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Field.png"), COL_TEXT, text, -1);
								free(text);
							}
							// Indexes
							gtk_tree_store_append(store, &iter3, &iter1);
							if(db->schema[i]->table[j]->n_indexes != 0){					  
					  		text = bold_text_with_count("Indexes", db->schema[i]->table[j]->n_indexes);
				 	  		gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Indexes.png"), COL_TEXT, text, -1);
				 			}
				 			else{
  							text = normal_text("Indexes");
  				 			gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Indexes.png"), COL_TEXT, text, -1);
				 			}  
							free(text);
							for(int l = 0; l < db->schema[i]->table[j]->n_indexes; l++){
								GtkTreeIter iter1;
								gtk_tree_store_append(store, &iter1, &iter3);
								text = normal_text(db->schema[i]->table[j]->index[l]->name);
				 				gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Index.png"), COL_TEXT, text, -1);
								free(text);
							}
							// Foreign Keys
							gtk_tree_store_append(store, &iter3, &iter1);
							if(db->schema[i]->table[j]->n_fkeys != 0){					  
					  		text = bold_text_with_count("Foreign keys", db->schema[i]->table[j]->n_fkeys);
				 	  		gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/ForeignKeys.png"), COL_TEXT, text, -1);
				 			}
				 			else{
  							text = normal_text("Foreign keys");
  				 			gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/ForeignKeys.png"), COL_TEXT, text, -1);
				 			}  
							free(text);
							for(int l = 0; l < db->schema[i]->table[j]->n_fkeys; l++){
								GtkTreeIter iter1, iter4, iter5;
								gtk_tree_store_append(store, &iter1, &iter3);
								gint len = strlen(db->schema[i]->table[j]->fkey[l]->name) + strlen(" (") + strlen(db->schema[i]->table[j]->fkey[l]->ref_schema) + strlen(".") + strlen(db->schema[i]->table[j]->fkey[l]->ref_table) + strlen(")");
								gchar * t = alloc_string(len+1);
								strcpy(t, db->schema[i]->table[j]->fkey[l]->name); strcat(t," ("); strcat(t, db->schema[i]->table[j]->fkey[l]->ref_schema); strcat(t,"."); strcat(t, db->schema[i]->table[j]->fkey[l]->ref_table); strcat(t,")");
								text = normal_text(t);
								free(t);
				 				gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/ForeignKey.png"), COL_TEXT, text, -1);
								free(text);
								gtk_tree_store_append(store, &iter4, &iter1);
								text = bold_text_with_count("Fields", db->schema[i]->table[j]->fkey[l]->n_columns);
				 	  		gtk_tree_store_set(store, &iter4, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Fields.png"), COL_TEXT, text, -1);
				 	  		free(text);
				 	  		for(int k = 0; k < db->schema[i]->table[j]->fkey[l]->n_columns; k++){
				 	  			gtk_tree_store_append(store, &iter5, &iter4);
				 	  			len = strlen(db->schema[i]->table[j]->fkey[l]->column[k]->name) + strlen(" <-> ") + strlen(db->schema[i]->table[j]->fkey[l]->ref_schema) + strlen(".") + strlen(db->schema[i]->table[j]->fkey[l]->ref_table) + strlen(".") + strlen(db->schema[i]->table[j]->fkey[l]->ref_column[k]->name);
									t = alloc_string(len+1);
									strcpy(t, db->schema[i]->table[j]->fkey[l]->column[k]->name); strcat(t," <-> "); strcat(t, db->schema[i]->table[j]->fkey[l]->ref_schema); strcat(t,"."); strcat(t, db->schema[i]->table[j]->fkey[l]->ref_table); strcat(t,"."); strcat(t, db->schema[i]->table[j]->fkey[l]->ref_column[k]->name);
									text = normal_text(t);
									free(t);
									gtk_tree_store_set(store, &iter5, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Field.png"), COL_TEXT, text, -1);
									free(text);
								}
							}				
							// Checks
							gtk_tree_store_append(store, &iter3, &iter1);
							if(db->schema[i]->table[j]->n_checks != 0){					  
					  		text = bold_text_with_count("Checks", db->schema[i]->table[j]->n_checks);
				 	  		gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Checks.png"), COL_TEXT, text, -1);
				 			}
				 			else{
  							text = normal_text("Checks");
  				 			gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Checks.png"), COL_TEXT, text, -1);
				 			}  
							free(text);
							for(int l = 0; l < db->schema[i]->table[j]->n_checks; l++){
								GtkTreeIter iter1;
								gtk_tree_store_append(store, &iter1, &iter3);
								text = normal_text(db->schema[i]->table[j]->check[l]->name);
				 				gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Check.png"), COL_TEXT, text, -1);
								free(text);
							}
							// Triggers
							gtk_tree_store_append(store, &iter3, &iter1);
							if(db->schema[i]->table[j]->n_triggers != 0){					  
					  		text = bold_text_with_count("Triggers", db->schema[i]->table[j]->n_triggers);
				 	  		gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Triggers.png"), COL_TEXT, text, -1);
				 			}
				 			else{
  							text = normal_text("Triggers");
  				 			gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Triggers.png"), COL_TEXT, text, -1);
				 			}  
							free(text);
							for(int l = 0; l < db->schema[i]->table[j]->n_triggers; l++){
								GtkTreeIter iter1;
								gtk_tree_store_append(store, &iter1, &iter3);
								text = normal_text(db->schema[i]->table[j]->trigger[l]->name);
				 				gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Trigger.png"), COL_TEXT, text, -1);
								free(text);
							}
							// Rules
							gtk_tree_store_append(store, &iter3, &iter1);
							if(db->schema[i]->table[j]->n_rules != 0){					  
					  		text = bold_text_with_count("Rules", db->schema[i]->table[j]->n_rules);
				 	  		gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Rules.png"), COL_TEXT, text, -1);
				 			}
				 			else{
  							text = normal_text("Rules");
  				 			gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Rules.png"), COL_TEXT, text, -1);
				 			}
							free(text);
							for(int l = 0; l < db->schema[i]->table[j]->n_rules; l++){
								GtkTreeIter iter1;
								gtk_tree_store_append(store, &iter1, &iter3);
								text = normal_text(db->schema[i]->table[j]->rule[l]->name);
				 				gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Rule.png"), COL_TEXT, text, -1);
								free(text);
							}
							// References
							gtk_tree_store_append(store, &iter3, &iter1);
							if(db->schema[i]->table[j]->n_references != 0){					  
					  		text = bold_text_with_count("References", db->schema[i]->table[j]->n_references);
				 	  		gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/ForeignKeys.png"), COL_TEXT, text, -1);
				 			}
				 			else{
  							text = normal_text("References");
  				 			gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/ForeignKeys.png"), COL_TEXT, text, -1);
				 			}  
							free(text);
							for(int l = 0; l < db->schema[i]->table[j]->n_references; l++){
								GtkTreeIter iter1, iter4, iter5;
								gtk_tree_store_append(store, &iter1, &iter3);
								gint len = strlen(db->schema[i]->table[j]->reference[l]->name) + strlen(" (") + strlen(db->schema[i]->table[j]->reference[l]->ref_schema) + strlen(".") + strlen(db->schema[i]->table[j]->reference[l]->ref_table) + strlen(")");
								gchar * t = alloc_string(len+1);
								strcpy(t, db->schema[i]->table[j]->reference[l]->name); strcat(t," ("); strcat(t, db->schema[i]->table[j]->reference[l]->ref_schema); strcat(t,"."); strcat(t, db->schema[i]->table[j]->reference[l]->ref_table); strcat(t,")");
								text = normal_text(t);
								free(t);
				 				gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/ForeignKey.png"), COL_TEXT, text, -1);
								free(text);
								gtk_tree_store_append(store, &iter4, &iter1);
								text = bold_text_with_count("Fields", db->schema[i]->table[j]->reference[l]->n_columns);
				 	  		gtk_tree_store_set(store, &iter4, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Fields.png"), COL_TEXT, text, -1);
				 	  		free(text);
				 	  		for(int k = 0; k < db->schema[i]->table[j]->reference[l]->n_columns; k++){
				 	  			gtk_tree_store_append(store, &iter5, &iter4);
				 	  			len = strlen(db->schema[i]->table[j]->reference[l]->column[k]->name) + strlen(" <-> ") + strlen(db->schema[i]->table[j]->reference[l]->ref_schema) + strlen(".") + strlen(db->schema[i]->table[j]->reference[l]->ref_table) + strlen(".") + strlen(db->schema[i]->table[j]->reference[l]->ref_column[k]->name);
									t = alloc_string(len+1);
									strcpy(t, db->schema[i]->table[j]->reference[l]->column[k]->name); strcat(t," <-> "); strcat(t, db->schema[i]->table[j]->reference[l]->ref_schema); strcat(t,"."); strcat(t, db->schema[i]->table[j]->reference[l]->ref_table); strcat(t,"."); strcat(t, db->schema[i]->table[j]->reference[l]->ref_column[k]->name);
									text = normal_text(t);
									free(t);
									gtk_tree_store_set(store, &iter5, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Field.png"), COL_TEXT, text, -1);
									free(text);
								}
							}							
					}
					
					// Views
					gtk_tree_store_append(store, &iter2, &iter1);
					if(db->schema[i]->n_views != 0){					  
					  text = bold_text_with_count("Views", db->schema[i]->n_views);
				 	  gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Views.png"), COL_TEXT, text, -1);
				 	}
				 	else{
  					text = normal_text("Views");
  				 	gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Views.png"), COL_TEXT, text, -1);
				 	}  
					free(text);
					for(int j = 0; j < db->schema[i]->n_views; j++){
						GtkTreeIter iter1, iter3;
						gtk_tree_store_append(store, &iter1, &iter2);
						text = normal_text(db->schema[i]->view[j]->name);
				 		gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/View.png"), COL_TEXT, text, -1);
						free(text);
							// Fields
							gtk_tree_store_append(store, &iter3, &iter1);
							if(db->schema[i]->view[j]->n_columns != 0){					  
					  		text = bold_text_with_count("Fields", db->schema[i]->view[j]->n_columns);
				 	  		gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Fields.png"), COL_TEXT, text, -1);
				 			}
				 			else{
  							text = normal_text("Fields");
  				 			gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Fields.png"), COL_TEXT, text, -1);
				 			}  
							free(text);
							for(int l = 0; l < db->schema[i]->view[j]->n_columns; l++){
								GtkTreeIter iter1;
								gtk_tree_store_append(store, &iter1, &iter3);
								text = normal_text(db->schema[i]->view[j]->column[l]->name);
				 				if(db->schema[i]->view[j]->column[l]->pkey) gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/FieldPK.png"), COL_TEXT, text, -1);
				 				else gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Field.png"), COL_TEXT, text, -1);
								free(text);
							}
							// Rules
							gtk_tree_store_append(store, &iter3, &iter1);
							if(db->schema[i]->view[j]->n_rules != 0){					  
					  		text = bold_text_with_count("Rules", db->schema[i]->view[j]->n_rules);
				 	  		gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Rules.png"), COL_TEXT, text, -1);
				 			}
				 			else{
  							text = normal_text("Rules");
  				 			gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Rules.png"), COL_TEXT, text, -1);
				 			}
							free(text);
							for(int l = 0; l < db->schema[i]->view[j]->n_rules; l++){
								GtkTreeIter iter1;
								gtk_tree_store_append(store, &iter1, &iter3);
								text = normal_text(db->schema[i]->view[j]->rule[l]->name);
				 				gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Rule.png"), COL_TEXT, text, -1);
								free(text);
							}
					}
					// Functions
					gtk_tree_store_append(store, &iter2, &iter1);
					if(db->schema[i]->n_functions != 0){					  
					  text = bold_text_with_count("Functions", db->schema[i]->n_functions);
				 	  gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Functions.png"), COL_TEXT, text, -1);
				 	}
				 	else{
  					text = normal_text("Functions");
  				 	gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Functions.png"), COL_TEXT, text, -1);
				 	}  
					free(text);
					for(int j = 0; j < db->schema[i]->n_functions; j++){
						GtkTreeIter iter1;
						gtk_tree_store_append(store, &iter1, &iter2);
						gint len= strlen(db->schema[i]->function[j]->name) + strlen("(") + strlen(db->schema[i]->function[j]->parameters) + strlen(")");
						gchar t[len+1]; t[0]='\0';
						strcat(t, db->schema[i]->function[j]->name);strcat(t, "(");strcat(t, db->schema[i]->function[j]->parameters);strcat(t, ")");
						text = normal_text(t);
						gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Function.png"), COL_TEXT, text, -1);
						free(text);
					}
					// Domains
					gtk_tree_store_append(store, &iter2, &iter1);
					if(db->schema[i]->n_domains != 0){					  
					  text = bold_text_with_count("Domains", db->schema[i]->n_domains);
				 	  gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Domains.png"), COL_TEXT, text, -1);
				 	}
				 	else{
  					text = normal_text("Domains");
  				 	gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Domains.png"), COL_TEXT, text, -1);
				 	}  
					free(text);
					for(int j = 0; j < db->schema[i]->n_domains; j++){
						GtkTreeIter iter1, iter3;
						gtk_tree_store_append(store, &iter1, &iter2);
						text = normal_text(db->schema[i]->domain[j]->name);
				 		gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Domain.png"), COL_TEXT, text, -1);
						free(text);
							// Checks
							gtk_tree_store_append(store, &iter3, &iter1);
							if(db->schema[i]->domain[j]->n_checks != 0){					  
					  		text = bold_text_with_count("Checks", db->schema[i]->domain[j]->n_checks);
				 	  		gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Checks.png"), COL_TEXT, text, -1);
				 			}
				 			else{
  							text = normal_text("Checks");
  				 			gtk_tree_store_set(store, &iter3, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Checks.png"), COL_TEXT, text, -1);
				 			}  
							free(text);
							for(int l = 0; l < db->schema[i]->domain[j]->n_checks; l++){
								GtkTreeIter iter1;
								gtk_tree_store_append(store, &iter1, &iter3);
								text = normal_text(db->schema[i]->domain[j]->check[l]->name);
				 				gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Check.png"), COL_TEXT, text, -1);
				 				free(text);
							}
					}
					// Aggregates
					gtk_tree_store_append(store, &iter2, &iter1);
					if(db->schema[i]->n_aggregates != 0){					  
					  text = bold_text_with_count("Aggregates", db->schema[i]->n_aggregates);
				 	  gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Aggregates.png"), COL_TEXT, text, -1);
				 	}
				 	else{
  					text = normal_text("Aggregates");
  				 	gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Aggregates.png"), COL_TEXT, text, -1);
				 	}  
					free(text);
					for(int j = 0; j < db->schema[i]->n_aggregates; j++){
						GtkTreeIter iter1;
						gtk_tree_store_append(store, &iter1, &iter2);
						gint len= strlen(db->schema[i]->aggregate[j]->name) + strlen(" ") + strlen(db->schema[i]->aggregate[j]->base_type);
						gchar t[len+1]; t[0]='\0';
						strcat(t, db->schema[i]->aggregate[j]->name);strcat(t, " ");strcat(t, db->schema[i]->aggregate[j]->base_type);
						text = normal_text(t);
				 		gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Aggregate.png"), COL_TEXT, text, -1);
						free(text);
					}
					// Sequences
					gtk_tree_store_append(store, &iter2, &iter1);
					if(db->schema[i]->n_sequences != 0){					  
					  text = bold_text_with_count("Sequences", db->schema[i]->n_sequences);
				 	  gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Sequences.png"), COL_TEXT, text, -1);
				 	}
				 	else{
  					text = normal_text("Sequences");
  				 	gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Sequences.png"), COL_TEXT, text, -1);
				 	}  
					free(text);
					for(int j = 0; j < db->schema[i]->n_sequences; j++){
						GtkTreeIter iter1;
						gtk_tree_store_append(store, &iter1, &iter2);
						text = normal_text(db->schema[i]->sequence[j]->name);
				 		gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Sequence.png"), COL_TEXT, text, -1);
						free(text);
					}					
					// Types
					gtk_tree_store_append(store, &iter2, &iter1);
					if(db->schema[i]->n_types != 0){					  
					  text = bold_text_with_count("Types", db->schema[i]->n_types);
				 	  gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Types.png"), COL_TEXT, text, -1);
				 	}
				 	else{
  					text = normal_text("Types");
  				 	gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Types.png"), COL_TEXT, text, -1);
				 	}  
					free(text);
					for(int j = 0; j < db->schema[i]->n_types; j++){
						GtkTreeIter iter1;
						gtk_tree_store_append(store, &iter1, &iter2);
						text = normal_text(db->schema[i]->type[j]->name);
				 		gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Type.png"), COL_TEXT, text, -1);
						free(text);
					}					
					// Operators
					gtk_tree_store_append(store, &iter2, &iter1);
					if(db->schema[i]->n_operators != 0){					  
					  text = bold_text_with_count("Operators", db->schema[i]->n_operators);
				 	  gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Operators.png"), COL_TEXT, text, -1);
				 	}
				 	else{
  					text = normal_text("Operators");
  				 	gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Operators.png"), COL_TEXT, text, -1);
				 	}  
					free(text);
					for(int j = 0; j < db->schema[i]->n_operators; j++){
						GtkTreeIter iter1;
						gtk_tree_store_append(store, &iter1, &iter2);
						text = normal_text(db->schema[i]->operator[j]->name);
				 		gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Operator.png"), COL_TEXT, text, -1);
						free(text);
					}
					// Triggers
					gtk_tree_store_append(store, &iter2, &iter1);
					if(db->schema[i]->n_triggers != 0){					  
					  text = bold_text_with_count("Triggers", db->schema[i]->n_triggers);
				 	  gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Triggers.png"), COL_TEXT, text, -1);
				 	}
				 	else{
  					text = normal_text("Triggers");
  				 	gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Triggers.png"), COL_TEXT, text, -1);
				 	}  
					free(text);
					for(int j = 0; j < db->schema[i]->n_triggers; j++){
						GtkTreeIter iter1;
						gtk_tree_store_append(store, &iter1, &iter2);
						gint len= strlen(db->schema[i]->trigger[j]->table_name) + strlen(".") + strlen(db->schema[i]->trigger[j]->name);
						gchar t[len+1]; t[0]='\0';
						strcat(t, db->schema[i]->trigger[j]->table_name);strcat(t, ".");strcat(t, db->schema[i]->trigger[j]->name);
						text = normal_text(t);
						gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Trigger.png"), COL_TEXT, text, -1);
						free(text);
					}
					// Rules
					gtk_tree_store_append(store, &iter2, &iter1);
					if(db->schema[i]->n_rules != 0){					  
					  text = bold_text_with_count("Rules", db->schema[i]->n_rules);
				 	  gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Rules.png"), COL_TEXT, text, -1);
				 	}
				 	else{
  					text = normal_text("Rules");
  				 	gtk_tree_store_set(store, &iter2, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Rules.png"), COL_TEXT, text, -1);
				 	}  
					free(text);
					for(int j = 0; j < db->schema[i]->n_rules; j++){
						GtkTreeIter iter1;
						gtk_tree_store_append(store, &iter1, &iter2);
						gint len= strlen(db->schema[i]->rule[j]->parent_name) + strlen(".") + strlen(db->schema[i]->rule[j]->name);
						gchar t[len+1]; t[0]='\0';
						strcat(t, db->schema[i]->rule[j]->parent_name);strcat(t, ".");strcat(t, db->schema[i]->rule[j]->name);
						text = normal_text(t);
				 		gtk_tree_store_set(store, &iter1, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Rule.png"), COL_TEXT, text, -1);
						free(text);
					}
				}

	    	// Languages
	    	gtk_tree_store_append(store, &prechild, &child);
	    	if(db->n_languages != 0){
	    		text = bold_text_with_count("Languages",db->n_languages);
	  			gtk_tree_store_set(store, &prechild, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Languages.png"), COL_TEXT, text, -1);
	    	}
	    	else{
	    	 text = normal_text("Languages");
				 gtk_tree_store_set(store, &prechild, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Languages.png"), COL_TEXT, text, -1);
	    	}
	    	free(text);
	    	for(int i = 0; i < db->n_languages; i++){
					GtkTreeIter iter;
					gtk_tree_store_append(store, &iter, &prechild);
					text = normal_text(db->language[i]->name);
				 	gtk_tree_store_set(store, &iter, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Language.png"), COL_TEXT, text, -1);
					free(text);
				}

	    	// Casts
	    	gtk_tree_store_append(store, &prechild, &child);
	    	if(db->n_casts != 0){
	    		text = bold_text_with_count("Casts",db->n_casts);
	  			gtk_tree_store_set(store, &prechild, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Casts.png"), COL_TEXT, text, -1);
	    	}
	    	else{
	    	 text = normal_text("Casts");
				 gtk_tree_store_set(store, &prechild, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Casts.png"), COL_TEXT, text, -1);
	    	}
	    	free(text);
	    	for(int i = 0; i < db->n_casts; i++){
					GtkTreeIter iter;
					gtk_tree_store_append(store, &iter, &prechild);
					text = normal_text(db->cast[i]->name);
				 	gtk_tree_store_set(store, &iter, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Cast.png"), COL_TEXT, text, -1);
					free(text);
				}

	    	// Database varialbes
				
	    }
	    else{			//  Not connected
	    	text = normal_text(o_tree.server[i]->profile[j]->alias);
		  	gtk_tree_store_append(store, &child, &root);
	    	gtk_tree_store_set(store, &child, COL_ICON, get_icon(PIXMAP_DIRECTORY "/Database_not_conn.png"), COL_TEXT, text, -1);
	    	free(text);
				gtk_tree_view_expand_to_path (tree, gtk_tree_path_new_from_string("0:0"));
	    }
	   
  	}
  }
	// restore expansion
 	restore_expanded_rows(tree, exp_rows);
 	
 	// free memory
	GList * list = exp_rows->list;
	for(list = g_list_first(list); list != NULL; list = list->next){
		free((gchar *)list->data);
	}
	if(exp_rows->selected_path != NULL) gtk_tree_path_free(exp_rows->selected_path);
	if(exp_rows != NULL) free(exp_rows);
}

void
map_expanded_rows(GtkTreeView *tree_view, GtkTreePath *path, gpointer user_data){
	T_Exp_rows ** rows = (T_Exp_rows **) user_data;
	
	(*rows)->list = g_list_append ((*rows)->list, gtk_tree_path_to_string(path));
}

void
restore_expanded_rows(GtkTreeView * tree, T_Exp_rows * rows){
	GtkTreeModel * model = gtk_tree_view_get_model(tree);

	GList * list = rows->list;
	for(list = g_list_first(list); list != NULL; list = list->next){
		 GtkTreePath * path = gtk_tree_path_new_from_string((gchar *)list->data);
		 GtkTreeRowReference * ref = gtk_tree_row_reference_new (model, path);
		 if(gtk_tree_row_reference_valid(ref)){
		 	gtk_tree_view_expand_to_path(tree, path);
		 	gtk_tree_path_free(path);
		 }
		 gtk_tree_row_reference_free(ref);
	}
	// restore selection
	if(rows->selected_path != NULL){
		GtkTreeSelection * selection;
		selection = gtk_tree_view_get_selection(tree);
		gint depth = gtk_tree_path_get_depth(rows->selected_path);
		gint * intpath = gtk_tree_path_get_indices(rows->selected_path);
		for(int i = depth-1; i >=0; i--){
			//make path to specified depth
			gchar * p = NULL;
			for(int j = 0; j <= i; j++){
				gchar * num = itoa_n(intpath[j],0);
				p = append_string(p, num);
				free(num);
				if(j < i) p = append_string(p, ":");
			}
			// check if valid
			GtkTreePath * selected_path = gtk_tree_path_new_from_string(p);
			free(p);
			GtkTreeRowReference * ref = gtk_tree_row_reference_new (model, selected_path);
			if(gtk_tree_row_reference_valid(ref)){
		 		gtk_tree_selection_select_path(selection, selected_path);
		 		return;
			}
			gtk_tree_row_reference_free(ref);
		}
		basic_sensitivity();
	}
}
